#!/usr/bin/env python
import os,sys,string
import MFselect_lib_2
import residues
import mylib
import keyfile
import copy
from MFselect_constants import *

doc= """
**************************************************
EtaxyR2ratio.py v.2
EKN 14.3

The program to select non-exchanging residues
by the criterion of Etaxy/R2 ratios
(don't care about overlaps here because you are
supposed later to select non-exchangers from datasets
which already have overlaps removed by 'Replace_resolved.py'

1. You need to prepare the Etaxy/R2 dataset by
    etaxyR2ratio_list Perl-script
    (it outputs the ratios for peaks with more than
    two datapoints in the Etaxy fitting curve)
    
    -> "for_etaxyR2ratio.dat"

2. EtaxyR2ratio2.py  for_etaxyR2ratio.dat    outfname
        (outfname.exch, outfname.nonexch and outfname.xmgr
        are created)


    - the dataset is trimmed 10% and its mean and rmsd are calculated
        (trimmed values, which are below mean are considered to be
        exchanging or artifacts)

    - peaks whose ratio values are BELOW "mean minus rmsd" line
        are selected as NON-exchanging

    - .excluded and .nonexch lists are written to files and
        XMGR file is created to visually control the selection


**************************************************
"""

if len(sys.argv)<3 :
    print doc
    sys.exit()


ratios_fname=sys.argv[1]
outfname=sys.argv[2]

SILENT,VERBOSE=0,1
ratios=residues.Annn_Num(ratios_fname,VERBOSE,1)

full_list=copy.deepcopy(ratios)

# columns
RATIO,DELTA=0,1

mean,sigma=ratios.statistics(RATIO)

#----------- find delta's ----------------
for res in ratios.get_keys():
    ratios.append_word(res,abs(mean-ratios[res][RATIO]))

#----------- trim 10% (big Rex terms and errors) -----------------
NumRes2Trim=len(ratios)/10

print "*"*60
print 

print "\nTotal: %d, trim %d residues\n" % (len(ratios), NumRes2Trim)
print "Mean: %.4f,  Sigma   %.4f" % (mean,sigma)
removed_res=[]
print "\n--------- Trim 10% ------------"
print "\nRemove ",
for i in range(NumRes2Trim):
    res=ratios.max(DELTA)
    removed_res.append(res)
    print  res,
    del ratios[res]

trimmed=copy.deepcopy(ratios)
ratios.set_name("Non-exchanging residues")

#--- remove all  below mean by more than 1 sigma now ---
print "\n\n--- remove all  below mean by more than 1 sigma now  ------------\n"
mean,sigma=ratios.statistics(RATIO)

print "Mean: %.4f,  Sigma   %.4f\n" % (mean,sigma)
print "Remove now: ",
res_list=ratios.sorted_keys()

#------------ remove outsiders -----------
for res in res_list:
    if mean-ratios[res][RATIO] > sigma:
        print res,
        del ratios[res]
        removed_res.append(res)


print "\n\n---------------------------------"
print "\n%d removed residues (file: %s.excluded):\n" % (len(removed_res), outfname)
print removed_res

print "\n%s non-exchanging residues  (file: %s.nonexch):\n" % (len(ratios), outfname)
print ratios.sorted_keys()


#----------- make output files ----------------

open(outfname+".excluded",'w').write(string.join(removed_res, '\n'))
open(outfname+".nonexch",'w').write(string.join(ratios.sorted_keys(), '\n'))

#----------- check Etaxy/R2 graph ---------------
import xmgr


nonex_data=xmgr.Dataset(ratios.to_xmgr("ResNum",RATIO), "non-exchanging")
nonex_data.diamond()
nonex_data.red_sym()
nonex_data.symbol_size(2)


full_data=xmgr.Dataset(full_list.to_xmgr("ResNum",RATIO), "all")
full_data.circle()
full_data.empty_sym()
full_data.symbol_size(2.2)


trimmed_data=xmgr.Dataset(trimmed.to_xmgr("ResNum",RATIO), "trimmed")
trimmed_data.circle()
trimmed_data.filled_sym()
trimmed_data.symbol_size(1)



graph=xmgr.Graph("EtaXY/R2 ratios analysis", outfname+"  (EtaxyR2ratio2.py)")
graph.add_dataset(nonex_data)
graph.add_dataset(full_data)
graph.add_dataset(trimmed_data)
graph.set_xaxis("Residue number", 0, 125)
graph.set_yaxis("ratio", 0.3, 1)
graph.writeout(outfname+".xmgr")


                      
