#!/usr/bin/env python2

doc="""------- Extract column from Sparky LT list -------
by Evgenii Kovrigin    04/13/04

Usage: LT_list_extract_column column_number fname

    - column number: 1,2,3,4,...

"""
import mylib, residues, string, sys


UserWarning="UserWarning"  # for compatibility with Python 1.5
YES,NO=1,0   # conditional flags
SILENT,VERBOSE=0,1  # work mode

KEY_COLUMN=0        # number of word in line which is a key

#--- DEFINE SPECIALIZED CLASS  ----
class Sparky_LT_Peaklist(residues.Sparky_lt_Peaklist):

    """deals with Sparky 'LT' peak lists, use double whitespace to recognize different column names
        - columns are fixed here: you cannot add or delete words"""

    #-------------------------------------------------------------------------
    def __init__(self, fname, mode=SILENT,print_separator="\t"):
        
        #-- create an empty instance first ---------
        residues.Annn_Num.__init__(self,"", mode, 0, print_separator)

        file=mylib.open_file(fname,'r')
        header=file.readline() #--- skip first line ----
        header=file.readline()
        #-- extract column names ----
        self.column_names=filter(None,map(string.strip, filter(None,string.split(header, '  '))))
        #-- check if 'Assignment' is there
        if self.column_names[0]!='Assignment':
            raise UserWarning, ("\a\a\a\n\nThe file '%s' must contain 'Assignment'\n in the first \
column! '%s' - found instead..." % (fname, self.column_names[0]))

        #--- load the file ----
        self.name=fname
        file_data=map(string.split,filter(lambda test_string : test_string.strip(), # is there anything in this line ?
                      file.readlines()))

        
        for line in file_data:
            resname=line[0][:-3]
            self._analyze_key(resname)  # - check the resname validity
            self.add(resname,map(float,line[1:]))
        
        self.column_names=self.column_names[1:]

#--------------     END OF CLASS DEFINITION -----------------

if len(sys.argv)!=3:
   print doc
   sys.exit()

#--- read the argument ---
colnum=int(sys.argv[1])-2
a=Sparky_LT_Peaklist(sys.argv[2])
if colnum>len(a[a.get_keys()[0]]):
    print "\n\nERROR! column number '%d' doesn't exist in '%s'!!!" %(colnum,sys.argv[2])
    sys.exit()
    
for res in a.get_keys():
    print res[1:],'\t', a[res][colnum]


