#!/usr/bin/env python
doc="""
Composite run: first we run NON-Exchanging set of residues
and then the full datasets using obtained approximation for
the tensor parameters

"""
import os,sys,string, time, pickle
import MFselect_lib_2 as MFselect_lib
import residues
import mylib
import keyfile
import copy
from MFselect_constants import *
from MFselect_flowcharts import *
import colors
from colors import *
#-- to make it portable: use color only when requested explicitly
if '-color' in sys.argv: colors.COLOR=ON
    


#----------------------------- NEW VERSION ---------------------
def composite_run(nonexch_inifile, full_inifile, mode=USE_INIT):
    """
    Non-exchanging residue set is used in the first iteration to get correct tensor estimate
    
    General selection scheme exactly reproducing the flowchart from Mandel et al. 1995

    We are doing the same selection order as Roger but watching for
    cases when the residue can be fit to M2 - 3 and M4-5
    In such cases - assign to 2 or 4 (as in old times)
    but put a comment about it
    """

    # create storage place for fitting parameters
    nonexch_fitted=MFselect_lib.Fitting_Parameters("nonexch")
    # load initial parameters and experimental data
    nonexch_init=MFselect_lib.Initial_Parameters(nonexch_inifile, nonexch_fitted)


    print 
    cprint(INV_RED, "******************* 'NO-EXCHANGE' DATASET **************************************")

    flow_chart(nonexch_init, nonexch_fitted, mode)


    cprint(INV_GREEN, "******************* FULL DATASET **************************************")

    # create storage place for fitting parameters
    fitted=MFselect_lib.Fitting_Parameters("full")
    # load initial parameters and experimental data
    init=MFselect_lib.Initial_Parameters(full_inifile, fitted)


    # transfer tensor parameters estimated in the NONEXCHANGE- run to 'init'

    fitted.Dratio[FITTEDval]=nonexch_fitted.Dratio[FITTEDval]
    fitted.Tm[FITTEDval]=nonexch_fitted.Tm[FITTEDval]
    fitted.Theta[FITTEDval]=nonexch_fitted.Theta[FITTEDval]
    fitted.Phi[FITTEDval]=nonexch_fitted.Phi[FITTEDval]


    flow_chart(init, fitted, mode)
#--------------------------------------------------








 
#--------------------------------------------------
# load initial parameters
full="MFselect_ini.full"
nonexch="MFselect_ini.nonexch"

t1=time.time()


#--- to run first nonexchanging sets and than full sets 
composite_run(nonexch, full)



#----- statistics -----
t2=time.time()
iter_time=t2-t1
cprint(BLINK, "\n\n----- ModelFree Analysis completed ------")
report="\n-----------\n Run time: %s min \n" % (iter_time/60.)
cprint(BLUE, report)
open("MFselect.log",'w').write(report)















