#!/usr/bin/env python
import os, sys, string, pickle
import MFselect_lib_2
import mylib
from MFselect_constants import *
import xmgr
doc="""
====================== MFselect_compare.py ======================

This is a simple program for visualization of MFselect results

Uses *.pickled files from MFselect

================================================================
"""
#------------------------------------------------------------
def remove_unassigned(data):
    'delete unassigned entries'
    to_remove=[]
    for res in data.Residue_list.get_keys():
        if data.Residue_list[res][MODEL]==UNASSIGNED: to_remove.append(res)

    print data.Residue_list.name, ": removing unassigned: ", to_remove
    for res in to_remove:
        del data.Residue_list[res]


#-------- the function plotting S2 vs residue explicitly ---------------------
def S2SvsRes(data7C,data20C,data27C,data37C,graph_name):
    'plot S2S vs residue number '
    s2s7C_data=data7C.Residue_list.to_xmgr(Xcol="Key", Ycol=S2Scurr, dYcol=S2Ssimerr)
    s2s20C_data=data20C.Residue_list.to_xmgr(Xcol="Key", Ycol=S2Scurr, dYcol=S2Ssimerr)
    s2s27C_data=data27C.Residue_list.to_xmgr(Xcol="Key", Ycol=S2Scurr, dYcol=S2Ssimerr)
    s2s37C_data=data37C.Residue_list.to_xmgr(Xcol="Key", Ycol=S2Scurr, dYcol=S2Ssimerr)


    s2s7=xmgr.Dataset(s2s7C_data,"7C");       s2s7.circle();   s2s7.black_sym()
    s2s20=xmgr.Dataset(s2s20C_data,"20C");   s2s20.circle(); s2s20.blue_sym()
    s2s27=xmgr.Dataset(s2s27C_data,"27C");   s2s27.circle(); s2s27.green_sym()
    s2s37=xmgr.Dataset(s2s37C_data,"37C");   s2s37.circle(); s2s37.red_sym()


    s2s_graph=xmgr.Graph("S2s")

    s2s_graph.add_dataset(s2s7)
    s2s_graph.add_dataset(s2s20)
    s2s_graph.add_dataset(s2s27)
    s2s_graph.add_dataset(s2s37)

    s2s_graph.set_xaxis("Residue",0,125)
    s2s_graph.set_yaxis("S2s",0,1.2,6)
    s2s_graph.set_errorbars("BOTH", 0.2)



    s2s_graph.writeout(graph_name)
#--------------------------------------------------------------------








#-------- the function plotting S2 vs T explicitly ---------------------   
def S2SvsT(data7C,data20C,data27C,data37C,graph_name):
    'plot S2S vs temperature '
    
    s2s7C_data=data7C.Residue_list.to_xmgr(Xcol="Key", Ycol=S2Scurr, dYcol=S2Ssimerr)
    s2s20C_data=data20C.Residue_list.to_xmgr(Xcol="Key", Ycol=S2Scurr, dYcol=S2Ssimerr)
    s2s27C_data=data27C.Residue_list.to_xmgr(Xcol="Key", Ycol=S2Scurr, dYcol=S2Ssimerr)
    s2s37C_data=data37C.Residue_list.to_xmgr(Xcol="Key", Ycol=S2Scurr, dYcol=S2Ssimerr)

    #--- create datasets S2S vs temperature
    T,S=0,1
    Texact={
             7:7.25,
             20:20.4,
             27:27.5,
             37:37.4,
           }

    res_list=mylib.common_key_list(data7C.Residue_list.get_keys(),
                             data20C.Residue_list.get_keys(),
                             data27C.Residue_list.get_keys(),
                             data37C.Residue_list.get_keys())

    #----------- plot S --------------------------------
    def add_point_S(S2vsTres,dataXC,Temp):
        'adding a point if it exists for this residue'
        if dataXC.Residue_list.has_key(res):
            S2vsTres.append([Temp,
                               dataXC.Residue_list[res][S2Scurr],
                               dataXC.Residue_list[res][S2Ssimerr]
                               ])
    #-----------------------------------------


    #----------- plot 1-S --------------------------------
    def add_point_1minusS(S2vsTres,dataXC,Temp):
        'adding a point if it exists for this residue'
        if dataXC.Residue_list.has_key(res):
            S2vsTres.append([Temp,
                               1-float(dataXC.Residue_list[res][S2Scurr]),
                               dataXC.Residue_list[res][S2Ssimerr]
                               ])


        
    dir=graph_name

    if not os.path.exists(dir):     os.mkdir(dir)
    
    S2vsT={} # res: [ [7, s2_7], [20, s2_20]...
    for res in res_list:
        S2vsT[res]=[]
        add_point_1minusS(S2vsT[res],data7C,Texact[7])
        add_point_1minusS(S2vsT[res],data20C,Texact[20])     
        add_point_1minusS(S2vsT[res],data27C,Texact[27])
        add_point_1minusS(S2vsT[res],data37C,Texact[37])


        #-------- save individual graphic files --------
        graph=xmgr.Graph(`res`)
        graph.add_dataset(xmgr.Dataset(S2vsT[res]))

        graph.set_xaxis("T, Deg C",0,50)
        graph.set_yaxis("1-S2s",0,.5,5)
        graph.set_errorbars("BOTH", 0.2)

        graph.writeout(dir+os.sep+`res`+".xmgr")
  


#--------------------------------------------------------------------
def tensor_average(*data):
    'averaging of tensor values'

    Dratio=Theta=Phi=0.0
    N=len(data)
    
    for dataset in data:
        Dratio+=float(dataset.Dratio[FITTEDval])
        Theta+=float(dataset.Theta[FITTEDval])
        Phi+=float(dataset.Phi[FITTEDval])

    Dratio/=N
    Theta/=N
    Phi/=N

    return {"Dratio" : Dratio,
            "Theta"  : Theta,
            "Phi"    : Phi}
        



#--------------------------------------------------------------------











#------------ Load and prepare datasets ----------------
#   
# Pickled results names:
fname_7C="7C.fixed_averaged_tensor.log.pickled"
fname_20C="20C.fixed_averaged_tensor.log.pickled"
fname_27C="27C.fixed_averaged_tensor.log.pickled"
fname_37C="37C.fixed_averaged_tensor.log.pickled"

db7C=mylib.open_file(fname_7C,'r')
db20C=mylib.open_file(fname_20C,'r')
db27C=mylib.open_file(fname_27C,'r')
db37C=mylib.open_file(fname_37C,'r')


data7C=pickle.load(db7C)
data20C=pickle.load(db20C)
data27C=pickle.load(db27C)
data37C=pickle.load(db37C)

# set proper names
data7C.Residue_list.set_name("7C")
data20C.Residue_list.set_name("20C")
data27C.Residue_list.set_name("27C")
data37C.Residue_list.set_name("37C")


#remove unassigned residues
remove_unassigned(data7C)
remove_unassigned(data20C)
remove_unassigned(data27C)
remove_unassigned(data37C)


#------------------------- Plotting --------------------------
S2SvsRes(data7C,data20C,data27C,data37C,"S2s_graph.xmgr")
S2SvsT(data7C,data20C,data27C,data37C,"1minusS2svsT.xmgr")
#print tensor_average(data7C,data20C,data27C,data37C) 
