#!/usr/bin/env python
doc="""
================== Extracting residues with the same model number =================
by Evgenii Kovrigin

The program extracts the data for those residues that have the same model in all submitted
files (unlimited number of files).

NOTE:
    1. Model number should sit in the SECOND column!
    2. No header lines are allowed


Usage:
     extract_same_model_residue_set.py d1 d2 



Outputs d1.extr d2.extr ... containing model assignments for the common set of residues

========================================================="""

import os, sys, string,  glob 
import mylib, xmgr, residues

MODEL=0  #- position of the model 

def same_model_set(set1,set2):
    'returns the subset of residues with the same model in two sets'
    same_models=[]
    for residue in set1.get_intersection(set2.get_keys()).get_keys():
        if set1[residue][MODEL]==set2[residue][MODEL]:
            same_models.append(residue)
    return set1.get_intersection(same_models)

NON_EMPTY_HEADER_LINES=0

if len(sys.argv) < 3:
    print doc
    sys.exit()


#-- source data
d=[]
for fname in sys.argv[1:]:
    d.append(residues.Word_Dict(fname,mode=1,nonempty_header_lines=NON_EMPTY_HEADER_LINES))

#-- find common set of residues
same_models=same_model_set(d[0],d[1])
if len(d)>2:
    for dataset in d[2:]:
        same_models=same_model_set(same_models,dataset)


print '*'*60

#--- form new datasets
d_extr=[]
for dataset in d:
    d_extr.append(dataset.get_intersection(same_models.get_keys()))

#--- export
for dataset in d_extr:
    dataset.writeout(dataset.get_name()+'.extr', header="off", verbose="on", mode='w')

