#!/usr/bin/env python
import os,sys,string, time, pickle
import MFselect_lib_2 as MFselect_lib
import residues
import mylib
import keyfile
import copy
from MFselect_constants import *

doc="""
MFselect.py    v.1
by Evgueni Kovriguine
EKN 13, flowchart (13-4)
- I assume you place resonable estimates for tm and tensor in the ini file?



- class FItting_Parameters creates the object keeping parameters of tensor and of the
    models from current and previous run.
    It also keeps track of models assignments (0 - initial, unassigned residue)
    Based on residues.Word_Dict() to be bug-proof

    It is being initilized by the Initial_Parameters constructor

- class Initial_Parameters creates the object keeping initial values of everything
    and experimental data

    Decision about convergence of global optimization is taken on the basis
    of comparison PREV and CURR values of model params and tensor

- model selection is done on the basis of Initial_Parameters.mfout
    data and newly assigned models are set in Fitting_Parameters object




- ! All the parameters are kept as text strings and need to be float() -ed
    to compare 



- use of USE_FITTED means that make_mfmodel() takes whatever value parameter
    had in the previous run NOT caring where it comes from !!!

- use:
        inifile="MFselect_ini"

        fitted_pars=MFselect_lib.Fitting_Parameters("fitting pars list name")

        init=MFselect_lib.Initial_Parameters(inifile, fitted_pars)

        run1=ModelFree(init, fitted, SINGLE_MODEL, USE_INIT, 1)


- tensor and tm fitted and current values: STARTval/FITTEDval - previous and current run
  results

  example:
  
    fitted.Tm[FITTEDval]=111
    fitted.Dratio[FITTEDval]=222
    fitted.Theta[FITTEDval]=333
    fitted.Phi[FITTEDval]=444

- after new iteration one can save new model parameters estimates:
 
    fitted_pars.take_new_tensor_pars(mfout)
    fitted_pars.take_new_model_pars(self, res_list, # residues to take
                            mfout):

1/7/04   - convergence criteria for tensor and S2 are now <= instead of <
"""
#---------- SSE test MODEL SELECTION -------------------
def select_Model_by_Gamma(mfout,init_pars):
    'selection of residues by SSE < SSE(critical)'
    list=[]

    for res in mfout.reslist.sorted_keys():
        # Gamma(i) < Gamma(i) critical?
        Gamma=float(mfout.reslist[res][GAMMA])
        Gamma_critical=float(mfout.reslist[res][GAMMAcritical])    
        if Gamma < Gamma_critical:
            list.append(res)
            
    return list


#---------------------------------------------------
def ftest_Model(mfout, init_pars):
    'selection of residues by Fvalue > Fvalue(critical)'
    list=[]

    for res in mfout.reslist.sorted_keys():
        # Fvalue > Fvalue(critical)
        F=float(mfout.reslist[res][FVALUE])
        Fcritical=float(mfout.reslist[res][FVALUEcritical])
        if F > Fcritical:
            list.append(res)

    return list    



#--------------------------------------
def select_Bad_Model_1(unassigned_list, mfout,init_pars):
    'Gamma < Gamma_cutoff_models_45 -> model 1'
    list=[]

    for res in unassigned_list:
        #  Gamma < Gamma_cutoff_models_45 -> model
        Gamma=float(mfout.reslist[res][GAMMA])
        Gamma_critical=float(init_pars.Gamma_cutoff_models_45)   
        if Gamma < Gamma_critical:
            list.append(res)
            
    return list
        



    
#--------------------------------------------
def select_Model_4(mfout, init):
    'Gamma < 0.001'
    list=[]

    for res in mfout.reslist.sorted_keys():
        # Gamma(i) < 0.001
        Gamma=float(mfout.reslist[res][GAMMA])

        if Gamma < 0.001:
            list.append(res)
            
    return list




#--------------------------------------------
def select_Model_5(mfout, init):
    'Gamma < 0.001 and te<tm and te>te_error'
    list=[]
    # read Tm
    mfout._get_tensor()
    tm=float(mfout.tm)
    
    for res in mfout.reslist.sorted_keys():
        # Gamma(i) < 0.001
        Gamma=float(mfout.reslist[res][GAMMA])
        te=float(mfout.reslist[res][TE])
        te_error=float(mfout.reslist[res][TEstddev])
        
        if (Gamma < 0.001) and (te<tm) and (te>te_error):   list.append(res)
            
    return list














def Rogers_version(inifile, mode=USE_INIT):
    '2 after 3 and 4 after 5'
    # create storage place for fitting parameters
    fitted=MFselect_lib.Fitting_Parameters("37C")
    # load initial parameters and experimental data
    init=MFselect_lib.Initial_Parameters(inifile, fitted)
    
    iteration=1

    while 1:
        log=open("%s_%d" % (fitted.Residue_list.get_name(),iteration),'a')
    #******** MODEL 1 **************
        print "\n********* MODEL 1 ********************"        
        full_list=fitted.Residue_list.sorted_keys()

        print "M1 submitted:", full_list
        # run Model 1 on the full dataset
        m1=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M1)
        m1.start(full_list,"m1")
        mfout1=m1.mfout("m1")


        m1_list=select_Model_by_Gamma(mfout1,init)

        print "m1_list\n", m1_list    
        fitted.assign_model(m1_list,mfout1, M1)




    #******** MODEL 2 **************
        print "\n********* MODEL 2 ********************"    
        unassigned_list=fitted.unassigned_list()
        if len(unassigned_list)>0:

            print "M2 submitted:", unassigned_list
            # run Model 2 
            m2=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M2)
            m2.start(unassigned_list,"m2")
            mfout2=m2.mfout("m2")
            
            m2_candidate_list=select_Model_by_Gamma(mfout2,init)

            if len(m2_candidate_list)>0:

                print "M12 submitted:", m2_candidate_list
                # run F-test on candidate
                m12=MFselect_lib.ModelFree(init, fitted, FTEST, mode, M1, M2)
                m12.start(m2_candidate_list,"m12")
                mfout12=m12.mfout("m12")

                m2_list=ftest_Model(mfout12,init)
                print "m2_list\n", m2_list
                # take fitted params
                fitted.assign_model(m2_list,mfout2, M2)


    #******** MODEL 3 **************
        print "\n********* MODEL 3 ********************"
        unassigned_list=fitted.unassigned_list()

        if len(unassigned_list)>0:

            print "M3 submitted:", unassigned_list
            # run Model 3 
            m3=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M3)
            m3.start(unassigned_list,"m3")
            mfout3=m3.mfout("m3")
              
            m3_candidate_list=select_Model_by_Gamma(mfout3,init)

            if len(m3_candidate_list)>0:

                print "M13 submitted:",  m3_candidate_list
                # run F-test on candidate
                m13=MFselect_lib.ModelFree(init, fitted, FTEST, mode, M1, M3)
                m13.start(m3_candidate_list,"m13")
                mfout13=m13.mfout("m13")

                m3_list=ftest_Model(mfout13,init)
                print "m3_list\n", m3_list
                # take fitted params
                fitted.assign_model(m3_list,mfout3, M3)


    #******** 'BAD' MODEL 1 **************
        print "\n********* BAD MODEL 1 ********************"
        unassigned_list=fitted.unassigned_list()
        print "bad M1 submitted:", unassigned_list       
        if len(unassigned_list)>0:
            
            bad_m1_list=select_Bad_Model_1(unassigned_list, mfout1, init)
            print "'bad' m1_list\n", bad_m1_list        
            fitted.assign_model(bad_m1_list, mfout1, M1, "bad m1")    


    #********* MODEL 4 ********************
        print "\n********* MODEL 4 ********************"
        unassigned_list=fitted.unassigned_list()   

        if len(unassigned_list)>0:

            print "M4 submitted:", unassigned_list
            m4=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M4)
            m4.start(unassigned_list,"m4")
            mfout4=m4.mfout("m4")
           

            m4_list=select_Model_4(mfout4,init)
            print "m4_list\n", m4_list        
            fitted.assign_model(m4_list,mfout4, M4)
            

    #********* MODEL 5 ********************
        print "\n********* MODEL 5 ********************"
        unassigned_list=fitted.unassigned_list()

        if len(unassigned_list)>0:
            
            print "M5 submitted:", unassigned_list  
            m5=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M5)
            m5.start(unassigned_list,"m5")
            mfout5=m5.mfout("m5")
         

            m5_list=select_Model_5(mfout5,init)
            print "m5_list\n", m5_list
            fitted.assign_model(m5_list,mfout5, M5)


    #************ TENSOR **********************
        print "\n********* Global Optimization ********************"
        list=fitted.assigned_list()
        print "Submitted: ", list
        opt=MFselect_lib.ModelFree(init, fitted, TENSOR_OPT, USE_FITTED)
        opt.start(list,"optimization")
        mfout_opt=opt.mfout("optimization")
        
        fitted.take_new_tensor_pars(mfout_opt)

        fitted.take_new_model_pars(list,mfout_opt)
        

        iteration+=1
        log.write(str(fitted))

        if fitted.S2_converged():
                log.write("S2 converged\n")
        else:
                log.write("S2 didn't converge\n")

        if fitted.tensor_converged():
            log.write("tensor converged\n")

        log.close()
        
        if fitted.S2_converged() and  fitted.tensor_converged():  break

        #reset assignments
        fitted.reset_models()

    print "Converged"
#-------------------------------------------------------------------------------




################################################################################
################################################################################
################################################################################
################################################################################












#--------------------- ACTUAL FLOWCHART as of 09/25/2002 ------------------------
from  MFselect_checklist_before_run import *

def flow_chart(init, fitted, mode, do_optimization=YES):
    'the actual model selection and optimization'


    #-- checking parameters before feeding them to FORTRAN
    MFselect_checklist(init)
    
    
    iteration=1  


    
    fields=MFselect_lib.make_fields_list(init.R1_fields_list, init.R2_fields_list, init.NOE_fields_list)
    
    # remove this check after F-test for 
    if len(fields)>1:
        print "Warning!  Number of fields used is %s : F-test for models 4 and 5 is\n\
                appropriate but not implemented yet!" % len(fields)


    while 1:
        start_time=time.time()

        print "\n\n", "="*80
        print "\t\tIteration ", iteration
        print "="*80

        logname="%s_%d" % (fitted.Residue_list.get_name(),iteration)
        log=open(logname,'a')
    #******** MODEL 1 **************
        print "\n********* MODEL 1 ********************"        
        full_list=fitted.Residue_list.sorted_keys()

        print "M1 submitted:", full_list
        # run Model 1 on the full dataset
        m1=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M1)
        m1.start(full_list,"m1")
        mfout1=m1.mfout("m1")


        m1_list=select_Model_by_Gamma(mfout1,init)

        print "Selected M1: \n", m1_list
        log.write("\nSelected M1:\n%s\n" % m1_list)
        fitted.assign_model(m1_list,mfout1, M1)




    #******** MODEL 2 **************
        print "\n********* MODEL 2 ********************"    
        unassigned_list=fitted.unassigned_list()
        if len(unassigned_list)>0:

            print "M2 submitted:", unassigned_list
            # run Model 2 
            m2=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M2)
            m2.start(unassigned_list,"m2")
            mfout2=m2.mfout("m2")
            
            m2_candidate_list=select_Model_by_Gamma(mfout2,init)

            m2_list=[]
            if len(m2_candidate_list)>0:

                print "M12 submitted:", m2_candidate_list
                # run F-test on candidate
                m12=MFselect_lib.ModelFree(init, fitted, FTEST, mode, M1, M2)
                m12.start(m2_candidate_list,"m12")
                mfout12=m12.mfout("m12")

                m2_list=ftest_Model(mfout12,init)

                print "Selected M2: \n", m2_list
                log.write("\nSelected M2:\n%s\n" % m2_list)
                # take fitted params
                fitted.assign_model(m2_list,mfout2, M2)
                fitted.get_ftest_quality(m2_list, mfout12)

    #******** MODEL 3 **************
        print "\n********* MODEL 3 ********************"

        if len(unassigned_list)>0:

            print "M3 submitted:", unassigned_list
            # run Model 3 
            m3=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M3)
            m3.start(unassigned_list,"m3")
            mfout3=m3.mfout("m3")
              
            m3_candidate_list=select_Model_by_Gamma(mfout3,init)

            if len(m3_candidate_list)>0:

                print "M13 submitted:",  m3_candidate_list
                # run F-test on candidate
                m13=MFselect_lib.ModelFree(init, fitted, FTEST, mode, M1, M3)
                m13.start(m3_candidate_list,"m13")
                mfout13=m13.mfout("m13")

                m3_list=ftest_Model(mfout13,init)

#************ Set a marker at those M2 which could be M3
#               but leave them being M2
                if len(m2_list)>0 and len(m3_list)>0:                
                    log.write("M3 as good as M2:")
                    print "M3 as good as M2:",
                    for res in m3_list:
                        if res in m2_list:
                            print "  %d  " % res
                            log.write("  %s  " % res)
                            fitted.comment(res,"good M3")
                            m3_list.remove(res)
                            
                print "\nSelected M3: \n", m3_list
                log.write("\nSelected M3:\n%s" % m3_list)

                fitted.assign_model(m3_list,mfout3, M3)
                fitted.get_ftest_quality(m3_list, mfout13)

    #******** 'BAD' MODEL 1 **************
        print "\n********* BAD MODEL 1 ********************"
        unassigned_list=fitted.unassigned_list()
        print "bad M1 submitted:", unassigned_list       
        if len(unassigned_list)>0:
            
            bad_m1_list=select_Bad_Model_1(unassigned_list, mfout1, init)
            log.write("\n'Bad' M1:\n%s" % bad_m1_list)
            print "'bad' m1_list\n", bad_m1_list        
            fitted.assign_model(bad_m1_list, mfout1, M1, "bad m1")    


    #********* MODEL 4 ********************
        print "\n********* MODEL 4 ********************"
        unassigned_list=fitted.unassigned_list()   

        m4_list=[]
        if len(unassigned_list)>0:

            print "M4 submitted:", unassigned_list
            m4=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M4)
            m4.start(unassigned_list,"m4")
            mfout4=m4.mfout("m4")
           

            m4_list=select_Model_4(mfout4,init)
            print "Selected M4: \n", m4_list
            log.write("\nSelected M4:\n%s" % m4_list)

            fitted.assign_model(m4_list,mfout4, M4)
            

    #********* MODEL 5 ********************
        print "\n********* MODEL 5 ********************"

        if len(unassigned_list)>0:
            
            print "M5 submitted:", unassigned_list  
            m5=MFselect_lib.ModelFree(init, fitted, SINGLE_MODEL, mode, M5)
            m5.start(unassigned_list,"m5")
            mfout5=m5.mfout("m5")
         

            m5_list=select_Model_5(mfout5,init)

#************ Set a marker at those M4 which could be M5
#               but leave them being M4

            if len(m5_list)>0 and len(m4_list)>0:                
                log.write("\nM5 as good as M4:")
                print "M5 as good as M4:",

                for res in m5_list:
                    if res in m4_list:
                        print "  %d  " % res
                        log.write("  %s  " % res)
                        fitted.comment(res,"good M5")
                        m5_list.remove(res)

            print "\nSelected M5: \n", m5_list
            log.write("\nSelected M5:\n%s" % m5_list)

            fitted.assign_model(m5_list,mfout5, M5)

        # --- skip optimization ---
        if do_optimization!=YES :
            log.write("\nTensor optimization skipped\n")
            log.close()
            break
        
    #************ TENSOR **********************
        print "\n********* Global Optimization ********************"
        list=fitted.list_for_tensor_optimization(init)
        if not len(list) > 0:
            raise UserWarning, "\n\n\a\a\aNo residues got assigned! Try another initial parameters..."
        
        print "Submitted: ", list
        opt=MFselect_lib.ModelFree(init, fitted, TENSOR_OPT, USE_FITTED)
        opt.start(list,"optimization")
        mfout_opt=opt.mfout("optimization")
        
        fitted.take_new_tensor_pars(mfout_opt)
        fitted.take_new_model_pars(list,mfout_opt)
        fitted.get_gamma_selection_quality(list, mfout_opt)
        

        iteration+=1
        log.write(str(fitted))

        end_time=time.time()
        iter_time=end_time-start_time
        log.write("\n-----------\nIteration time: %s min\n" % (iter_time/60.))


        if fitted.S2_converged():
            log.write("S2 converged within 1/%s simulated error\n" % fitted.factor)
        else:
            log.write("S2 DIDN't converge within 1/%s simulated error\n" % fitted.factor)
        
        if fitted.tensor_converged():
            log.write("Tensor converged within 1/%s simulated error\n" % fitted.factor)
        else:
            log.write("Tensor DIDN't converge within 1/%s simulated error\n" % fitted.factor)            
        log.close()
        
        if fitted.S2_converged() and fitted.tensor_converged():
            break

        #reset assignments
        fitted.reset_models()
    #---------- end of optimization loop ---------------


    if do_optimization==YES :    
    #--------- LAST RUN to update low S2 model parameter using optimized tensor
    #           (low S2 are excluded from tensor optimization runs)
        list=fitted.assigned_lowS2_list(init)
        print "\n*********** Low S2 updating ***********\n"
        if list:
            print "Submitted low S2S list:", list
            lowS2=MFselect_lib.ModelFree(init, fitted, LAST_RUN, USE_INIT)
            lowS2.start(list,"lowS2")
            mfout_lowS2=lowS2.mfout("lowS2")
            fitted.take_new_model_pars(list,mfout_lowS2)
            fitted.get_gamma_selection_quality(list, mfout_lowS2)


    #--- final output ----------
    model_assignments=MFselect_lib.show_models(fitted)
    open(fitted.Residue_list.get_name()+".final", "w").write(str(fitted)+model_assignments)
    #--- save 'fitted' data for future loads
    mydb=open(fitted.Residue_list.get_name()+".pickled",'w')
    pickle.dump(fitted, mydb)
    mydb.close()
 
    print "Converged"
#-------------------------------------------------------------------------------
   

