#!/usr/bin/env python
import string, sys
doc=""" 

========= Convert MicroCal Origin ITC data to LineShapeKin format ========

v.1    12/07/08

by Evgenii Kovrigin, Medical College of Wisconsin




Usage: Microcal_to_LineShapeKin.py DATfile [LPratio_column DH_column ]
    
    where:
    
        DATfile - a .DAT exported from MicroCal Origin with options
                    - 'Include column names'
                    - Separator: TAB
                    
    optional arguments:
    
        LPratio_column  - column for the titrant/target ratio (X-axis)
                            if other than 5th
        DH_column       - column for enthalpy per mol of injectant 
                            if other than 6th
    


Description: 
- Reads DAT file output by Microcal Origin ITC processing package
- Removes column headers 
- Removes any line containing '--' in the DH column
- Extracts LPratio and DH columns 
- Output file has added .txt extension
=================================================================

"""
import string

#-- columns (from 1,...)
LPratio_column=5
DH_column=6


#-- get command line arguments
Nargs=len(sys.argv)
if (Nargs <2) or  Nargs > 2 and Nargs!=4:
    print doc
    sys.exit()
    
fname=sys.argv[1]

if Nargs==4:
    LPratio_column=sys.argv[2]
    DH_column=sys.argv[3]
    
#-- open file
fid=open(fname, 'r')
source_file=fid.readlines()
fid.close()

#-- get rid of header and a last line
print 'Dropping first and last lines:'
print 'First line was:\n %s' %  source_file[0],
print 'Last line was:\n %s' %  source_file[-1]

source_file=source_file[1:-1]



#-- extract data
XY_data=''
for line in source_file:
    words=line.split()
    LPratio=words[LPratio_column-1]
    DH=words[DH_column-1]
    
    #-- skip excluded point
    if DH=='--': continue
    
    XY_data+=LPratio+'\t'+DH+'\n'
    
print 'Extracted data:\n   L/P       DH \n------------------'
print XY_data

#--- save data
fid=open(fname+'.txt', 'w')
fid.write(XY_data)
fid.close()

    



















