#--- 'NOEratio_lib.py' --------
# library of functions for 'NOEratio.py'

import os, sys, string
from math import *
import keyfile

from NOEratio_ini import *  # import constants

#----- reads Sparky 'lt' list of peak heghts -------

LT_RESIDUE,LT_HEIGHT=0,3  # column numbers in 'lt' list

def lt_file(path,fname):
    'read "lt" format -> Keyword_File object (res, height)'
    lastword="Height"
    # load
    file=keyfile.Keyword_File(path,fname)
    # make sure it is 'lt' generated peak list
    if not file[0][-1]==lastword:
        raise UserWarning, ("\n\n\a\a\aFile '%s' is not a peak list!\n\
(last word in the header of a Sparky 'lt' file is '%s' while '%s' found!)"
                            % (file.filename, lastword, file[0][-1]))
    # remove header
    file.text=file.text[1:]
    
    # remove N-H suffix from residue names
    # replace string data with numeric
    for line in file:
        line[LT_RESIDUE]=line[LT_RESIDUE][:-3]
        line[LT_HEIGHT]=string.atof(line[LT_HEIGHT])
        # remove chemical shifts
        del(line[1])
        del(line[1])
    return file
#----------------------------------------------
# calculates Havg and dH/Havg
def dHbyH(list1,list2):
    
    diff_list=[]
    if len(list1)!=len(list2):
        raise UserWarning, (
            "\n\n\a\a\aDifferent sizes of datasets! %s and %s"
            % (len(list1),len(list2)))
    for i in range(len(list1)):
        if list1[i][RESIDUE]!=list2[i][RESIDUE]:
            raise UserWarning, (
                "\n\n\a\a\aAssignments are different in files:\n'%s'\n'%s'\n\
%s data are found on the line for %s!" %
                (list1.filename,list2.filename,list1[i][RESIDUE],list2[i][RESIDUE]))
        # calculations
        Havg=(list1[i][HEIGHT]+list2[i][HEIGHT])/2
        dH=(list1[i][HEIGHT]-list2[i][HEIGHT])
        diff_list.append([Havg,dH/Havg])
    return diff_list

#---------- determine sigma and mean of dH/Havg
# not including peaks with heights lower than cutoff
def dHbyH_statistics(HdHbyH_list, height_cutoff ) :
    'eats [(Havg,dH/H),(., ),...] -> mean, sigma'
    #remove low height points
    new_list=HdHbyH_list[:]
    for H,dH in new_list:
        if H<height_cutoff:
            new_list.remove([H,dH])
    # check if anything is left
    if len(new_list)<1:
        raise UserWarning, (
            "All peaks are lower than the required cutoff %s!!!" % height_cutoff
            )
    # statistics
    print "Initial dataset is %d peaks" % len(HdHbyH_list)
    print "Above cutoff (used in sigma estimate): %d peaks" % len(new_list)
    sum=0
    sumsquare=0
    N=len(new_list)
    for i in range(N) :
        dHbyH=new_list[i][1]
        sum=sum+dHbyH
        sumsquare=sumsquare+dHbyH*dHbyH
    mean=sum/N
    palmer_sigma=sqrt(((sumsquare-sum*sum/N)/(N-1)))
    return mean,palmer_sigma/sqrt(2.0)

#----------------- add proper sigmas to peaks in 'added' spectrum ----------
def add_sigmas(peak_list, sigma, added_noise_rmsd):
    # append sigmas as the third column
    for i in range(len(peak_list)):
        sigmaH= peak_list[i][HEIGHT]*sigma+added_noise_rmsd
        peak_list[i].append(sigmaH)
#----------------------------------------------------------------------------   

#--- replace residue name with its number (new list created)   
def residue2number(list):
    'A109 -> int(109)'
    new_list=[]
    for i in range(len(list)):
        new_list.append(list[i])
        new_list[i][RESIDUE]=string.atoi(new_list[i][RESIDUE][1:])
    return new_list
