import os,sys,string
import keyfile, residues
from MFselect_constants import *




class Fitting_Parameters:
    'Main data storage for residues in work: parameters of current and previous fitting'
    def __init__(self,data_name):
        # residue parameters  to keep track during fitting
        self.Residue_list=residues.Word_Dict() # it is loaded by Initial_Parameters constructor
        self.Residue_list.set_name(data_name+"_Residues")

        # tensor parameters to keep track during fitting
        self.Tensor_type=""
        # keys=     STARTval,  - reference to previous parameters values
        #           FITTEDval - keeps current 
        self.Dratio={}
        self.Tm={}
        self.Theta={}
        self.Phi={}
        self.factor=1

    def __repr__(self):
        text="\nTensor variable\tPrevious\tCurrent\t\tSimError\n"
        text+="Tm\t\t%s\t\t%s\t\t%s\n" % (self.Tm[STARTval],self.Tm[FITTEDval],self.Tm[SIMerror])
        text+="Dratio\t\t%s\t\t%s\t\t%s\n" % (self.Dratio[STARTval],self.Dratio[FITTEDval],self.Dratio[SIMerror])        
        text+="Theta\t\t%s\t\t%s\t\t%s\n" % (self.Theta[STARTval],self.Theta[FITTEDval],self.Theta[SIMerror])        
        text+="Phi\t\t%s\t\t%s\t\t%s\n" % (self.Phi[STARTval],self.Phi[FITTEDval],self.Phi[SIMerror])
        text+="\n%s" % self.Residue_list
        return text
        
    def _shift_tensor_pars(self):
        'move CURRENT tensor results to PREVIOUS\' place before loading new iteration parameters'
        self.Dratio[STARTval]=self.Dratio[FITTEDval]
        self.Tm[STARTval]=self.Tm[FITTEDval]
        self.Theta[STARTval]=self.Theta[FITTEDval]
        self.Phi[STARTval]=self.Phi[FITTEDval]

    def _shift_model_pars(self, res_list):
        'move CURRENT model pars to PREVIOUS\' place before loading new iteration parameters'
        for i in res_list:
            
            self.Residue_list.set_word(i,S2Sprev,self.Residue_list[i][S2Scurr]) 
            self.Residue_list.set_word(i,S2Fprev,self.Residue_list[i][S2Fcurr])
            self.Residue_list.set_word(i,TEprev,self.Residue_list[i][TEcurr])
            self.Residue_list.set_word(i,REXprev,self.Residue_list[i][REXcurr])

    def _shift_all(self, res_list):
        'moves "current" iteration results to "previous" one place'
        self.shift_tensor_pars()
        self.shift_model_pars(res_list)

    def unassigned_list(self):
        '->list of unassigned residues'
        list=[]
        for res in self.Residue_list.sorted_keys():
            if self.Residue_list[res][MODEL]==UNASSIGNED:
                list.append(res)
        return list

    def assigned_list(self):
        list=[]
        for res in self.Residue_list.sorted_keys():
            if self.Residue_list[res][MODEL]!=UNASSIGNED:
                list.append(res)
        return list

    def list_for_tensor_optimization(self, inipars):
        'assigned + S2S<cutoff value'
        list=self.assigned_list()
        for res in list:
            S2=float(self.Residue_list[res][S2Scurr])
            if S2<inipars.S2min:
                list.remove(res)
        return list
            

    def take_new_tensor_pars(self,mfout):
        'get new tensor estimates from mfout'
        self._shift_tensor_pars()

        self.Dratio[FITTEDval]=mfout.Dratio
        self.Tm[FITTEDval]=mfout.tm
        self.Theta[FITTEDval]=mfout.Theta
        self.Phi[FITTEDval]=mfout.Phi

        self.Dratio[SIMerror]=mfout.tm_err
        self.Tm[SIMerror]=mfout.Dratio_err
        self.Theta[SIMerror]=mfout.Theta_err
        self.Phi[SIMerror]=mfout.Phi_err


    def take_new_model_pars(self, res_list, # residues to take
                            mfout):
        'get new model parameter estimates from mfout'
        self._shift_model_pars(res_list)

        for res in res_list:
            self.Residue_list.set_word(res,S2Scurr,mfout.reslist[res][S2S]) 
            self.Residue_list.set_word(res,S2Fcurr,mfout.reslist[res][S2F])
            self.Residue_list.set_word(res,TEcurr,mfout.reslist[res][TE])
            self.Residue_list.set_word(res,REXcurr,mfout.reslist[res][REX])


            self.Residue_list.set_word(res,S2Ssimerr,mfout.reslist[res][S2Sstddev]) 
            self.Residue_list.set_word(res,S2Fsimerr,mfout.reslist[res][S2Fstddev])
            self.Residue_list.set_word(res,TEsimerr,mfout.reslist[res][TEstddev])
            self.Residue_list.set_word(res,REXsimerr,mfout.reslist[res][REXstddev])
            self.Residue_list.set_word(res,GAMMAfitted ,mfout.reslist[res][GAMMA])

    def comment(self,residue, comment):
            self.Residue_list.set_word(residue,COMMENT,comment)
        
    def assign_model(self, res_list, mfout, model_number, comment="no comment"):
        'sets particular residues to the Model'
        for res in res_list:
            self.Residue_list.set_word(res,MODEL,model_number)
            self.Residue_list.set_word(res,COMMENT,comment)
       
        self.take_new_model_pars(res_list, mfout)



    def tensor_converged(self):
        'yes/no'
        
        a1= abs(float(self.Dratio[FITTEDval])-float(self.Dratio[STARTval])) < float(self.Dratio[SIMerror])/self.factor
        a2= abs(float(self.Tm[FITTEDval])-float(self.Tm[STARTval])) < float(self.Tm[SIMerror])/self.factor
        a3= abs(float(self.Theta[FITTEDval])-float(self.Theta[STARTval])) < float(self.Theta[SIMerror])/self.factor
        a4= abs(float(self.Phi[FITTEDval])-float(self.Phi[STARTval])) < float(self.Phi[SIMerror])/self.factor

        if a1 and a2 and a3 and a4: return YES
        
        return NO



    
            
    def S2_converged(self):
        'yes/no'
        a=1
        for res in self.assigned_list():
            
            curr=float(self.Residue_list[res][S2Scurr])
            prev=float(self.Residue_list[res][S2Sprev])
            err=float(self.Residue_list[res][S2Ssimerr])

            b=abs(curr-prev) < err/self.factor

            a*=b

        if a :return YES

        return NO

    def reset_models(self):
        'set MODEL to UNASSIGNED'
        for res in self.Residue_list.get_keys():
            self.Residue_list.set_word(res,MODEL,UNASSIGNED)
            self.Residue_list.set_word(res,COMMENT,"none")


 





#------------------ Initial parameters storage ------------------
class Initial_Parameters:
    'keeps all initial parameters together. Ini file and data - current directory'
    # Main data storage: .Residue_list  (Word_Dict)

    def __init__(self, inifname, # parameter file name
                 Fit_Pars    # Fitting_Parametes object to initialize (residue model parameters and tensor)
                 ):
        #---- load file 
        self.inidata=keyfile.Keyword_File(os.curdir,inifname,"verbose")

        #---- check number of fields
        self.R1_fields_list=self.inidata.get_list("<R1_fields_list>")
        self.R2_fields_list=self.inidata.get_list("<R2_fields_list>")
        self.NOE_fields_list=self.inidata.get_list("<NOE_fields_list>")

        #check for at least 1 R1 dataset:
        if len(self.R1_fields_list)<1 :
            raise UserWarning, "\n\n\a\a\aError! You must have at least one R1 dataset!"

        # PDB data file processed by pdbinertia and 
        self.pdb=self.inidata.get_string("<pdb>")

        # data files
        self.R1_data={}
        for i in range(len(self.R1_fields_list)):
            R1_name="<R1_field%d>" % (i+1)
            temp=residues.Annn_Text(self.inidata.get_string(R1_name))
            self.R1_data[self.R1_fields_list[i]]=temp
            if i==0 :
                reference_list=temp  # first dataset serves as reference for residue lists
            else:
                residues.check_keys_match(temp,reference_list)
                

        self.R2_data={}
        for i in range(len(self.R2_fields_list)):
            R2_name="<R2_field%d>" % (i+1)
            temp=residues.Annn_Text(self.inidata.get_string(R2_name))
            self.R2_data[self.R2_fields_list[i]]=temp
            residues.check_keys_match(temp,reference_list)
 

        self.NOE_data={}
        for i in range(len(self.NOE_fields_list)):
            NOE_name="<NOE_field%d>" % (i+1)
            temp=residues.Annn_Text(self.inidata.get_string(NOE_name))
            self.NOE_data[self.NOE_fields_list[i]]=temp
            residues.check_keys_match(temp,reference_list)

        # create residue list
        Annn_reslist=self.R1_data[self.R1_fields_list[0]].sorted_keys()



        

        # ------- Main data storage for residues in work ----------
        for name in Annn_reslist:
            Fit_Pars.Residue_list.add(int(name[1:]),[0]*FITTED_PARS_COLUMNS)

            

        #------------ ModelFree parameters -------------------------
        # tensor type: axial/isotropic
        self.tensor=self.inidata.get_string("<tensor>")
        Fit_Pars.tensor_type=self.tensor
        Fit_Pars.factor=float(self.inidata.get_string("<convergence_factor>"))
        
        # grid search during tensor optimization: Yes/No
        self.axial_gridsearch=self.inidata.get_string("<axial_gridsearch>")
        
        #mfin parameters
        self.optimization_type=self.inidata.get_string("<optimization_type>")    
        self.RND_generator_seed=self.inidata.get_string("<RND_generator_seed>")   
        self.Simulations_number=self.inidata.get_string("<Simulations_number>")
        
        # Initial global tm guess (from 'tmest'), limits and the steps number for the gridsearch
        self.tm=self.inidata.get_string("<tm>")       
        self.tmMin=self.inidata.get_string("<tmMin>")
        self.tmMax=self.inidata.get_string("<tmMax>") 
        self.tmGrid=self.inidata.get_string("<tmGrid>")         

        Fit_Pars.Tm[STARTval]=self.tm
        Fit_Pars.Tm[FITTEDval]=self.tm
        Fit_Pars.Tm[SIMerror]=0

        # Parameters of axial symmetric tensor determined by 'r2r1_diffusion'.
        if self.tensor=="axial":
            self.Dratio=self.inidata.get_string("<Dratio>")            
            self.DratioMin=self.inidata.get_string("<DratioMin>") 
            self.DratioMax=self.inidata.get_string("<DratioMax>") 
            self.DratioGrid=self.inidata.get_string("<DratioGrid>")

            # Tensor orientation angles
            self.Theta=self.inidata.get_string("<Theta>") 				    
            self.ThetaMin=self.inidata.get_string("<ThetaMin>") 
            self.ThetaMax=self.inidata.get_string("<ThetaMax>") 
            self.ThetaGrid=self.inidata.get_string("<ThetaGrid>")  

            self.Phi=self.inidata.get_string("<Phi>") 
            self.PhiMin=self.inidata.get_string("<PhiMin>")				
            self.PhiMax=self.inidata.get_string("<PhiMax>")
            self.PhiGrid=self.inidata.get_string("<PhiGrid>")

            # set fitting parameters object
            Fit_Pars.Dratio[STARTval]=self.Dratio
            Fit_Pars.Dratio[FITTEDval]=self.Dratio
            Fit_Pars.Dratio[SIMerror]=0

            Fit_Pars.Theta[STARTval]=self.Theta
            Fit_Pars.Theta[FITTEDval]=self.Theta
            Fit_Pars.Theta[SIMerror]=0

            Fit_Pars.Phi[STARTval]=self.Phi
            Fit_Pars.Phi[FITTEDval]=self.Phi
            Fit_Pars.Phi[SIMerror]=0


        #------------ Parameters Optimization Limits ------------
        self._model_pars={}
        # ******** Model 1 ***********************************
        self._model_pars[S2S]=self.inidata.get_list("<S2s_ini>")
        self._model_pars[S2F]=self.inidata.get_list("<S2f_ini>") 
        self._model_pars[TE]=self.inidata.get_list("<te_ini>")
        self._model_pars[REX]=self.inidata.get_list("<Rex_ini>") 

        # te is to be checked not to exceed tm in the model 5
        # so it should be able to exceed it if it wants: 
        self._model_pars[TE][MAXval]=str(float(self.tm)*1500)

        # load them now in the Fitting_Parameters object:
        for i in Fit_Pars.Residue_list.get_keys():

            Fit_Pars.Residue_list.set_word(i,S2Sprev,self._model_pars[S2S][INITval])
            Fit_Pars.Residue_list.set_word(i,S2Scurr,self._model_pars[S2S][INITval])

            Fit_Pars.Residue_list.set_word(i,S2Fprev,self._model_pars[S2F][INITval])
            Fit_Pars.Residue_list.set_word(i,S2Fcurr,self._model_pars[S2F][INITval])

            Fit_Pars.Residue_list.set_word(i,TEprev,self._model_pars[TE][INITval])
            Fit_Pars.Residue_list.set_word(i,TEcurr,self._model_pars[TE][INITval])

            Fit_Pars.Residue_list.set_word(i,REXprev,self._model_pars[REX][INITval])
            Fit_Pars.Residue_list.set_word(i,REXcurr,self._model_pars[REX][INITval])


        # ****** all: if diffusion model - local
        # not really used anywhere
        self.tloc=self.inidata.get_list("<tloc>")  
        self.theta=self.inidata.get_list("<theta>")
        

        #====== technical parameters =======

        # Maximum number of iterations.
        #self.Max_Iterations=self.inidata.get_string("<Max_Iterations>")
        # Sets range for Gamma deviations: 0.90/0.95/etc
        self.Gamma_conf_level=self.inidata.get_string("<Gamma_conf_level>")
        # This is the cutoff for the Ftests
        self.F_test_level=self.inidata.get_string("<F_test_level>")

        # Gamma should exceed Gamma(Gamma_conf_level) this many times
        # to be considered for Models 4 and 5 selection. If Gamma
        # doesn't exceed - assign it to 'Bad Model 1'
        self.Gamma_cutoff_models_45=self.inidata.get_string("<Gamma_cutoff_models_45>")

        # Minimal value S2 can have for the residue to be used in
        # in full tensor optimization                                
        self.S2min=float(self.inidata.get_string("<S2min>"))

        #====================== Atom parameters ===========================
        self.Nucleus=self.inidata.get_string("<Nucleus>")
        self.Gamma=self.inidata.get_string("<Gamma>")
        self.Bond_length=self.inidata.get_string("<Bond_length>")
        self.CSA=self.inidata.get_string("<CSA>")
        self.Atom_1=self.inidata.get_string("<Atom_1>")
        self.Atom_2=self.inidata.get_string("<Atom_2>")


        #================== Software paths ================================
        # ModelFree search paths
        self.modelfree=self.inidata.get_string("<modelfree>") 
#------------------------ End of initial parameters loading --------------------
    
    
def make_fields_list(R1_fields_list, R2_fields_list, NOE_fields_list):
    """to make the same order of fields in mfinput and mfdata files.
    used by make_mfinput() and make_mfdata()"""
    fields_list=[]
    for i in R1_fields_list:
        if i not in fields_list: fields_list.append(i)
    for i in R2_fields_list:
        if i not in fields_list: fields_list.append(i)
    for i in NOE_fields_list:
        if i not in fields_list: fields_list.append(i)
    return fields_list[:]
#------------------------------------------------------------------    


















#----- Modelfree_Output ------------

# column positions in the 'mfout' file
RES, FITVAL, SIMERR, USED=0, 1, 5, 3
SSE,PERCENTILE,SSEcritical=1,0,1
RES_MODEL=1
 
class Modelfree_Output:
    """The class for reading of the 'mfout' file 
    - loads necessary parameters depending on the load mode:
        SINGLE_MODEL - loads model parameters
        FTEST   -   F values for F-test
        TENSOR_OPT  -   models' and tensor parameters
    - use:
        - mfout=Modelfree_Output("mfout",TENSOR_OPT,ini)
        - mfout.reslist[res_num][column_name]
            columns: S2S, S2Sstddev, S2F, S2Fstddev,TE, TEstddev,
                     REX, REXstddev,
                     GAMMA, GAMMAcritical, # SSE, SSE simulated at p
                     FVALUE, FVALUEcritical #  F-test value, F value simulated at p
        - mfout.tm          
        - mfout.Dratio      # tensor parameters
        - mfout.Theta
        - mfout.Phi
        
    """
    
    #------------------------------------------------------------------
    def __init__(self, fname,work_mode, initial_parameters):
        self.reslist=residues.Word_Dict() # main parameter storage
        self.mfout=keyfile.Keyword_File(os.curdir,fname)
        self.ini=initial_parameters
        
        # load current residue list
        linenum=self.mfout.get_linenum("_Title")+1
        while self.mfout[linenum][0].isdigit():
            self.reslist.add(int(self.mfout[linenum][0]),[0]*MFINPUT_COLUMNS)
            linenum=linenum+1

        self._load_mfout_params(work_mode, initial_parameters)
#------------------------------------------------------------------
        
    def _get_tensor(self):
        'load tm, Dratio, Theta and Phi into the attributes'
        linenum=self.mfout.get_linenum("_Diffusion_name")+1

        # load small portion
        self.tensor=residues.Word_Dict()
        for i in range(4):
            line=self.mfout[linenum+i]
            self.tensor.add(line[0],line[1:])
        
        # assign new values
        self.tm=self.tensor['tm'][1]
        self.Dratio=self.tensor['Dratio'][1]
        self.Theta=self.tensor['Theta'][1]
        self.Phi=self.tensor['Phi'][1]

        self.tm_err=self.tensor['tm'][5]
        self.Dratio_err=self.tensor['Dratio'][5]
        self.Theta_err=self.tensor['Theta'][5]
        self.Phi_err=self.tensor['Phi'][5]
        
#-------------------------------------------------------------

       
    def _get_models(self):
        'loading models for residues'
        global model_codes
        
        linenum=self.mfout.get_linenum("data_spin_parameters")+3
        
        for i in range(len(self.reslist)):
            list=self.mfout[linenum]
            resnum=int(self.mfout[linenum][RES])
            try:
                model=model_codes[list[RES_MODEL]]
            except KeyError:
                raise UserWarning, (
                "\n\n\a\a\aError!\nResidue %s has unknow model code '%s' in %s!"
                % (resnum, list[RES_MODEL], self.mfout.filename))

           self.reslist.set_word(resnum,MODEL_USED,model)
           linenum=linenum+1



#-------------------------------------------------------------
    def _get_S2s(self):
        'load S2 and its simulated standard deviation - it is S2S by definition'
        # it is identical to S2s data
        linenum=self.mfout.get_linenum("S2")+1
        while self.mfout[linenum][0]!="stop_":
            list=self.mfout[linenum]
            if list[USED]==`YES`:
                resnum=int(self.mfout[linenum][RES])
                self.reslist.set_word(resnum,S2S,list[FITVAL])
                self.reslist.set_word(resnum,S2Sstddev,list[SIMERR])
            linenum=linenum+1

    #------------------------------------------------
##   ! Duplicates S2 in mfout files     
##    def _get_S2s(self):
##        'load S2s and its simulated standard deviation'
##        linenum=self.mfout.get_linenum("S2s")+1
##        while self.mfout[linenum][0]!="stop_":
##            list=self.mfout[linenum]
##            if list[USED]==`YES`:
##                resnum=int(self.mfout[linenum][RES])
##                self.reslist.set_word(resnum,S2S,list[FITVAL])
##                self.reslist.set_word(resnum,S2Sstddev,list[SIMERR])
##            linenum=linenum+1
##
    #------------------------------------------------

        
    def _get_S2f(self):
        'load S2f and its simulated standard deviation'
        linenum=self.mfout.get_linenum("S2f")+1
        while self.mfout[linenum][0]!="stop_":
            list=self.mfout[linenum]
            if list[USED]==`YES`:
                resnum=int(self.mfout[linenum][RES])
                self.reslist.set_word(resnum,S2F,list[FITVAL])
                self.reslist.set_word(resnum,S2Fstddev,list[SIMERR])
            linenum=linenum+1

    #------------------------------------------------


        
    def _get_te(self):
        'load te and its simulated standard deviation'
        linenum=self.mfout.get_linenum("te")+1
        while self.mfout[linenum][0]!="stop_":
            list=self.mfout[linenum]
            if list[USED]==`YES`:
                resnum=int(self.mfout[linenum][RES])
                self.reslist.set_word(resnum,TE,list[FITVAL])
                self.reslist.set_word(resnum,TEstddev,list[SIMERR])
            linenum=linenum+1

    #------------------------------------------------

        
    def _get_Rex(self):
        'load Rex and its simulated standard deviation'
        linenum=self.mfout.get_linenum("Rex")+1
        while self.mfout[linenum][0]!="stop_":
            list=self.mfout[linenum]
            if list[USED]==`YES`:
                resnum=int(self.mfout[linenum][RES])
                self.reslist.set_word(resnum,REX,list[FITVAL])
                self.reslist.set_word(resnum,REXstddev,list[SIMERR])
            linenum=linenum+1

    #------------------------------------------------
    def _get_Gammas(self, Gamma_conf_level):
        'load Gamma and Gamma(simulated at Percentile)'
        linenum=self.mfout.get_linenum("data_sse")+5
        # need to count residues
        residues_left=len(self.reslist.get_keys())

        while residues_left:
            list=self.mfout[linenum]
            resnum=int(list[RES])
            # get experimental deviation
            Gamma=float(list[SSE])
            self.reslist.set_word(resnum,GAMMA,Gamma)
            
            # get calculated maximal deviation for 'Percentile'
            Percentile=int(float(Gamma_conf_level)*100) # convert 0.95 to int 95 for 'if'
            linenum=linenum+1
            list=self.mfout[linenum]
            while list[0]!="stop_":
                Perc_found=int(float(self.mfout[linenum][PERCENTILE])*100)
                if Perc_found==Percentile:
                    Gamma_critical=float(self.mfout[linenum][SSEcritical])
                    self.reslist.set_word(resnum,GAMMAcritical,Gamma_critical)    
                    break
                linenum=linenum+1
                list=self.mfout[linenum]
            else:
                raise UserWarning, (
                    "\n\nError! Percentile %s is not found in mfout file!\n-> %s"
                    % (Gamma_conf_level, self.mfout.filename))
            
            # go to the next residue
            while self.mfout[linenum][0]!="stop_": linenum=linenum+1
            linenum=linenum+1
            residues_left-=1

    #------------------------------------------------
    def _get_Ftest(self,F_test_level):
        'load F-test SSE and simulated SSE'
        linenum=self.mfout.get_linenum("data_F_dist")+5
        # need to count residues
        residues_left=len(self.reslist.get_keys())

        while residues_left:
            list=self.mfout[linenum]
            resnum=int(list[RES])
            # get experimental deviation
            Fvalue=float(list[SSE])
            self.reslist.set_word(resnum,FVALUE,Fvalue)
            
            # get calculated maximal deviation for 'Percentile'
            Percentile=int(float(F_test_level)*100) # convert 0.95 to int 95 for 'if'
            linenum=linenum+1
            list=self.mfout[linenum]
            while list[0]!="stop_":
                Perc_found=int(float(self.mfout[linenum][PERCENTILE])*100)
                if Perc_found==Percentile:
                    Fvalue_critical=float(self.mfout[linenum][SSEcritical])
                    self.reslist.set_word(resnum,FVALUEcritical,Fvalue_critical)    
                    break
                linenum=linenum+1
                list=self.mfout[linenum]
            else:
                raise UserWarning, (
                    "\n\nError! Percentile %s is not found in mfout file!\n-> %s"
                    % (F_test_level, self.mfout.filename))
            
            # go to the next residue
            while self.mfout[linenum][0]!="stop_": linenum=linenum+1
            linenum=linenum+1
            residues_left-=1

    #------------------------------------------------


    def _load_mfout_params(self,work_mode, Gamma_conf_level):
        'loads S2, S2s, S2f, te, Rex, SSE, Ftest and tensor depending on the mode'
        if work_mode==SINGLE_MODEL :
             self._get_S2s()
             self._get_S2f()
             self._get_te()
             self._get_Rex()
             self._get_Gammas(initial_parameters.Gamma_conf_level)
             #self._get_models() Unnecessary  + there is a bug in modelFree codes for models!
             return
            
        elif work_mode==FTEST:
            self._get_Ftest(initial_parameters.F_test_level)
            return
        
        elif work_mode==TENSOR_OPT:
            self._get_tensor()   
            self._get_S2s()
            self._get_S2f()
            self._get_te()
            self._get_Rex()
            self._get_Gammas(initial_parameters.Gamma_conf_level)
            #self._get_models()  Unnecessary  + there is a bug in modelFree codes for models!
#----------------------------------------------------------
    









def make_mfinput(fname, ini, fitted, kind):
    '(fname,Initial_Parameters, Fitted_Parameters, SINGLE/FTEST/TENSOR_OPT, USE_INIT/USE_FITTED'
    # USE_FITTED is the only necessary value. Set by default just to touch code less
    #---------- single model, tensor fixed, no F-test -----------
    if kind==SINGLE_MODEL:
        mfinput=(
"""optimization %(optimization_type)s
seed %(RND_generator_seed)s
search grid
diffusion %(tensor)s none
algorithm fix grid 1
simulations pred %(Simulations_number)s 0.0
selection none
sim_algorithm fix none 1""" % vars(ini))


    #---------- two models, tensor fixed, F-test -----------
    elif kind==FTEST:
        mfinput=(
"""optimization %(optimization_type)s
seed %(RND_generator_seed)s
search grid
diffusion %(tensor)s none
algorithm fix grid 1
simulations pred %(Simulations_number)s 0.0
selection ftest
sim_algorithm fix grid 1""" % vars(ini))


    #---------- tensor optimization -----------
    elif kind==TENSOR_OPT:

        mfinput=(
"""optimization %(optimization_type)s
seed %(RND_generator_seed)s
search %(axial_gridsearch)s
diffusion %(tensor)s none
algorithm powell %(axial_gridsearch)s 1
simulations pred %(Simulations_number)s 0.0
selection none
sim_algorithm powell none 1""" % vars(ini))
        
    else:
        raise UserWarning, "\n\n'make_mfinput() was called with wrong 'kind' parameter!"

    # compile the list of fields used
    fields_list=make_fields_list(ini.R1_fields_list,
                                 ini.R2_fields_list,
                                 ini.NOE_fields_list)
    
    mfinput+="\n\nfields %s" % len(fields_list)
    for i in fields_list: mfinput+=" "+i

    #----- start from fitted parameters ------------

    #get fitted values
    tm=fitted.Tm[FITTEDval]
    Dratio=fitted.Dratio[FITTEDval]        
    Theta=fitted.Theta[FITTEDval]
    Phi=fitted.Phi[FITTEDval]        

    mfinput+=("""\n
tm\t%s\t1\t2\t%s\t%s\t%s"""
              % (tm,ini.tmMin,ini.tmMax,ini.tmGrid))
    if ini.tensor=="axial":
        mfinput+=("""
Dratio\t%s\t1\t2\t%s\t%s\t%s    
Theta \t%s\t1\t2\t%s\t%s\t%s  
Phi   \t%s\t1\t2\t%s\t%s\t%s    
"""         % (Dratio,ini.DratioMin,ini.DratioMax,ini.DratioGrid,
           Theta,ini.ThetaMin,ini.ThetaMax,ini.ThetaGrid,
           Phi,ini.PhiMin,ini.PhiMax,ini.PhiGrid))



    open(fname,"w").writelines(mfinput)
# ----------- end of make_mfinput -----------------

 










#---------------------- mfdata -----------------------
def make_mfdata(init_pars, residue_list, outfname):
    'creates "mfdata" files'
    # create a list of fields
    fields_list=make_fields_list(init_pars.R1_fields_list,
                                 init_pars.R2_fields_list,
                                 init_pars.NOE_fields_list)
    
    # create datasets
    text=""
    for res in residue_list:
        text+="spin\t%s\n" % res
        for field in fields_list:
            text+="R1\t%s" % field
            if field not in init_pars.R1_data.keys():
                text+="\t0\t0\t0\n"
            else:
                text+="\t%s\t%s\t1\n" % (
                    init_pars.R1_data[field].get_resnum(res)[0],
                    init_pars.R1_data[field].get_resnum(res)[1])
              
            text+="R2\t%s" % field
            if field not in init_pars.R2_data.keys():
                text+="\t0\t0\t0\n"
            else:
                text+="\t%s\t%s\t1\n" % (
                    init_pars.R2_data[field].get_resnum(res)[0],
                    init_pars.R2_data[field].get_resnum(res)[1])
                
            text+="NOE\t%s" % field
            if field not in init_pars.NOE_data.keys():
                text+="\t0\t0\t0\n"
            else:
                text+="\t%s\t%s\t1\n" % (
                    init_pars.NOE_data[field].get_resnum(res)[0],
                    init_pars.NOE_data[field].get_resnum(res)[1])
        text+="\n"
    open(outfname,'w').write(text)
#-------------------- end of mfdata -----------------------------------


#---------------- mfpar file --------------------------------------------
def make_mfpar(init_params, res_list, fname):
    'creates mfpar file'
    mfpar=""
    for res in res_list:
        mfpar+=("spin  %d\nconstants\t%d\t%s\t%s\t%s\t%s\nvector\t%s  %s\n\n"
                % (res, res, init_params.Nucleus, init_params.Gamma, init_params.Bond_length,
                   init_params.CSA, init_params.Atom_1, init_params.Atom_2))

    open(fname,'w').write(mfpar)
#----------------------------------------------------------------------------


def create_model(model_num,ini):
    'makes a text chunk for the model entry in "mfmodel"'
    mod_num="M%d"   # to insert 1 or 2 later
    value="%s"      # to insert necessary parameter value later

    # adding lines for tloc and theta - unused parameters
    text= "%s\ttloc\t%s\t%s\t2\t%s\t%s\t%s\n" % (mod_num, value, model_matrix[model_num][0],
                                              ini.tloc[MINval],ini.tloc[MAXval],ini.tloc[GRIDsteps])
    text+="%s\tTheta\t%s\t%s\t2\t%s\t%s\t%s\n" % (mod_num, value, model_matrix[model_num][1],
                                              ini.theta[MINval],ini.theta[MAXval],ini.theta[GRIDsteps])
    # adding lines for parameters which matter
    text+="%s\tS2f\t%s\t%s\t2\t%s\t%s\t%s\n" % (mod_num, value, model_matrix[model_num][2],
                                                ini._model_pars[S2F][MINval],
                                                ini._model_pars[S2F][MAXval],
                                                ini._model_pars[S2F][GRIDsteps])

    text+="%s\tS2s\t%s\t%s\t2\t%s\t%s\t%s\n" % (mod_num, value, model_matrix[model_num][3],
                                                ini._model_pars[S2S][MINval],
                                                ini._model_pars[S2S][MAXval],
                                                ini._model_pars[S2S][GRIDsteps])

    text+="%s\tte\t%s\t%s\t2\t%s\t%s\t%s\n" % (mod_num, value, model_matrix[model_num][4],
                                                ini._model_pars[TE][MINval],
                                                ini._model_pars[TE][MAXval],
                                                ini._model_pars[TE][GRIDsteps])

    text+="%s\tRex\t%s\t%s\t2\t%s\t%s\t%s\n" % (mod_num, value, model_matrix[model_num][5],
                                                ini._model_pars[REX][MINval],
                                                ini._model_pars[REX][MAXval],
                                                ini._model_pars[REX][GRIDsteps])



    return text
#-----------------------------------------------------------------------------------------------------





#--------------  MFMODELS using INIT and FITTED---------------------- 




# if USE_FITTED - fitted values are used for M2,3 - S2S, M4,5 - S2S, te

def make_single_mfmodel_for_model_selection(model_num,
                                     fit_mode,      # use initial or fitted parameters
                                     res_list,      # list of residues to fit
                                     ini,           # Initial_Parameters
                                     fitted,        # Fitted_Parameters
                                     outfname
                                     ):
    'single model mfmodel file '
    text=""

    #------------ start fitting from initial parameters of models
    if fit_mode==USE_INIT:

        for res in res_list:
            text+="spin\t%d\n" % res
            model_text=create_model(model_num,ini)
            text+=model_text % (1, ini.tloc[INITval],
                                1, ini.theta[INITval],
                                1, ini._model_pars[S2F][INITval],
                                1, ini._model_pars[S2S][INITval],
                                1, ini._model_pars[TE][INITval],
                                1, ini._model_pars[REX][INITval]
                                )
            text+="\n"
            
        open(outfname,'w').write(text)

    #--------- start fitting using parameters of models from previous run        
    elif fit_mode==USE_FITTED:

        for resnum in res_list:
            # replace "init" values with results of fitting
            S2Svalue=ini._model_pars[S2S][INITval]
            TEvalue=ini._model_pars[TE][INITval]
            REXvalue=ini._model_pars[REX][INITval]
            S2Fvalue=ini._model_pars[S2F][INITval]

            
            if model_num in (1,2,3,4,5):
                # S2s
                S2Svalue=fitted.Residue_list[resnum][S2Scurr]

##            if model_num in (2,4,5):
##                # te
##                TEvalue=fitted.Residue_list[resnum][TEcurr]
##            
##            if model_num in (3,4):
##                # Rex
##                REXvalue=fitted.Residue_list[resnum][REXcurr]
##
##            if model_num in (5,):
##                # Rex
##                S2Fvalue=fitted.Residue_list[resnum][S2Fcurr]


            text+="spin\t%d\n" % resnum

            model_text=create_model(model_num,ini)
            
            text+=model_text % (1, ini.tloc[INITval],
                                1, ini.theta[INITval],
                                1, S2Fvalue,
                                1, S2Svalue,
                                1, TEvalue,
                                1, REXvalue
                                )
            text+="\n"
            
        open(outfname,'w').write(text)

    else:
        raise UserWarning, "\n\n\a\a\aWrong 'fit_mode' flag in make_mfmodel_for_model_selection()!"

    

        
#----------------------------------------------------------------------------------------------------



# if USE_FITTED - fitted values are used for M2,3 - S2S, M4,5 - S2S, te

def make_double_mfmodel_for_Ftest(model_1_num, model_2_num,
                                     fit_mode,      # use initial or fitted parameters
                                     res_list,      # list of residues to fit
                                     ini,           # Initial_Parameters
                                     fitted,        # Fitted_Parameters
                                     outfname
                                     ):
    'double model mfmodel file '
    text=""

    #------------ start fitting from initial parameters of models
    if fit_mode==USE_INIT:

        for res in res_list:
            text+="spin\t%d\n" % res

            model_text=create_model(model_1_num,ini)
            text+=model_text % (1, ini.tloc[INITval],
                                1, ini.theta[INITval],
                                1, ini._model_pars[S2F][INITval],
                                1, ini._model_pars[S2S][INITval],
                                1, ini._model_pars[TE][INITval],
                                1, ini._model_pars[REX][INITval]
                                )
            text+="\n"

            model_text=create_model(model_2_num,ini)
            text+=model_text % (2, ini.tloc[INITval],
                                2, ini.theta[INITval],
                                2, ini._model_pars[S2F][INITval],
                                2, ini._model_pars[S2S][INITval],
                                2, ini._model_pars[TE][INITval],
                                2, ini._model_pars[REX][INITval]
                                )
            text+="\n\n"
 
        open(outfname,'w').write(text)
           
   

    #--------- start fitting using parameters of models from previous run        
    elif fit_mode==USE_FITTED:

        for resnum in res_list:

            
            # replace "init" values with results of fitting
            S2Svalue=ini._model_pars[S2S][INITval]
            TEvalue=ini._model_pars[TE][INITval]
            REXvalue=ini._model_pars[REX][INITval]
            S2Fvalue=ini._model_pars[S2F][INITval]

            #---- 1st model ---------
            
            if model_1_num in (1,2,3,4,5):
                # S2s
                S2Svalue=fitted.Residue_list[resnum][S2Scurr]

##            if model_1_num in (2,4,5):
##                # te
##                TEvalue=fitted.Residue_list[resnum][TEcurr]
##            
##            if model_1_num in (3,4):
##                # Rex
##                REXvalue=fitted.Residue_list[resnum][REXcurr]
##
##            if model_1_num in (5,):
##                # Rex
##                S2Fvalue=fitted.Residue_list[resnum][S2Fcurr]


            text+="spin\t%d\n" % resnum

            model_text=create_model(model_1_num,ini)
            
            text+=model_text % (1, ini.tloc[INITval],
                                1, ini.theta[INITval],
                                1, S2Fvalue,
                                1, S2Svalue,
                                1, TEvalue,
                                1, REXvalue
                                )
            text+="\n"


            #---- 2nd model ---------
            
            if model_2_num in (1,2,3,4,5):
                # S2s
                S2Svalue=fitted.Residue_list[resnum][S2Scurr]

##            if model_2_num in (2,4,5):
##                # te
##                TEvalue=fitted.Residue_list[resnum][TEcurr]
##            
##            if model_2_num in (3,4):
##                # Rex
##                REXvalue=fitted.Residue_list[resnum][REXcurr]
##
##            if model_2_num in (5,):
##                # Rex
##                S2Fvalue=fitted.Residue_list[resnum][S2Fcurr]
##

            text+="spin\t%d\n" % resnum

            model_text=create_model(model_2_num,ini)
            
            text+=model_text % (2, ini.tloc[INITval],
                                2, ini.theta[INITval],
                                2, S2Fvalue,
                                2, S2Svalue,
                                2, TEvalue,
                                2, REXvalue
                                )
            text+="\n\n"


        open(outfname,'w').write(text)

    else:
        raise UserWarning, "\n\n\a\a\aWrong 'fit_mode' flag in make_mfmodel_for_model_selection()!"

    

        
#------------TENSOR OPTIMIZATION ---------------------------------------------------------------------------------------
# Need corresponding mf input!!!
# You should not include into the 'res_list' any unassigned residue!!!

def make_single_mfmodel_tensor_optimization(
                                     fit_mode,      # use initial or fitted parameters
                                     res_list,      # list of residues to fit
                                     ini,           # Initial_Parameters
                                     fitted,        # Fitted_Parameters
                                     outfname
                                     ):
    'single model mfmodel file '
    text=""

    #------------ start fitting from initial parameters of models
    if fit_mode==USE_INIT:

        for resnum in res_list:
            text+="spin\t%d\n" % resnum
            # takes models as assigned
            model_num=fitted.Residue_list[resnum][MODEL]
            if model_num==UNASSIGNED :
                raise UserWarning, (
                "\n\n\a\a\aError! You included residue '%s' into the tensor optimization list\n\
                but it is unassigned yet!!!\n Dataset: %s, requested mfmodel file: \n%s/%s"
                % (resnum,fitted.Residue_list.name, os.getcwd(), outfname))
            
            model_text=create_model(model_num,ini)
            text+=model_text % (1, ini.tloc[INITval],
                                1, ini.theta[INITval],
                                1, ini._model_pars[S2F][INITval],
                                1, ini._model_pars[S2S][INITval],
                                1, ini._model_pars[TE][INITval],
                                1, ini._model_pars[REX][INITval]
                                )
            text+="\n"
            
        open(outfname,'w').write(text)

    #--------- start fitting using parameters of models from previous run        
    elif fit_mode==USE_FITTED:

        for resnum in res_list:
            text+="spin\t%d\n" % resnum
            # replace "init" values with results of fitting
            S2Svalue=ini._model_pars[S2S][INITval]
            TEvalue=ini._model_pars[TE][INITval]
            REXvalue=ini._model_pars[REX][INITval]
            S2Fvalue=ini._model_pars[S2F][INITval]

            # takes models as assigned
            model_num=fitted.Residue_list[resnum][MODEL]
            if model_num==UNASSIGNED :
                raise UserWarning, (
                "\n\n\a\a\aError! You included residue '%s' into the tensor optimization list\n\
                but it is unassigned yet!!!\n Dataset: %s, requested mfmodel file: \n%s/%s"
                % (res,fitted.Residue_list.name, os.getcwd(),outfname))



            
            if model_num in (1,2,3,4,5):
                # S2s
                S2Svalue=fitted.Residue_list[resnum][S2Scurr]

##            if model_num in (2,4,5):
##                # te
##                TEvalue=fitted.Residue_list[resnum][TEcurr]
##            
##            if model_num in (3,4):
##                # Rex
##                REXvalue=fitted.Residue_list[resnum][REXcurr]
##
##            if model_num in (5,):
##                # Rex
##                S2Fvalue=fitted.Residue_list[resnum][S2Fcurr]
##


            model_text=create_model(model_num,ini)
            
            text+=model_text % (1, ini.tloc[INITval],
                                1, ini.theta[INITval],
                                1, S2Fvalue,
                                1, S2Svalue,
                                1, TEvalue,
                                1, REXvalue
                                )
            text+="\n"
            
        open(outfname,'w').write(text)

    else:
        raise UserWarning, "\n\n\a\a\aWrong 'fit_mode' flag in make_mfmodel_for_model_selection()!"

    

        
#------------------------------------------------





























class ModelFree:
    """Each object runs ModelFree in the pre-set mode, initial parameters and models.
    However, one can supply different residue lists"""
    # constants
    mfinput_file="mfinput"
    mfpar_file="mfpar"
    mfdata_file="mfdata"
    mfmodel_file="mfmodel"
    mfout_file="mfout"

    def __init__(self, initial_parameters, # Initial_Parameters instance
                        fitted_parameters, # Fitted_Parameters instance
                             mf_mode,  # SINGLE_MODEL,FTEST,TENSOR_OPT
                             fit_params_use_flag, # USE_INIT,USE_FITTED
                             *models  # N or N,M  :  a list of models
                             ):
        'prepares mf run'
        self.inipars=initial_parameters
        self.fitpars=fitted_parameters
        self.mode=mf_mode  
        self.fit_params_use=fit_params_use_flag 
        self.models=models 

    # parallel-ready : allows to split dataset with everything else the same
    def start(self, residue_list, run_name):
        'run ModelFree'
        # overwrite directory if exists
        if os.path.exists(run_name):
            os.system("rm -r "+run_name)
            #os.system("rm -r "+run_name+os.sep+self.inipars.pdb)
        os.mkdir(run_name)

        # use PDB file or not
        if self.inipars.tensor=="axial" :
            self.pdb_opt="-s "+os.pardir+os.sep+self.inipars.pdb  
        else: self.pdb_opt=""
        os.chdir(run_name)
 
        make_mfinput(self.mfinput_file,self.inipars, self.fitpars, self.mode)
        make_mfdata(self.inipars, residue_list, self.mfdata_file)
        make_mfpar(self.inipars,residue_list,self.mfpar_file)


        if self.mode==SINGLE_MODEL:
            make_single_mfmodel_for_model_selection(
                                                          self.models[0],
                                                          self.fit_params_use,
                                                          residue_list,
                                                          self.inipars,
                                                          self.fitpars,
                                                          self.mfmodel_file)
                                                          

        elif self.mode==FTEST:
            if len(self.models)<2:
                raise UserWarning, ("\n\n\a\a\aRun '%s' needs two models in the model list!\n\
                                    Instead, I have: %s" % (run_name, self.models))

            make_double_mfmodel_for_Ftest(self.models[0],
                                                       self.models[1],
                                                       self.fit_params_use,
                                                       residue_list,
                                                       self.inipars,
                                                       self.fitpars,
                                                       self.mfmodel_file)

        elif self.mode==TENSOR_OPT:
            make_single_mfmodel_tensor_optimization(
                                                       self.fit_params_use,
                                                       residue_list,
                                                       self.inipars,
                                                       self.fitpars,
                                                       self.mfmodel_file)



        else:
            raise UserWarning, "\n\n\a\a\aWrong mode passed to ModelFree()!: %s" % self.mode

        self._modelfree()
        os.chdir(os.pardir)
        os.system("mv %s.rotate %s/" % (self.inipars.pdb, run_name))

    def mfout(self, run_name):
        return Modelfree_Output(run_name+os.sep+self.mfout_file,
                                             self.mode, self.inipars)
#----------------------------------------------------------------------------------


    def _modelfree(self):
        'actually runs modelfree'
        #export_ext_data=" -e ext"
        export_ext_data=" "
        
        command=self.inipars.modelfree+' -i %s -d %s -p %s -m %s -o %s %s %s\n' % (
                                                        self.mfinput_file,
                                                        self.mfdata_file,
                                                        self.mfpar_file,
                                                        self.mfmodel_file,
                                                        self.mfout_file,
                                                        self.pdb_opt,
                                                        export_ext_data)
        out=os.popen(command,'r',0)
        print "ModelFree started",
        log=open("mflog","w")
        log.writelines(out.readlines())
        print "\nFinished"
        log.close()
        out.close()


#======================================================================
