#!/usr/bin/env python2
doc="""
=================== Separate interleaved data =====================

v.1 by Evgenii Kovrigin 12/18/03



This program processes and separates 2D data interleaved by
arraying a parameter.

The arrayed parameter should be listed FIRST in the 'array' variable
in 'dg'-window (parameter,phase).

--------------------

First, the 'fid' file is processed by fid.separate_fids.com' macro.
Make sure - settings are correct. 'np' and 'ni' are read and
adjusted automatically.

Then the program looks up in the PROCPAR file for the parameter array
values and separates fids based using nmrPipe function QMIX.
Fids are named by corresponding parameter value.



Usage: separate_fids.py  arrayed_parameter_name


To rename the fid files in some proper way - use 'rename_fids.py'

====================================================================
"""
import os, sys, string
import mylib

if len(sys.argv) !=2:
    print doc
    sys.exit()

parameter=sys.argv[1]
print """
=================== Separate interleaved data =====================
v.1 by Evgenii Kovrigin 12/18/03

Arrayed parameter: %s  """ % parameter

#------ reading procpar file -----
procpar=mylib.String_2_Words_2D(mylib.Load_Text_File('procpar'))



#---- find parameter location ------
param_line=mylib.Find_Keyword_First_in_Line(procpar,parameter)
if param_line==-1:
    print """
ERROR !!!!
Parameter '%s' not found in the supplied 'procpar' file!!!
""" % parameter
    sys.exit()



#---------------- various checks to make it fool-proof ----------
    
#--- check than parameter was listed FIRST in the 'array'
array_line=mylib.Find_Keyword_First_in_Line(procpar,'array')
array= string.split(
                string.split(
                            string.strip(procpar[array_line+1][1])
                            ,'"')[1]
                  ,',')
if not (parameter in array and parameter==array[0]):
    print """
ERROR!  Parameter '%s' must have been listed in the 'array' variable
        and be the FIRST but instead we have in the procpar file line %d:
        %s
This means that the way FIDs are interleaved is different from the one expected.
The file cannot be separated by the current implementation of the program.
""" % (parameter, array_line+1, procpar[array_line+1])
    sys.exit()

#-- check that only ONE additional parameter is arrayed ----
if len(array)!=2:
    print """
ERROR!  The experiment has to have '%s' arrayed together with 'phase'!
Instead 'array' parameter contains: %s""" %(parameter,array)
    sys.exit()

#------ END of checks --------------------------------------

#--- Load parameter values array 
expts=procpar[param_line+1][1:]
print '"%s" values found: '% parameter, expts
series_length=len(expts)


#---- Processing with var2pipe
np_line=mylib.Find_Keyword_First_in_Line(procpar,'np')
np=string.atoi(procpar[np_line+1][1])
print "np=",np

ni_line=mylib.Find_Keyword_First_in_Line(procpar,'ni')
ni=string.atoi(procpar[ni_line+1][1])
print "ni=",ni
#--- window functions of NMRPipe need this number be increased 
fake_ni=ni*series_length
fid_command="fid.separate_fids.com %s %s %s %s" % (
            np, np/2, fake_ni*2, fake_ni)
print '\nExecuting: ===> "%s"\n' % fid_command, 30*'-'
os.system(fid_command)
print 30*'-'



#--- Proceed to separation -----

for expt_num in range(series_length):
    #-- generate extration list
    extraction_string=""
    count=0
    for count in range(len(expts)):
        if count==expt_num:
            extraction_string+=" 1 0 0 1"
        else:
            extraction_string+=" 0 0 0 0"

    nmrPipe_command="nmrPipe -in ./test.fid | nmrPipe -fn QMIX -ic %d -oc 2 -cList %s -time -ov -out %s.fid" % (
                    series_length*2, extraction_string, expts[expt_num])
    print nmrPipe_command
    os.system(nmrPipe_command)
    print "%s point is done." % expts[expt_num]
    
os.system('rm test.fid')
                        
    
