#!/usr/bin/env python2

doc="""
#   'ProcessAll.py'
#
#   v.4.1  by Evgenii Kovrigin      06/23/2003
#
#   Batch processing of series of experiments
#  -----------------------------------------------------------

    Usage: ProcessAll.py ini.file [-ps] [-1 experiment_directory] [-color]
    
        
                                
        - ini.file =>  'ProcessAll.ini' file where all necessary parameters are set
        
        - check fid.com, nmrproc1.P1.com,  nmrproc1.P2.com and nmrproc2.com macros
          for specifics of your processing needs
        
    - I expect all subdirectories in 'data_path/' to contain fids!

    - For post-processing phase shifting use option: -ps

    - To do the phase shifting with individual set of phases for some datasets:
      create 'fid_dir_name.phases' text file in the 'data_path' directory
      and insert phases as
      <p0> xx
      <p1> yy
      

    - Processing of one selected experiment:
       options:  -1 experiment_directory

               
"""

import os
import sys
import string
import shutil

import mylib
import keyfile


#--- begin here ------

if len(sys.argv)<2:
	print doc
	sys.exit()

import colors
from colors import *
#-- to make it portable: use color only when requested explicitly
if '-color' in sys.argv: colors.COLOR=ON


#---- load parameters -----------

ini_filename=sys.argv[1]
inipars=keyfile.Keyword_File(os.curdir,ini_filename)   


#---- load parameters -----------
experiment_type=inipars.get_string("<Experiment_type>")

temperature=inipars.get_string("<temperature_label>")

xCar=inipars.get_string("<xCar>")

p0=inipars.get_string("<p0>")  # common phases

p1=inipars.get_string("<p1>")

protocols_to_run=inipars.get_integer("<protocols_num>")

path_to_fids=inipars.get_string("<path_to_fids>")

#------- macros' definitions -----------
var2pipe_batch_macro="fid.com"

protocols=[ "nmrproc1.P1.com",
            "nmrproc1.P2.com"
          ]

if protocols_to_run>len(protocols):
    print "\a\a\aError! Maximum number of protocols is set to %s!!!" % len(protocols)
    sys.exit()

second_part_macro="nmrproc2.com"

#-----  make a directory list --------
dirlist=mylib.list_dirs_only(path_to_fids)

#--- check mode of work requested
Yes=1
No=0
DoJustPhase=No
if '-ps' in sys.argv : DoJustPhase=Yes


if '-1' in sys.argv:  # take next word after '-1'
    dirname=sys.argv[sys.argv.index('-1')+1]
    dirlist=[dirname]


#-------------- begin processing -----------------
cprint(INV_RED,"-"*60)
print " BATCH PROCESSING OF %s " % experiment_type
cprint(INV_RED,"-"*60)
for experiment in dirlist:

    cprint(INV, "\n*********** Experiment %s ***********" % experiment)

    #--- copy fid and run 'fid.com' ---------------    
    if DoJustPhase==No :
        #---copy fid to working processing directory ------------
        shutil.copyfile(os.path.join(path_to_fids,experiment,"fid"), "fid")

        os.system(var2pipe_batch_macro+" "+xCar)

    # --- nmrPipe processing ---------------
    for x in range(protocols_to_run):
	num=x+1
	print clr(UNDERLINE, "--------- Protocol %s ---------"%num)
        

        dirname='Protocol_%d/' % num
        nmrPipe_output_name=os.path.join(dirname,experiment+".for_nmrproc2")
        for_sparky_name=os.path.join(dirname,
		experiment_type+"."+temperature+"."+experiment+".P%s"% num)
        
        if DoJustPhase==No :
            #--- create a directory --------
            if not os.path.isdir(dirname): os.mkdir(dirname)
         
            #--- run 'nmrproc1.Px.com' ------
            print "Processing ",experiment

            protocol_to_run=protocols[x]
            os.system(protocol_to_run)
            
            os.rename("result_for_nmrproc2", nmrPipe_output_name)

        #-- shift phase ----
        print "Shifting phase of ", experiment
        #-- check if phases for this experiment a given separately
        phase_fname=experiment+'.phases'
        if os.path.isfile(os.path.join(path_to_fids,phase_fname)):
		phase_file=keyfile.Keyword_File(path_to_fids,phase_fname)
		phase0=phase_file.get_string("<p0>")
		phase1=phase_file.get_string("<p1>")
		print "Use individual phases from '%s'\np0= %s\tp1= %s" % (phase_fname, phase0, phase1)
	else:
		phase0=p0
		phase1=p1
		print "Use common phases from '%s'" % ini_filename

        os.system(second_part_macro +   " %s %s %s %s"
              % (nmrPipe_output_name, phase0, phase1, for_sparky_name))

#--- clean up a bit ------
if DoJustPhase==No :
    os.system('rm for_nmrPipe')        
    os.system('rm fid')

mylib.beep()
cprint(BLINK, "\nDone...")
