#!/usr/bin/env python
import os
import sys
import string
import shutil

import mylib
import keyfile

doc="""
#   'ProcessNoeSeparately3.py'
#
#   v.3.3  by Evgenii Kovrigin           
#
#   Processing of NOE relaxation data by a number of protocols
#   with and without adding TWO parts - keep three files separately
#   to allow error analysis using duplicate parts
#
#   Needs the following macros in its working directory:
#   fid.com, nmrproc2.com and your different protocols named as
    nmrproc1.Px.com, where x- 1,2, ...

    'ProcessAllNoe.ini' - the configuration file located in the WORKING
                          directory

                        - allows to set temperature, xCAR, phases and
                          total number of fids in 'fid' file to process

                        - number of protocols to use in processing
                        

    Usage:
    
    ProcessNoeSeparately3.py [-ps]

        -ps  - for phase shifting w/o processing use option

01/06/04  - the issue  of yN fixed:  yT=ni*2 and yN=yT*2 now to allow SP work correctly   
	 
#--------------------------------------------------------------
"""

#---- configuration file name --------
ini_filename="ProcessAllNoe.ini"
params=keyfile.Ini_File(ini_filename)

# ---- get processing parameteres -----
num_protocols=params.get_integer("<num_protocols>")
   
#--- get fid paths -----------
path_to_fid1=params.get_string("<path_to_fid1>")
path_to_fid2=params.get_string("<path_to_fid2>")
if not os.path.isdir(path_to_fid1) or not os.path.isdir(path_to_fid2) :
    print "\a\a\a\nError! Check pathnames to fids!!!" 
    sys.exit()
dirlist=[path_to_fid1,path_to_fid2]




#---- processing macros ------------
fid_com="fid.com"                   # var2pipe
extract_com="extract.com"           # extract NOE and no Noe parts and add
second_part_macro="nmrproc2.com"    # phase shifting  

#-- generate a list of protocol names
protocol_mask="nmrproc1.P%d.com"
protocols_to_run=[]
for i in range(num_protocols):
    protocol_name=protocol_mask % (i+1)
    if not os.path.isfile(protocol_name):
        print "\a\a\a\n\nError! Macro '%s' not found !!!" % protocol_name
        sys.exit()
    protocols_to_run.append(protocol_name)





#-------------------------------------------------------------------------



#--- check mode of work requested
YES,NO=1,0   
DoJustPhase=NO
# ------ check command line arguments ----------
if '-ps' in sys.argv : DoJustPhase=YES

#------------ Var2Pipe processing block -----------------
if DoJustPhase==NO :

    #---copy files to working processing directory ------------
    shutil.copyfile(os.path.join(dirlist[0],"fid"),
                    os.path.join(os.getcwd(),"fid1"))
    shutil.copyfile(os.path.join(dirlist[1],"fid"),
                    os.path.join(os.getcwd(),"fid2"))

    #--- run fid.com with a correct -xCAR --------------
    yT=2*int(params.get_string("<-yT>"))

    yN=yT*2
    os.system(fid_com+" %s fid1 fid1_for_nmrPipe %s %s" % (params.get_string("<xCar>"),yN,yT))
    os.system(fid_com+" %s fid2 fid2_for_nmrPipe %s %s" % (params.get_string("<xCar>"),yN,yT))
    #--- extract noe and no-noe parts and add the together -----
    os.system(extract_com)



#------------ NmrPipe processing block -----------------
for x in range(num_protocols):
    num=x+1
    dirname='Protocol_%d' % num
    noe_name=dirname+"/noe.%s" % params.get_string("<label>")
    NOnoe_name=dirname+"/no_noe.%s" % params.get_string("<label>")
    
    if DoJustPhase==NO :
        #--- create a directory
        if not os.path.isdir(dirname): os.mkdir(dirname)
        
        #--- document processing scheme --------
        shutil.copyfile(fid_com, os.path.join(dirname,fid_com))
        shutil.copyfile(extract_com, os.path.join(dirname,extract_com))
        shutil.copyfile(protocols_to_run[x], os.path.join(dirname,protocols_to_run[x]))
        shutil.copyfile(second_part_macro, os.path.join(dirname,second_part_macro))

        #----------- nmrproc.com processing -----------
        print "\nProcessing ",noe_name
        os.system(protocols_to_run[x] + " noe.part1.fid  %s.part1.zero_phases" % noe_name)
        os.system(protocols_to_run[x] + " noe.part2.fid  %s.part2.zero_phases" % noe_name)
        os.system(protocols_to_run[x] + " noe.added.fid  %s.added.zero_phases" % noe_name)

        print "\nProcessing ",NOnoe_name
        os.system(protocols_to_run[x] + " no_noe.part1.fid  %s.part1.zero_phases" % NOnoe_name)
        os.system(protocols_to_run[x] + " no_noe.part2.fid  %s.part2.zero_phases" % NOnoe_name)
        os.system(protocols_to_run[x] + " no_noe.added.fid  %s.added.zero_phases" % NOnoe_name)

    # --------- phase shift -------------------
    print "\nShifting phase of ", noe_name
    noe_p0=params.get_string("<noe.p0>")
    noe_p1=params.get_string("<noe.p1>")
    
    os.system(second_part_macro +   " %s %s %s.part1.zero_phases %s.part1.ft2 %s.P%s.part1.sp"
          % (noe_p0, noe_p1, noe_name, noe_name, noe_name, num)
             )
    os.system(second_part_macro +   " %s %s %s.part2.zero_phases %s.part2.ft2 %s.P%s.part2.sp"
          % (noe_p0, noe_p1, noe_name, noe_name, noe_name, num)
             )
    os.system(second_part_macro +   " %s %s %s.added.zero_phases %s.added.ft2 %s.P%s.added.sp"
          % (noe_p0, noe_p1, noe_name, noe_name, noe_name, num)
             )
    
    print "\nShifting phase of ", NOnoe_name
    NOnoe_p0=params.get_string("<no_noe.p0>")
    NOnoe_p1=params.get_string("<no_noe.p1>")

    os.system(second_part_macro +   " %s %s %s.part1.zero_phases %s.part1.ft2 %s.P%s.part1.sp"
          % (NOnoe_p0, NOnoe_p1, NOnoe_name, NOnoe_name, NOnoe_name, num)
             )
    os.system(second_part_macro +   " %s %s %s.part2.zero_phases %s.part2.ft2 %s.P%s.part2.sp"
          % (NOnoe_p0, NOnoe_p1, NOnoe_name, NOnoe_name, NOnoe_name, num)
             )
    os.system(second_part_macro +   " %s %s %s.added.zero_phases %s.added.ft2 %s.P%s.added.sp"
          % (NOnoe_p0, NOnoe_p1, NOnoe_name, NOnoe_name, NOnoe_name, num)
             )


print "\a\a\a\nDone..."
