#!/usr/bin/env python
"""PymolLib.py
Classes for creating piece of a macro for PyMol to paint residues in
colors corresponding to the value of some parameter
Makes also XMGR file to check
"""

import os, sys
import mylib, residues, xmgr
from pymol import cmd



#============== Library of useful functions ==================



#------------ show_ball_and_stick -------------------------------
def show_ball_and_stick(residue_number):
    cmd.show('spheres','resi %d ' % residue_number)
    cmd.show('sticks','resi %d' % residue_number)




#----------------------- select_residue_list ---------------------------------
def select_residue_list(residue_number_list,selection_name='residue_list'):
    'select residues using a list of the NUMBERS'
    selection=''
    for residue_number in residue_number_list:
        selection+='%d+'%residue_number
    selection=selection[:-1] #--- strip the last plus sign
    print 'color_residue_list> Selecting: ', selection
    cmd.select(selection_name,'resi '+ selection)
    




    

#------------------ color_residue_by_elem -------------------------------
def color_residue_by_elem(residue_number):
    cmd.color('green', 'resi %d and elem C' % residue_number)
    cmd.color('white', 'resi %d and elem H' % residue_number)
    cmd.color('red', 'resi %d and elem O' % residue_number)
    cmd.color('blue', 'resi %d and elem N' % residue_number)


#----------------------- color_residue_list ---------------------------------
def color_residue_list(color,residue_number_list,selection_name='residue_list'):
    'color residues using a list of the NUMBERS'
    selection=''
    for residue_number in residue_number_list:
        selection+='%d+'%residue_number
    selection=selection[:-1] #--- strip the last plus sign
    print 'color_residue_list> Selecting: ', selection
    cmd.select(selection_name,'resi '+ selection)
    cmd.color(color, 'resi '+ selection)


#----------------------- color_Annn_residue_list ---------------------------
def color_Annn_residue_list(color,residue_name_list,selection_name='residue_list'):
  'Color normal residue name list of ANNN format'
  #-- convert to a list of residue numbers --
  residue_number_list=[]
  for resname in residue_name_list:
        print 'color_Annn_residue_list>  Coloring: ', resname
    #-- strip the first letter
	resnum=int(resname[1:])
	residue_number_list.append(resnum)
  #-- color this list now:
  color_residue_list(color,residue_number_list,selection_name)
  
#----------------------- color_Annn_N-H_residue_list ---------------------------
def color_AnnnN_H_residue_list(color, residue_name_list, selection_name='residue_list'):
  'Color normal residue name list of AxxxN-H format'
  #-- convert to a list of residue numbers --
  residue_number_list=[]
  for resname in residue_name_list:
        print 'color_AnnnN_H_residue_list>  Coloring: ', resname
    #-- strip the first letter
	temp=resname[1:]
	#-- strip the last three letters
	temp=temp[:-3]
	#-- convert into integer
	
	resnum=int(temp)
	#-- save
	residue_number_list.append(resnum)
  #-- color this list now:
  color_residue_list(color,residue_number_list,selection_name)
 

  

#-------------------- color_difference_list ----------------------------
def color_difference_list(color,residue_list1, residue_list2):

    for residue_number in residue_number_list:
        cmd.color(color, 'resi %d' % residue_number)


#-----------  protein_structure  -----------------
class protein_structure:
    """pdb file and its orientation"""
    def __init__(self,pdbname,orientation):
        """orientation - as copied from clipboard"""
        self.pdbname=pdbname
        self.orientation=orientation
        
    def display(self):
        cmd.load(self.pdbname)
        cmd.do(self.orientation)



#----------- precise_tubes ----------------------------
def precise_tubes(radius=0.3):
    'create PRECISE tubes (going correctly along the backbone!)'
    cmd.do("""

    
    set cartoon_loop_quality=30

    set cartoon_oval_quality=30

    set sphere_quality=30

    set stick_quality=30

    
    cartoon tube

    set cartoon_tube_radius, %f

    alter (all), ss='L'

    set cartoon_smooth_loops, 0

    set cartoon_loop_quality=30



    """ % radius)
    
           

#----------------------  text2list ----------------------------------------
def text2list (fname) :
    """converts text file containting 'int float' pairs into a list [int,float]"""
    if not os.path.isfile(fname): 
        print "\n\n\nError!\nCannot open '%s' to read!!!\n\n\a\a\a" % fname
	sys.exit()
    input=open(fname,'r')	
    L=input.readlines()
    V=[]
    for line in L:
        words=line.split()
        if len(words):
                    values=[int(float(words[0])),float(words[1])];
                    V.append(values)
    input.close()
    return V



#======== Color group of residues from list according to values of the parameter:
#         assign specific color for residues with value of parameter between minVal and maxVal.
#         ResidueNum_Value_data  must be array of vectors [ResNum,..., parameter-value, ... whatever]
#           parameter_column tells me where to take parameter values from.
def Color_Group_by_Parameter(ResidueNum_Value_data, parameter_column, minVal, maxVal, color, selection_name=''):

  #-- build a residue list to color
  residues_to_color=[]
  for line in ResidueNum_Value_data:
    if line[parameter_column] >= minVal and line[parameter_column] <= maxVal:
          residues_to_color.append(line[0])
          
  #-- submit it for coloraiton
  if selection_name=='': selection_name=color
  
  color_residue_list(color,residues_to_color,selection_name)








#=================	Color protein sequence by parameter =====
doc= """
 Color protein structure according to supplied 
 per-residue floating-point data values.

                                  - by J.Kempf, 25 May 2006

Credit to A. Kuechler & K. Rother for their 'color_protscale' python code 
and PyMOL plugin, which provides for coloration by a.a. type according to 
related physical params, e.g., hydrophobicity. That code provided guidance
for writing this site-specific coloration program.

05/27/08 -  modified by Evgenii Kovrigin. Now this program is to be called 
			from within another Pymol-python program, which is responsible 
			for building the structure model first.

 Color_Structure_by_Parameter(ResidueNum_Value_data,clrschm,thrshld)
 
	ResidueNum_Value_data - an array of pairs (int float) for residue number
							and value of the parameter

    clrschm - Choose color scheme & cartoon structure representation. Available 
			options: bluered, redblue, whtred, whtblue, whtgreen
			
    thrshld - threshold, i.e., value of data at or above which coloration
             set to maximum. Effectively disabled by choosing arb large value.
"""




# getColorMap():  Converts data to res-specific RGB values
#   seqscale - the residue-specific data to map 
#     scheme - various choices for coloration
#     thresh - data threshold, above which no color variation 

def getColorMap(seqscale,scheme,thresh):
   cMap = []
   temp = []
   [xMin,xMax,message] = [10000.0,-10000.0,'']
   for vals in seqscale:
     [ires,x] = vals
     if x >= thresh: x = thresh
     temp.append([ires,x])
     if x > xMax: xMax = x 
     if x < xMin: xMin = x 
   # set x range 0 to 1 and define corresponding color 
   for vals in temp:   
     done = 0
     [ires,x] = [vals[0],((vals[1]-xMin)/(xMax-xMin))]
     if scheme ==  'bluered':  [r,g,b,done] = [(2*x-x**2),0.0,(1-x**2), 1]
     if scheme ==  'redblue':  [r,g,b,done] = [(1-x**2),0.0,(2*x-x**2), 1]
     if scheme ==   'whtred':  [r,g,b,done] = [(0.8+0.2*(1-x)),(1-x),(1-x), 1]
     if scheme ==  'whtblue':  [r,g,b,done] = [(1-x),(1-x),(0.8+0.2*(1-x)), 1]
     if scheme == 'whtgreen':  [r,g,b,done] = [(1-x),(0.8+0.2*(1-x)),(1-x), 1]
     if scheme ==     'test':  [r,g,b,done] = [2*x-x**2,0.0,(1-x**2), 1]
     if   done == 0         :  message = ('** ERROR ** \nInvalid color scheme: ' + scheme + '\n** ERROR **')
     else:                     cMap.append([ires,r,g,b])
   return (cMap,message)


# Main Program:


# color the sequence according the value of the parameter given in the
# ResidueNum_Value_data as residue-number  parameter-value pairs
def Color_Structure_by_Parameter(ResidueNum_Value_data,clrschm,thrshld):

  clrMap,message = getColorMap(ResidueNum_Value_data,clrschm,thrshld)
  if message != '':  print '\n%s\n\n%s' % (message, doc)
  else:
    for resinfo in clrMap:
      [ires,r,g,b] = resinfo
      cmd.set_color('clr_%d'%ires,[r,g,b])
      cmd.color("clr_%d"%ires, 'resi %d'%ires)

    print '\nOptions: (coloration,threshold) = (%s, %s)' %(clrschm,thrshld)



#================= End of library ============================





