#!/usr/bin/env python2
doc="""
================== Calculation of delta (Rex) from 2 R1ro-R1 datasets =================
Rates should be ANNN x dx
========================================================="""

import os, sys, string,  glob 
import mylib, xmgr, residues, math
PI=3.1415926
def rad2deg(angle):
    'radian -> degrees'
    return 180*angle/PI

#============= INPUT YOUR PARAMETERS AND FILENAMES BELOW =================

#--- lt -list from sparky 
peak_position_list_fname="D121A.R1roR1.20C.minus2000.0ms.P1.sp.list"

wN='w1' #-- Nitrogen shift column name in the sparky lt-list


#-- source data filenames  [small_offset, large_offset]
##data_fnames=["D121A.R1roR1.20C.minus2000.P1.eh.rates","D121A.R1roR1.20C.plus5000.P1.eh.rates"]
##offsets=[-2000,5000] #-- Hz, [small, large] corresponding to data file names

##data_fnames=["D121A.R1roR1.20C.plus2000.P1.eh.rates","D121A.R1roR1.20C.plus5000.P1.eh.rates"]
##offsets=[2000,5000] #-- Hz, [small, large] corresponding to data file names


##data_fnames=["D121A.R1roR1.20C.plus2000.P1.eh.rates","D121A.R1roR1.20C.plus10000.P1.eh.rates"]
##offsets=[2000,10000] #-- Hz, [small, large] corresponding to data file names

##data_fnames=["D121A.R1roR1.20C.minus2000.P1.eh.rates","D121A.R1roR1.20C.plus10000.P1.eh.rates"]
##offsets=[-2000,10000] #-- Hz, [small, large] corresponding to data file names

data_fnames=["D121A.R1roR1.20C.minus2000.P1.eh.rates","D121A.R1roR1.20C.plus2000.P1.eh.rates"]
offsets=[-2000,2000] #-- Hz, [small, large] corresponding to data file names




dfrq2=60.7703329 #-- MHz,  Nitrogen Bo field
yCar=120         #-- ppm,  Nitrogen carrier frequency

B1pw90=300  #-- microsecond, B1 (spin lock) 90 degree pulse time


#============   END OF USER INPUT ==========================================

print "\n================== Calculation of delta (Rex) from 2 R1ro-R1 datasets ================="
print "Files:"
for fname in data_fnames:
    print fname
print "\nPeak positions are read from: '%s'\n" % peak_position_list_fname

print "\n---- field calculations -----\n"

w1=1/(4*B1pw90*1e-6) #-- Hz,  B1 field
print "Calculated B1 on-resonance Rabi frequency: %.2f Hz\n" % w1

we=map(lambda x: math.sqrt(w1*w1+x*x) ,offsets)
phi=map(lambda x: math.atan(w1/x), offsets)

print "Calculated effective fields and tilt angles (for carrier position):\n",'-'*40
for i in range(len(we)):
    print "(%d) we= %.2f Hz ;  phi= %.2f Deg"% (i, we[i], rad2deg(phi[i]))

print "\n---- per-residue calculations -----"
#-- load data ---
peak_positions=residues.Sparky_lt_Peaklist(peak_position_list_fname)
data1=residues.Annn_Num(data_fnames[0])
data2=residues.Annn_Num(data_fnames[1])

#-- check that all peak lists match: ---
residues.check_keys_match(data1,data2)
residues.check_keys_match(data1,peak_positions)

residue_list=peak_positions.sorted_keys()
print len(residue_list), " residues\n"

result=data1.make_empty_copy("Delta R1rho/sin(phi)**2")
for name in residue_list:
    delta_w1=(float(peak_positions[name][peak_positions.col_names().index(wN)])-yCar)*dfrq2   #-- Hz, offset of the peak from carrier

    omega_small=offsets[0]-delta_w1
    omega_large=offsets[1]-delta_w1
    
    phi_small=math.atan(w1/omega_small)
    phi_large=math.atan(w1/omega_large)
    #print name,data1[name]
    #print name,data2[name]
    #print "%s\toffsets: %2.f, %.2f Hz,\tphi: %.2f, %.2f\tdeg" % (
    #    name, omega_small, omega_large, rad2deg(phi_small), rad2deg(phi_large))

    
    deltaR1rho=data1[name][0]/(math.sin(phi_small)**2) - data2[name][0]/(math.sin(phi_large)**2)
    error_deltaR1rho=data1[name][1]/(math.sin(phi_small)**2) + data2[name][1]/(math.sin(phi_large)**2)
    result[name]=[deltaR1rho,error_deltaR1rho]
    #print "%s\t%.2f\t+-%.2f\n" %(name,deltaR1rho, error_deltaR1rho)
    
result.writeout("%.2f_%.2f.dat" % (offsets[0],offsets[1]))

resdata=xmgr.Dataset(result.to_xmgr(Xcol="ResNum",Ycol=0,dYcol=1))
resdata.solid_line()
graph=xmgr.Graph("R1rho-R1 data analysis","offsets: (%.2f) - (%.2f)" % (offsets[0],offsets[1]))
graph.add_dataset(resdata)                     

#--- graph settings ----
graph.set_xaxis("Residue number",0, 125)
graph.set_yaxis("Delta R1rho/sin(phi)**2",-50, 100)
graph.set_errorbars("BOTH", 0.1,2)

graphfname="%.2f_%.2f.xmgr" % (offsets[0],offsets[1])
graph.writeout(graphfname)
