#!/usr/bin/env python2
doc="""
================== Calculation of delta (Rex) from 2 R1ro datasets =================
Rates should be ANNN x dx
Use 'extract_common_residue_set.v2.py' utility to make lists the same
========================================================="""

import os, sys, string,  glob 
import mylib, xmgr, residues, math
PI=3.1415926
def rad2deg(angle):
    'radian -> degrees'
    return 180*angle/PI

#============= INPUT YOUR PARAMETERS AND FILENAMES BELOW =================

#--- lt -list from sparky 
peak_position_list_fname="D121A.R1roR1.20C.minus2000.0ms.P1.sp.list"

wN='w1' #-- Nitrogen shift column name in the sparky lt-list

#-- R1 dataset
R1data_fname="R1.ekn114.v600.diluted.D121A.20C.P1.eh.rates.extr"


data_fnames=["D121A.R1ro.20C.plus2000.P1.eh.rates.extr","D121A.R1ro.20C.plus5000.P1.truncated.eh.rates.extr"]
offsets=[2000,5000] #-- Hz, [small, large] corresponding to data file names

dfrq2=60.7703329 #-- MHz,  Nitrogen Bo field
yCar=120         #-- ppm,  Nitrogen carrier frequency

B1pw90=300  #-- microsecond, B1 (spin lock) 90 degree pulse time


#============   END OF USER INPUT ==========================================

print "\n================== Calculation of delta (Rex) from R1ro and R1 data ================="
print "R1rho(eff) files:"
for fname in data_fnames:
    print fname
print "\nPeak positions are read from: '%s'\n" % peak_position_list_fname
print "\nR1 data are read from: '%s'\n" % R1data_fname


print "\n---- field calculations -----\n"

w1=1/(4*B1pw90*1e-6) #-- Hz,  B1 field
print "Calculated B1 on-resonance Rabi frequency: %.2f Hz\n" % w1

we=map(lambda x: math.sqrt(w1*w1+x*x) ,offsets)
phi=map(lambda x: math.atan(w1/x), offsets)

print "Calculated effective fields and tilt angles (for carrier position):\n",'-'*40
for i in range(len(we)):
    print "(%d) we= %.2f Hz ;  phi= %.2f Deg"% (i, we[i], rad2deg(phi[i]))

print "\n---- per-residue calculations -----"
#-- load data ---
peak_positions=residues.Sparky_lt_Peaklist(peak_position_list_fname)
R1data=residues.Annn_Num(R1data_fname)
#-- offset names (for convenience)
SMALL=0
LARGE=1
data=[residues.Annn_Num(data_fnames[SMALL]),residues.Annn_Num(data_fnames[LARGE])]
#-- data columns in the files
VALUE=0
ERROR=1

#-- check that all peak lists match: ---
residues.check_keys_match(data[SMALL],data[LARGE])
residues.check_keys_match(data[SMALL],R1data)


residue_list=data[SMALL].sorted_keys()
print len(residue_list), " residues\n"

result=data[SMALL].make_empty_copy("Delta Rex from R1rho and R1")
for name in residue_list:
    if name not in peak_positions.get_keys():
        print "WARNING!  No assignment data data for '%s' was found! Skip it..." % name
        continue

    #--- OK assignment data exist ----
    delta_w1=(float(peak_positions[name][peak_positions.col_names().index(wN)])-yCar)*dfrq2   #-- Hz, offset of the peak from carrier

    omega_small=offsets[SMALL]-delta_w1
    omega_large=offsets[LARGE]-delta_w1
    
    phi_small=math.atan(w1/omega_small)
    phi_large=math.atan(w1/omega_large)

##    print name,data[SMALL][name]
##    print name,data[LARGE][name]
##    print name, R1data[name]
##    print "%s\toffsets: %2.f, %.2f Hz,\tphi: %.2f, %.2f\tdeg" % (
##        name, omega_small, omega_large, rad2deg(phi_small), rad2deg(phi_large))

    
    DR1rho=( data[SMALL][name][VALUE]/(math.sin(phi_small)**2) -
             data[LARGE][name][VALUE]/(math.sin(phi_large)**2) )
##    print DR1rho

    R1trig_factor=(1/(math.tan(phi_small)**2) - 1/(math.tan(phi_large)**2)) 
    
    deltaR1rho=DR1rho - R1data[name][VALUE]*R1trig_factor
    
    if abs(omega_small)>abs(omega_large):
        #-- reverse the difference sign!
        deltaR1rho=-deltaR1rho
        print "sign was reversed for ", name

    error_deltaR1rho=(abs(data[SMALL][name][ERROR]/(math.sin(phi_small)**2)) +
                      abs(data[LARGE][name][ERROR]/(math.sin(phi_large)**2)) +
                      abs(R1data[name][ERROR]*R1trig_factor))
    
    result[name]=[deltaR1rho,error_deltaR1rho]
    #print "%s\t%.2f\t+-%.2f\n" %(name,deltaR1rho, error_deltaR1rho)
    
result.writeout("%.2f_%.2f.dat" % (offsets[SMALL],offsets[LARGE]),header="OFF")

resdata=xmgr.Dataset(result.to_xmgr(Xcol="ResNum",Ycol=0,dYcol=1))
resdata.solid_line()
graph=xmgr.Graph("R1rho data analysis","offsets: (%.2f) - (%.2f)" % (offsets[SMALL],offsets[LARGE]))
graph.add_dataset(resdata)                     

#--- graph settings ----
graph.set_xaxis("Residue number",0, 125)
graph.set_yaxis("Delta Rex",-50, 100)
graph.set_errorbars("BOTH", 0.1,2)

graphfname="%.2f_%.2f.xmgr" % (offsets[SMALL],offsets[LARGE])
graph.writeout(graphfname)
