#!/usr/bin/env python
import os,sys,string
import residues, xmgr
import mylib
import keyfile
import copy
from math import *

doc= """
***************** R2R1 for tm estimate ************************

Calculation of R2/R1 ratios and BASED ON THAT selection of "non-exchanging" peaks
for estimating of correlation time and the tensor parameters 


Usage: 'R2R1_for_tmest.py trim_percent R2dataset R1dataset'

    - R2/R1 ratio is evaluated
    - the ratios are trimmed and their mean and rmsd are calculated
    - XMGR file is generated
    - R2R1ratios.dat - the file for R2R1 diffusion

**************************************************
""" 

if len(sys.argv)!=4 :
    print doc
    sys.exit()


r2_data_fname=sys.argv[2]
r1_data_fname=sys.argv[3]
trim_percent=int(sys.argv[1])

VALUE,ERROR = 0,1
SILENT,VERBOSE=0,1
R1=residues.Annn_Num(r1_data_fname,mode=VERBOSE,header_lines=1)
R2=residues.Annn_Num(r2_data_fname,mode=VERBOSE,header_lines=1)


#--- check length of lists submitted -------------
N=len(R2)
if len(R1)!=N:
    raise UserWarning, "You residue lists are different length!"

print "# %s / %s (%s entries)" % (sys.argv[2],sys.argv[3],N)


#--- calculate ratios and errors
R2R1=R1.make_empty_copy("R2R1")


for res in R2.get_keys():

    r1val=R1[res][VALUE]
    r2val=R2[res][VALUE]
    
    err1=R1[res][ERROR]
    err2=R2[res][ERROR]

    #-- calculations ---
    ratio=r2val/r1val

    #-- error propagation ---
    sq_ratio=ratio**2
    sq_err1=(err1/r1val)**2
    sq_err2=(err2/r2val)**2
    error=sqrt(sq_ratio*(sq_err1+sq_err2))

    #--- store it -------
    R2R1[res]=[ratio,error]


R2R1trimmed = R2R1.trim_data(VALUE, trim_percent)

r2r1mean, r2r1sigma = R2R1trimmed.statistics(VALUE)

#-- output for R2R1diffusion -----

outfile=mylib.open_file("R2R1ratios.dat", 'w')
outfile.write("# %s / %s\n# Trimmed %s%% dataset: %s entries\n"
              % ( sys.argv[2],sys.argv[3],sys.argv[1],len(R2R1trimmed)))

outfile.write("# Average values: R2/R1 = %.4f\t err=%.4f\n" % (r2r1mean,r2r1sigma))

for res in R2R1trimmed.sorted_keys():
    outfile.write("%s\t%.4f\t%.4f\n" % (res[1:], R2R1trimmed[res][VALUE], R2R1trimmed[res][ERROR]))

#------------- presentation -------------

full=xmgr.Dataset(R2R1.to_xmgr("ResNum",VALUE), "all")
full.circle()
full.empty_sym()
full.symbol_size(2.2)


trimmed=xmgr.Dataset(R2R1trimmed.to_xmgr("ResNum",VALUE), "trimmed at %s%%" % trim_percent)
trimmed.circle()
trimmed.filled_sym()
trimmed.symbol_size(1)



graph=xmgr.Graph("R2/R1","Trimmed mean: %.4f,  Sigma   %.4f" %
                 (r2r1mean, r2r1sigma) )
graph.add_dataset(full)
graph.add_dataset(trimmed)
graph.set_xaxis("Residue number", 0, 125)
graph.set_yaxis("value", 0.3, 1)
graph.writeout("R2R1.xmgr")


                      



