#!/usr/bin/env python
description="""
#----------------------------------------
# 'R2R1ratio.py'
#
# Prepares a data file for 'r2/r1_diffusion'.
# Needs two files with [residue R Err] structure and data
# for NON-EXCHANGING residues.
#
# Usage: R2R1ratio.py  R2 R1 > outfile
#
---------------------------------------------
"""

import sys,os,string
from math import *
from keyfile import *

if len(sys.argv)<3 :
    print description
    sys.exit()


R2=Keyword_File(os.curdir,sys.argv[1])
R1=Keyword_File(os.curdir,sys.argv[2])

RESIDUE,VALUE,ERROR = 0,1,2       # columns in datasets

def sigma_sqrt(list) :
    'list of x -> x(mean), sigma_x'
    sum=0
    sumsquare=0
    N=len(list)
    for i in range(N) :
        x=list[i]
        sum=sum+x
        sumsquare=sumsquare+x*x
    return sum/N, sqrt((sumsquare-sum*sum/N)/(N-1))


N=len(R2)
if len(R1)!=N:
    raise UserWarning, "You residue lists are different length!"

print "# %s / %s (%s entries)" % (sys.argv[1],sys.argv[2],N)

ratios=[]
for resline in R2:
    # get data
    residue=resline[RESIDUE]

    resnum=residue[1:]

    r2=float(R2.get_all_line(residue)[VALUE])

    r1=float(R1.get_all_line(residue)[VALUE])

    err2=float(R2.get_all_line(residue)[ERROR])

    err1=float(R1.get_all_line(residue)[ERROR])

    # calculations        
    ratio=r2/r1

    ratios.append(ratio)

    sq_ratio=ratio**2
    sq_err1=(err1/r1)**2
    sq_err2=(err2/r2)**2
    error=sqrt(sq_ratio*(sq_err1+sq_err2))
    # output
    print "%s\t%.4f\t%.4f" % (resnum, ratio, error)
    
x,sigma=sigma_sqrt(ratios)
print "#Average values: R2/R1 = %.4f\t err=%.4f" % (x,sigma)
