#!/usr/bin/env python


#======= USER INPUT ========

#-- no PDB extension here !
pdbname='5P21'

###-- data from 500 MHz
dataset_name='EKN26.Ras_GNP.CP.500'
###-- FitResult.txt from Matlab RexFitting program
dataname="/Users/kovrigin/Documents/Workspace/Data/Data.XI/EKN26.rcCPMG.Ras_GNP.10mM_NaCl.500/data/rcCPMG/added/sp/Sparky/es_list.assigned.txt.full_parsed/FitResults.txt" 
##

#-- data from 600 MHz
#dataset_name='EKN24.Ras_GNP.CP.600'
#-- FitResult.txt from Matlab RexFitting program
#dataname="/Users/kovrigin/Documents/Workspace/Data/Data.XI/EKN24.rcCPMG.Ras_GNP.10mM_NaCl.600/data/rcCPMG/added/sp/Sparky/es_list.assigned.txt.full_parsed/FitResults.txt" 





overlaps=[26,85, 131, 135] #-- integer numbers for overlapped residues
dataname_nonempty_header_lines=1
#-- column order (1,2,3...)
Rex_col=4
RexErr_col=5

#======= END OF USER INPUT ===========




import os,sys, string
import residues, mylib 
from pymol import cmd
import PymolLib 




#--- clean up workspace
cmd.delete("all")
cmd.remove('all')





cmd.load(pdbname + '.pdb')
cmd.hide('all')




#--- load data file
data=residues.Word_Dict(dataname,mode=1, nonempty_header_lines=dataname_nonempty_header_lines)





#------- PROCESS REX DATA
#------- 1. convert residue names to integer indices and save residue names
#---     2. convert data to float
#---     3. only save ones with significant Rex
print "Find significant Rex values:"

positiveRex_data=residues.Word_Dict()
positiveRex_data.set_name('ResNum   R2  dR2   Rex   dRex  ResLabel')
negativeRex_data=residues.Word_Dict()
negativeRex_data.set_name('ResNum   R2  dR2   Rex   dRex  ResLabel')

all_data=residues.Word_Dict()

data.convert_values_to_float()


for key in data.get_keys():

  resnum=int(key[1:-3])	#--- extract number from standard assignment lagel
  if resnum in overlaps:
      print "Dropping %s" % key
      continue

  #-- convert index label into number
  all_data.add(resnum,data[key])
  all_data.append_word(resnum,key)



  #--- check for significant Rex
  Rex=data[key][Rex_col-2]
  RexErr=data[key][RexErr_col-2]
 
  if abs(Rex)>RexErr:
    print "%s  Rex= %.2f/s +- %.2f " % (key, Rex, RexErr)

    #-collect significant Rex values
    if Rex>0:
        positiveRex_data.add(resnum,data[key])
        positiveRex_data.append_word(resnum,key)
    else:
        negativeRex_data.add(resnum,data[key])
        negativeRex_data.append_word(resnum,key)






#--- show the structure
#PymolLib.precise_tubes()
cmd.do('show cartoon')
cmd.color('grey')
cmd.show('sticks','resi 167')
cmd.color('lightteal','resi 167')
cmd.show('spheres','resi 168')
cmd.color('sand','resi 168')


#------- color assigned residues
PymolLib.color_residue_list('green',all_data.get_keys(),'all_assigned')

#------ color positive Rex
PymolLib.color_residue_list('red',positiveRex_data.get_keys(),'positive_Rex')

#------ color negative (artifactual) Rex
PymolLib.color_residue_list('blue',negativeRex_data.get_keys(),'negative_Rex')







#--- color all residues with positive Rex > error


cmd.do("""
set_view (\
    -0.340855271,   -0.393157929,    0.853962004,\
     0.274187803,   -0.910435736,   -0.309720635,\
     0.899245083,    0.128573686,    0.418123662,\
     0.000000000,    0.000000000, -119.854667664,\
     5.957344055,   25.755182266,   12.153440475,\
    85.511306763,  154.198013306,    0.000000000 )



""")
  
cmd.do('deselect')

#---- save the data
cmd.do('png %s' % dataset_name+'.Rex.png')
positiveRex_data.writeout(dataset_name+'.positiveRex.txt')
negativeRex_data.writeout(dataset_name+'.negativeRex.txt')








