#!/usr/bin/env python
doc="""
**********************************************************************************

Making a macro for molmol with 1-S2 (minimum - grey or sth) on the sausage diagram

----------------------------------------------------------------------------------

Usage:

S2_to_molmol.py [-c] -d data_file.pickled [-Smin number]

    where:
            data_file.pickled   - the output of MFselect.py in pickle format
    
            -c                  - if you want a colorized output
            
            -min number  -max number
                                - if you want to set minimal and maximal 1-S2 value
                                  rather than use automatic scaling


-----------------------------------------------------------------------------------
Note:   internally residues are referenced only by the number nnn and some fake
        name Xnnn (X44 etc).

***********************************************************************************
"""
import os, sys, string, pickle
import MFselect_lib_2
import mylib, textfile, residues
from MFselect_constants import *
import molmol
import xmgr
import colors
from colors import *
#-- to make it portable: use color only when requested explicitly
if '-c' in sys.argv: colors.COLOR=ON

#------------------------------------------------------------
def remove_unassigned(data):
    'delete entries which were not assigned by ModelFree'
    to_remove=[]
    for res in data.Residue_list.get_keys():
        if data.Residue_list[res][MODEL]==UNASSIGNED: to_remove.append(res)

    print data.Residue_list.name, ": removing unassigned: ", to_remove
    for res in to_remove:
        del data.Residue_list[res]






#############################################################################


#--- check command line for minimal parameters ----
if (len(sys.argv)<4 or ('-d' not in sys.argv) or (sys.argv.index('-d')==(len(sys.argv)-1))
    ):
    print doc
    sys.exit()

cprint(YELLOW, "\n---------  Mapping S2 onto the protein structure -------------\n")
    
#--- input filename ----
fname_index=sys.argv.index('-d')
fname=sys.argv[fname_index+1]
cprint(GREEN,"Input filename: '%s'" % fname)

#--- Clipping level on S2: everything less than that is set to S2_min value
if '-min' in sys.argv and '-max' in sys.argv:
    ind=sys.argv.index('-min')+1
    S2_min=float(sys.argv[ind])
    ind=sys.argv.index('-max')+1
    S2_max=float(sys.argv[ind])
    scaling="manual"
    cprint(GREEN,"1-S2 scaling levels (min, max): %s, %s" % (S2_min, S2_max))
else:
    scaling="auto"
    S2_min=S2_max="not_set"  
    cprint(GREEN,"1-S2 scaling - automatic")


#-- read data ---
data=pickle.load(mylib.open_file(fname,'r'))
data.Residue_list.set_name(fname)

#--- remove residues without model assignment by ModelFree 
remove_unassigned(data)

#-- get S2: create a new residue list ---
S2_reslist=residues.Annn_Num()
resnum_list=data.Residue_list.get_keys()
for resnum in resnum_list:
    #-- make a fake residue name ---
    resname="X"+str(resnum)
    #-- get S2 and append to final list ---
    s2=float(data.Residue_list[resnum][S2Scurr])
    
#    cprint(BLUE, "%s  -   %s" % (resname, s2))

   
    S2_reslist.add(resname,[1-s2])



#-- generate a MolMol sub-macro --

import molmol

Rnase=molmol.Protein(124, S2_reslist)
Rnase.set_assigned_colors(molmol.Yellow, molmol.Red)
Rnase.set_unassigned_color(molmol.Color(0.5,0.5,0.5))
Rnase.set_background_color(molmol.Black)
Rnase.colorize_by_parameter(parameter_col_num=0, scaling=scaling, min=S2_min, max=S2_max)
#Rnase.highlight(["X66","X119"],molmol.Red)
#-- display 1-S2 ---
Rnase.write_macro("molmol.mac")
