#!/usr/bin/env python
import string, mylib, glob, os, sys
doc="""

=========== add_3D.py ===================
by Evgenii Kovrigin   2/29/08
Medical College of Wisconsin
v.1.3

This program adds 3D datasets together. It assumes that they are identical
and adds plane-by-plane to improve signal/noise ratio. I assume you had
processed 'ser' files with identical macros and .ft2 files are located in
data/ folder of each experiment.

Bugs and fixes:
1/11/08 - added/ - folder creation added
          other warnings added
2/29/08 - clean-up option added
4/11/08 - now only adding *ft2 files (filter added)
5/12/08 - now clean up option removing only ft2 files, not the folder itself
2/17/09 - multiple processing protocols supported 
============================================================================



"""

#===== USER INPUT SECTION =====
#-- new dataset name
dataname='ekn59.KRasGNP.added.20C.TCI.500'

#protocol='LowRes'
protocol='HighRes'
#protocol='' # if you are not using alternative processing schemes



#-- put in paths to your multiple experiments you wish to add together
#-- Paths must be relative to the current folder.
data_paths=[
    '3',
    '5',
    '7',
    '9'
    ]

#-- output folder 
outdir="added"



#-- default folder with .ft2 files in each experiment
#ft2_folder=''       # for addition of HSQC datasets
ft2_folder='data'  # for addition of rcCPMG datasets

#-- folder for Sparky .sp files
sp_folder='sp'


#-- Mode: remove ft2 files from original and added folders?
cleanup_mode='no'

#===== END OF USER INPUT SECTION =====

# make protocol-specific folder
if protocol!='':
    outdir+='.'+protocol
    ft2_folder+='.'+protocol
    


#--- make  output directory
if not os.path.isdir(outdir): os.mkdir(outdir)
if not os.path.isdir(os.path.join(outdir,ft2_folder)): os.mkdir(os.path.join(outdir,ft2_folder))
if not os.path.isdir(os.path.join(outdir,sp_folder)): os.mkdir(os.path.join(outdir,sp_folder))


planes_list=map(os.path.basename,glob.glob(os.path.join(data_paths[0],ft2_folder,"*.ft2")))

if len(planes_list)==0:
  print "\nERROR!\nNo data files are found in '%s'!\n\n" % os.path.join(data_paths[0],ft2_folder)
  sys.exit()

for plane in planes_list:
    print "Reading plane: '%s'" % plane

    #-- copy first dataset to output folder
    os.system("cp %s %s" % (os.path.join(data_paths[0],ft2_folder,plane),os.path.join(outdir,ft2_folder,plane)))
    print "Adding: '%s'" % data_paths[0]
   
    for dataset in data_paths[1:]:
        print "Adding: '%s'" % dataset

        temp_file='temp.ft2'
        original_file=os.path.join(outdir,ft2_folder,plane)
        file_to_add=os.path.join(dataset,ft2_folder,plane)

        os.system("addNMR -in1 %s -in2 %s -add -out %s" % (original_file,file_to_add,temp_file))
        os.system("mv %s %s" % (temp_file,original_file))

    #-- make Sparky file
    sp_fname=plane.split('.')[0]+'.'+ dataname+'.sp'
    os.system('pipe2ucsf %s %s' % (os.path.join(outdir,ft2_folder,plane),
                              os.path.join(outdir,sp_folder,sp_fname)) )    

print "\nDone. Added files are in '%s/'." % os.path.join(outdir,ft2_folder)
print "Sparky files are in '%s/'.\n" % os.path.join(outdir,sp_folder)


#-- remove ft2 files
if cleanup_mode=='yes':
    print "\nRemoving nmrPipe files..."
    #-- added ft2
    os.system('rm  %s/*ft2'% os.path.join(outdir,ft2_folder));
    #-- original ft2
    for dataset in data_paths:
        print "Cleaning: '%s'" % dataset
        os.system("rm  %s/*ft2" % os.path.join(dataset,ft2_folder))

    print "Done."




