#!/usr/bin/env python
import mylib, residues 
import os, string, sys


doc="""
===== Add_Peak_Labels =======
v.2
by Evgenii Kovrigin, 1/9/07
Medical College of Wisconsin


This program addresses a problem that if you pick peaks in Sparky they come
up as ?-? labels and will not work with many of my extensions. This program adds
peak labels to ?-? peaks in Sparky save files to make processing of peak intensities possible.

Usage:
    1. Peak peaks in Sparky and save peak list from 'lt' with
       peak label, frequencies and data heights.
       
    2. add_peak_labels.py peak_list_fname save_template_fname out_save_fname
       
         where
            peak_list_fname - 'lt' list containing ?-? assignments, frequencies and heights
            save_template_fname - any .save file from Sparky project to be used as a template
            out_save_fname - output .save file name

Bugs and fixes:

2/10/09 - command line usage added

"""

if len(sys.argv)<4:
    print doc
    sys.exit()
    

#====== USER INPUT SECTION =======

#-- 'lt' list containing ?-? assignments
peak_list_fname=sys.argv[1]

#-- template .save file from Sparky
save_template_fname=sys.argv[2]

#--- output .save file name
out_save_fname=sys.argv[3]

#====== END USER INPUT SECTION =======




print "Label automatically picked peaks"
#------ PROCESS 'lt' LIST ------
#-- get a peak list
lt_list=mylib.Load_Text_File(peak_list_fname)

#-- get  header and check for data height
header=lt_list[0].split()
if len(header)!=5 or header[-1]!='Height':
    print "\n\n\nERROR! You should save 'lt' list with peak label, frequencies and data height columns!"
    print "Instead I am getting: \n%s\n\n\n" % lt_list[0]
    sys.exit()
    
#-- get rid of empty lines and split words
final_list=[]
for line in lt_list[1:]:
    if string.strip(line)=='': continue
    final_list.append(line.split())



#------- PROCESS '.save' TEMPLATE ----
start_of_peaks_keyword='<ornament>\n'
end_of_peaks_keyword='<end ornament>\n'

#-- get a .save file
template_save_file=mylib.Load_Text_File(save_template_fname)

#--- split into two parts to get rid of existing peaks
save_beginning=[]
save_ending=[]


for line in template_save_file:
        if line==start_of_peaks_keyword:
            #-- first part finished. Get out
            break
        save_beginning.append(line)


for line in template_save_file:
        save_ending.append(line)
        if line==end_of_peaks_keyword:
            #-- reinitialize array
            save_ending=[]
            

#------ GENERATE NEW SPARKY .save FILE ------


#--- generate ornament entries for all the peaks
label_shift_w1=-0.5
label_shift_w2=0.05

peak_template="""type peak
color 0 2 0 white
flags 0 0 0 0
id %d
pos %.3f %.3f
height 0.000 %s
rs |x%d|A| |x%d|X|
[
type label
color 2 0 0 white
flags 0 0 0 1
mode 2
pos  %.3f %.3f
label x%dA-X
xy %.3f,%.3f  %.3f,%.3f
]
"""
    
num_peaks=len(final_list)

peaks=[]

for count in range(num_peaks):
    peak_number=count+1
    peak_line=final_list[count]
    w1=float(peak_line[1])
    w2=float(peak_line[2])
    H=peak_line[3]
    peak=peak_template % (peak_number, w1, w2, H, peak_number, peak_number,
                           w1, w2, peak_number,
                          w1,w2+label_shift_w2, w2, w1+label_shift_w1)

    peaks.append(peak)

print "Number of peaks processed: %d" % num_peaks

#-- save the final file
outfile=open(out_save_fname,'w')

#-----first part
for line in save_beginning:
    outfile.write(line)


#----- peaks
outfile.write(start_of_peaks_keyword)
for line in peaks:
    outfile.write(line)
outfile.write(end_of_peaks_keyword)        


#-----last part
for line in save_ending:
    outfile.write(line)

outfile.close()
print "Output .save Sparky file '%s' written." % out_save_fname
