#!/usr/bin/env python
import string, mylib, glob, os, sys
doc="""

=========== analyze_interleaved_replicates.py ===================
by Evgenii Kovrigin   6/6/08
Medical College of Wisconsin
v.1.1

This program helps to analyze replicate points within pseudo-3D datasets.
To find listing of experiments it uses
'lists/freq_ek_rccpmg.txt' file produced by lists/ek_rccpmg.py at the time of
experiment set-up on a spectrometer.

It will report on-screen indices of specific frequencies

Usage:   analyze_interleaved_replicates.py path1 [path2 [path3...]]]
             where 
                path - is path to raw spectral data

============================================================================



"""

if len(sys.argv) < 2:
  print doc
  sys.exit()
  
  

#===== USER INPUT SECTION =====

# (edited) file with a list of effective frequencies of experiments
frequency_fname='freq_ek_rccpmg.txt'  
ttotal_fname='ttotal_ek_rccpmg.txt'

#-- folder with ek_rccpmg.py output (frequency lists)
lists_folder='lists'

#-- put in paths to experiments you wish to analyze
#-- Paths must be relative to the current folder.
data_paths=sys.argv[1:]

#===== END OF USER INPUT SECTION =====




#--- go through all repeated data sets one by one
#--- DO I NEED MULTIPLE DATASETS?  THEY ARE GOING TO BE ALL IDENTICAL!!!
for current_dataset in range(len(data_paths)):

        print "\nCurrent dataset: %s/" % data_paths[current_dataset]

        #-- read frequency list from specific data folder and convert to integer array
        freqlist_fname=os.path.join(data_paths[current_dataset],lists_folder,frequency_fname)
        frequency_list=mylib.Load_Text_File_NonEmpty_Lines (freqlist_fname)

        #-- total time list
        ttotal_file=os.path.join(data_paths[current_dataset],lists_folder,ttotal_fname)
        ttotal_list=mylib.Load_Text_File_NonEmpty_Lines (ttotal_file)

        #-- go through frequency list, collect indices of experiments to average, by frequency
        experiments_to_add={}

        #--- collect total time of the experiments at specific freqencies freq:ttotal
        frequency_ttotal={}
        
        for current_expt in range(len(frequency_list)):
            line=frequency_list[current_expt]
            ttotal=ttotal_list[current_expt]

            #-- recognize lines to be removed by presence of THE SECOND WORD in the line
            words=string.split(line)
            if len(words)>1: continue

            #-- convert word to int
            cur_frequency=int(float(words[0]))
            if cur_frequency not in experiments_to_add.keys():
                experiments_to_add[cur_frequency]=[current_expt]
                #-- collect ttotal
                frequency_ttotal[cur_frequency]=float(ttotal)
            else:
                experiments_to_add[cur_frequency].append(current_expt)


        #--- REPORT FREQUENCY INDICES
        for current_frequency in experiments_to_add.keys():

         
            #-- get experiment indices
            planes=experiments_to_add[current_frequency]
            
            #-- total number of planes
            N=len(planes)

            #-- report frequencies
            print '%d Hz = planes:   '% current_frequency, 
            for plane in planes:
                print ' ', plane,

            print


        #-- SAVE FREQUENCY VS ttotal list
        outlist=[]
        for current_frequency in frequency_ttotal.keys():
            outlist.append([current_frequency, frequency_ttotal[current_frequency]])
        mylib.write_lists_out(outlist,'frequency_ttotal.txt')
                           
            
            


print "\nDone."

        
        
    








