#!/usr/bin/env python
import os,sys,string, math
import residues
import mylib,  molmol
import grace as xmgr
import colors
from colors import *
#-- to make it portable: use color only when requested explicitly
if '-color' in sys.argv: colors.COLOR=ON

doc="""
************    Analysis of chemical shift changes   *********
************ in the course of titration ***********************

v.2.1 by Evgenii Kovrigin    11/7/02 

Usage: chem_shift_change.py [-color]


Edit section USER INPUT to insert your peak lists 

Bugs and fixes:
3/19/09 - selecting and saving groups of residues 
          with dw > cutoff2 and cutoff1 < dw < cutoff2
          for plotting in Pymol

***************************************************************
"""





#=========================== USER INPUT ==============================




#-- X value : peak list name (don't touch 1.54 - it does not matter, this is simply last point) ---
peak_lists_fnames={

            0       :   "ekn63.hsqc_2.20C.KRas_GNP.noGNP.TCI.500.sp.list",
            1.54    :   "ekn59.H_HR.N_HR.sp.save.list"

            }

#--- cutoffs to select  groups of residues dw > cutoff2 and cutoff1 < dw < cutoff2
cutoff1=0.005 
cutoff2=0.010

#-- cutoff for maximal chemical shift allowed on graphs ---
max_chemshift=1000




#=========================== END USER INPUT ==============================





#---------------------------------------------------
def difference(list1, list2, selected_peaks=None, w1_col_name="w1", w2_col_name="w2"):
    '-> list with delta w1(N), list with delta w2(H)'
    
    residues.check_keys_match(list1,list2)
    
    if selected_peaks: #-- take selected ----
        dw1=selected_peaks.make_empty_copy("delta w1")
        dw2=selected_peaks.make_empty_copy("delta w2")        
        selection=selected_peaks.get_keys()
    else:#-- take all of them
        dw1=list1.make_empty_copy("delta w1")
        dw2=list1.make_empty_copy("delta w2")        
        selection=list1.get_keys()

    for res in selection:

        l1w1=list1.get(res,w1_col_name)
        l2w1=list2.get(res,w1_col_name)
        dw1[res]=[l2w1-l1w1]

        l1w2=list1.get(res,w2_col_name)
        l2w2=list2.get(res,w2_col_name)
        dw2[res]=[l2w2-l1w2]

    return dw1,dw2

#---------------------------------------------------
def abs_norm_difference(list1, list2, selected_peaks=None, w1_col_name="w1", w2_col_name="w2"):
    '-> list with |delta w1(N)/5|, list with |delta w2(H)|'
    
    residues.check_keys_match(list1,list2)
    

    if selected_peaks: #-- take selected ----
        dw1=selected_peaks.make_empty_copy("delta w1")
        dw2=selected_peaks.make_empty_copy("delta w2")        
        selection=selected_peaks.get_keys()
    else:#-- take all of them
        dw1=list1.make_empty_copy("delta w1")
        dw2=list1.make_empty_copy("delta w2")        
        selection=list1.get_keys()

    for res in selection:

        l1w1=list1.get(res,w1_col_name)
        l2w1=list2.get(res,w1_col_name)
        dw1[res]=[abs(l2w1-l1w1)/5]

        l1w2=list1.get(res,w2_col_name)
        l2w2=list2.get(res,w2_col_name)
        dw2[res]=[abs(l2w2-l1w2)]

    return dw1,dw2




#---------------------------------------------------
def Bax_difference(list1, list2, selected_peaks=None, w1_col_name="w1", w2_col_name="w2"):
    '-> list with averaged chemical shift change'
    
    residues.check_keys_match(list1,list2)    
    
    dw_av=list1.make_empty_copy("delta w averaged")

    if selected_peaks: #-- take selected ----
        dw_av=selected_peaks.make_empty_copy("delta w averaged")
        selection=selected_peaks.get_keys()
    else:#-- take all of them
        dw_av=list1.make_empty_copy("delta w averaged")
        selection=list1.get_keys()

    for res in selection:

        l1w1=list1.get(res,w1_col_name)
        l2w1=list2.get(res,w1_col_name)
        dw1=l2w1-l1w1

        l1w2=list1.get(res,w2_col_name)
        l2w2=list2.get(res,w2_col_name)
        dw2=l2w2-l1w2

        dw_av[res]= [math.sqrt( (dw2**2 + (dw1**2)/25)/2)]


    return dw_av












#--------------- load files ---------
peak_lists={} # X:data_table

start_list=residues.Sparky_lt_Peaklist(peak_lists_fnames[0])
end_list=residues.Sparky_lt_Peaklist(peak_lists_fnames[1.54])

#--- now extract common set of residues ----
peak_lists[0]=start_list.get_intersection(end_list.get_keys())
peak_lists[1.54]=end_list.get_intersection(start_list.get_keys())

#- display ones which disappear on titration --
start_res=start_list.get_keys()
end_res=end_list.get_keys()
cprint(BLUE, "These peaks disappear on titration")
disappeared=[]
for res in start_res:
    if res not in end_res:
        print res
        disappeared.append(res)

print "-------"


#print peak_lists[0]
#print peak_lists[1.54]

##for X_value in peak_lists_fnames.keys():
##    peak_lists[X_value]=residues.Sparky_lt_Peaklist(peak_lists_fnames[X_value])
    
    
##selected_peaks=residues.Sparky_lt_Peaklist(
##    selected_peaks_fname).make_empty_copy(
##        "selected peaks").get_intersection(peak_lists[0])

#--- take all peaks ---
selected_peaks=peak_lists[0].make_empty_copy("selected")


#****************  First and Last spectrum differences *********************


#---- calculate differences ----

dw1_abs,dw2_abs=abs_norm_difference(peak_lists[0], peak_lists[1.54],selected_peaks)

dw_av=Bax_difference(peak_lists[0], peak_lists[1.54] ,selected_peaks)



#========= SELECT AND SAVE GROUPS WITH LARGE DW ==========
#--- show group that is dw>cutoff2
outstring= "Residues with Bax's average dw  > %f :\n" % cutoff2
print outstring
keys =dw_av.get_keys()

for key in keys:
  if dw_av[key][0]>cutoff2:
    print key
    outstring+= "%s,\t"% key[1:]
fname="dw_larger_than_cutoff2.txt"
fid=open(fname,'w')
fid.write(outstring)
fid.close()
print "File '%s' written.\n\n" % fname
  
#--- show group that has cutoff1 < dw < cutoff2
outstring= "Residues with %f < Bax's average dw < %f :\n" % (cutoff1, cutoff2)
print outstring
keys =dw_av.get_keys()

for key in keys:
  if dw_av[key][0]>cutoff1:
    if dw_av[key][0]<cutoff2:
      print key
      outstring+= "%s,\t"% key[1:]
fname="dw_between_cutoff1_and_2.txt"
fid=open(fname,'w')
fid.write(outstring)
fid.close()
print "File '%s' written.\n\n" % fname
  


#---- Data representation -----

#-- clip outliers ---
for res in dw_av.get_keys():
    if  dw_av[res][0]>max_chemshift :
        cprint(YELLOW, "clip '%s' (%.4f)" % (res,dw_av[res][0]))
        dw_av.set_word(res,0,max_chemshift)


#-- export MolMol macro
Rnase=molmol.Protein(124,dw_av)
Rnase.set_assigned_colors(molmol.White, molmol.Blue)
Rnase.set_background_color(molmol.DarkGrey)
Rnase.colorize_by_parameter(0)
Rnase.highlight(disappeared, molmol.Red)
Rnase.highlight(["Q11","K41","T99"],molmol.Green)
Rnase.write_macro('molmol.mac')

#--- XMGR ----
dw1vsRes=xmgr.Dataset(dw1_abs.to_xmgr("ResNum",0),"15-N deltas/5")
dw1vsRes.circle()
dw1vsRes.black_sym()
dw1vsRes.symbol_size(0.5)
dw1vsRes.empty_sym()

dw2vsRes=xmgr.Dataset(dw2_abs.to_xmgr("ResNum",0),"H deltas")
dw2vsRes.circle()
dw2vsRes.red_sym()
dw2vsRes.empty_sym()

dw_avvsRes=xmgr.Dataset(dw_av.to_xmgr("ResNum",0),"average deltas")
dw_avvsRes.circle()
dw_avvsRes.blue_sym()
dw_avvsRes.symbol_size(0.5)

dw1vsResGraph=xmgr.Graph("Absolute deltas")
dw1vsResGraph.add_dataset(dw1vsRes)
dw1vsResGraph.add_dataset(dw2vsRes)
dw1vsResGraph.set_xaxis("Residue",0,125)
dw1vsResGraph.set_yaxis("delta",0,0.4)
dw1vsResGraph.writeout("cs_absdeltas.xmgr")

dw_avvsResGraph=xmgr.Graph("Bax's average deltas", "")
dw_avvsResGraph.add_dataset(dw_avvsRes)
dw_avvsResGraph.set_xaxis("Residue",0,80,10,5)
dw_avvsResGraph.set_yaxis("delta",0,0.08)
dw_avvsResGraph.writeout("cs_Baxdeltas.I")
dw_avvsResGraph.set_xaxis("Residue",81,170,10,5)
dw_avvsResGraph.writeout("cs_Baxdeltas.II")

dw1vsRes.export_data('dw1.txt')
dw2vsRes.export_data('dw2.txt')
dw_avvsRes.export_data('dw_av.txt')

cprint(INV_RED, "Don't do titration curves!")
sys.exit()
#*****************  Follow Titration Curves *********************


# =======  Bax's Average delta vs X-value =====

deltas_av={}
X_range=peak_lists_fnames.keys()
X_range.sort()

#-- enough datasets ? ----
if len(X_range)<2:
    raise UserWarning, "\n\n\a\a\a You cannot make titration curves with just\n\
    one dataset!!!"

#-- calculate differences for each X and residue
for X in X_range:
    deltas_av[X]=Bax_difference(peak_lists[X_range[0]], peak_lists[X] ,selected_peaks)    



#-- plot them
titr_graph=xmgr.Graph("Titration","Averaged weighted chemical shift")
symbol=2
for res in selected_peaks.sorted_keys():
    titr_data=[]  # X : dw_av

    for X in X_range:
        titr_data.append([X,deltas_av[X][res][0]])

    titr_dts=xmgr.Dataset(titr_data, res)
    titr_dts.symbol=symbol
    titr_dts.solid_line()
    titr_graph.add_dataset(titr_dts)
    #symbol+=1

titr_graph.set_xaxis("L/P",0,1.6)
titr_graph.set_yaxis("delta aver, ppm",0,0.15)
    
titr_graph.writeout("cs_titr_av.xmgr.%s" % sublist)



###=============== 'Polar' plot (w1 vs w2) =========================
##wN,wH="nitrogen","proton"
##
###-- calculate differences for each X and residue
##delta_abs={}
##for X in X_range:
##    delta_abs[X]={}
##    dw1,dw2=difference(peak_lists[X_range[0]], peak_lists[X] ,selected_peaks)  
##    delta_abs[X][wN]=dw1
##    delta_abs[X][wH]=dw2
##
###-- plot them
##NH_graph=xmgr.Graph("Titration","Absolute chem. shift changes")
##symbol=2
##for res in selected_peaks.sorted_keys():
##    NH_data=[]  # X : dw_av
##
##    for X in X_range:
##        dwN=delta_abs[X][wN][res][0]
##        dwH=delta_abs[X][wH][res][0]
##        NH_data.append([dwH,dwN])
##
##    NH_dts=xmgr.Dataset(NH_data, res)
##    NH_dts.symbol=symbol
##    NH_dts.solid_line()
##    NH_graph.add_dataset(NH_dts)
##    #symbol+=1
##
##NH_graph.set_xaxis("delta H, ppm",-0.2,0.2)
##NH_graph.set_yaxis("delta N, ppm",-1,1)
##   
##NH_graph.writeout("cs_titr_absNH.xmgr.%s" % sublist)
##
##    
##
##






print "Done..."
