doc="""
cleanex_ek.py

Fitting of cleanex-pm data
--------------------------

This extension is based on Tom's 'relax.py'.
The following features changed:

- It does R calculation instead of T
- "Save Brief" Button - Saves data as plain Residue-Rate-Error format
- Check your fits: now you can walk through the residue list with 'Show next plot'
                    'Show prev plot' buttons and look at your fits.
- Relative scale on fitting plots is set in Setup and is shown as red positive and blue
  negative  bar
- Rates and errors are the same as calculated with the curvefit
- In order to exclude some points from fits: use LT selection and run eh separately and
  save them separately



Previous patch included the following:

#8/23/02   fixes from Tom Goddard combined with previous script features by Evgueni
# The version includes: 
#   1. scaling factor for peak heights
#   2. Roger's 3x3 grid height determination
#   3. "rh" taking peak heights at the peak maxima
#	(if you want to measure heights at the fixed position of the 
#  	peak maximum in the current window, just click on the button
#-------------------------------------------------------------------------
#
"""



import curvefit3_functions
reload(curvefit3_functions)

import sputil
import relax_dialog
reload(relax_dialog)

class cleanex_dialog(relax_dialog.relaxation_dialog):
  def __init__(self, session, title='Fitting of cleanex signal buildup'):
     relax_dialog.relaxation_dialog.__init__(self, session, title)
  # ---------------------------------------------------------------------------
  #
  def time_height_pairs(self, pos, assignment, spectrum_times):

    th_pairs = []
      
    #-- check if HSQC spectrum is supplied ---
    HSQCheight='None'
    for s, t, f in spectrum_times:
      if t==-1:
         HSQCheight='HSQC found'
    if HSQCheight=='None':
      self.session.command_characters('py')
      raise UserWarning, ('\n' + 20*'='+
"\n\nWARNING!\n You need to have an HSQC spectrum in the list so\n\
that the cleanex intensities can be normalized to the HSQC intensities.\n\
\nPlease, set time for the HSQC spectrum to -1")




    #-- determine peak heights
    for s, t, f in spectrum_times:
      if assignment:
	# Use position of peak with matching assignment if available
	peak = s.find_peak(assignment)
      else:
	peak = None
      if peak:
	h = self.average_height(s, peak.position)*f
      else:
	h = self.average_height(s, pos)*f

      # separate HSQC
      if t==-1:
        HSQCheight=h
      else:
       th_pairs.append([t, h])

    #-- normalize heights ---
    if HSQCheight!='None':
      for pair in th_pairs:
        pair[1]/=HSQCheight

    #print th_pairs
    return th_pairs



  # ---------------------------------------------------------------------------
  #
  def fit_function(self, th_pairs, assignment):

#   params = (0.01,0.01)
   trials = self.error_estimate_trials
   epsilon=1e-10
   
   print "------------------------ Fitting %s -----------------------------" % assignment
   for line in th_pairs:
      print "t=%.2e  I=%.2e" % (line[0],line[1])

##   #-- run my Any_Function object now -----
##   params=[0.01,0.01]
##   func = curvefit3_functions.init_Cleanex_Buildup()
##   func.set_data(th_pairs)
##   func.set_init_params(params)
##
##   #-- try fitting with a simpler form ---   
##   try:
##     func.fit_data(trials,epsilon)
##   except OverflowError:
##     self.session.command_characters('py')
##     print "\n\nFitting warning!   Assignment %s didn't fit !!!" % assignment
##     func.fit.param_sd=[0]*len(func.fit.params)
##     return func.fit
##   print "succesful initial fit!"
##   print "Parameters: ",func.fit.params
##   print "Point SD: %.2e " % func.fit.point_sd
##
##   #-- use fitting results as initial estimates for full form
##   print "--- Try full form of equation now ---"
   R1A=0.01#func.fit.params[0]
   k=0.01#func.fit.params[1]
   R1B=R1A/2

   
   estimated_params=[R1A,k,R1B]
   func = curvefit3_functions.Cleanex_Buildup()
   func.set_data(th_pairs)
   func.set_init_params(estimated_params)   
   try:
     func.fit_data(trials,epsilon)
   except OverflowError:
     self.session.command_characters('py')
     print "\n\nFitting warning!   Assignment %s didn't fit !!!" % assignment
     func.fit.param_sd=[0]*len(func.fit.params)
     return func.fit
   
   print "Successful final fit!"
   print "Parameters: ",func.fit.params
   print "Point SD: %.2e " % func.fit.point_sd

   
   #-- try error estimate ---
   try:
     func.set_same_dy(func.fit.point_sd)
     func.fit.estimate_parameter_error(trials)
     
   except OverflowError:
     self.session.command_characters('py')
     print "\n\nFitting warning!   Error estimate for %s failed !!!" % assignment
     func.fit.param_sd=[0]*len(func.fit.params)

   print "Parameter SD: ", func.params_sd()
   return func.fit


  # ---------------------------------------------------------------------------
  #
  def peak_line(self, peak, fit):

    htext = ''
    for t, h, dh in fit.xy_pairs:
      htext = htext + '%10.4g ' % h

    line = ('%15s    %.4f +-%.4f   ' % (peak.assignment[:-3], -fit.params[1], fit.param_sd[1])  + htext)
    return line

  # ---------------------------------------------------------------------------
  #
  # ---------------------------------------------------------------------------
  #



def run(session):
  sputil.the_dialog(cleanex_dialog,session).show_peak_heights()
