#!/usr/bin/env python
doc="""
************ Compare protocols: resolving overlaps - the effect on rates ****
by Evgenii Kovrigin
v.2


    - you calculate something (usu rate or NOE) using spectra processed by
      two different protocols, P1 and P2, and supply two files with
      res-value-error structure 
    
    - the output is:
    
            - a list of residue sorted from min to max (err1+err2)/abs(Val1-Val2)
            
            -XMGR file with the same quantities
            
    - now you can judge the amount of improvement in the calculated
      value when you use higher resolution protocol for overlapping peak pair
      compared to sum of errors of this value taken from both protocols.
      Artifactual improvements manifest themselves as 'improvements' for
      peak resolved well in Protocol 1 - this is a level above which you
      shouldn't go...

Usage:

    compare_protocols.py P1.data P2.data

    (order is not important)
    
****************************************************************************
"""

import residues, xmgr
import os,sys,copy


if len(sys.argv) < 3:
    print doc
    sys.exit()



P1=residues.Annn_Num(sys.argv[1])
P2=residues.Annn_Num(sys.argv[2])


#--- check
residues.check_keys_match(P1,P2)

result=residues.Annn_Num()

R,ERR=0,1 # columns in rates files

for res in P1.get_keys():
    err=P1[res][ERR]+P2[res][ERR]
    diff=abs(P1[res][R]-P2[res][R])
##    
##    if factor > err/diff: print res+" - different!"
##    else: print res
    if diff: result.add(res,[err/diff])
    else: result.add(res,[1000])

#--- sort results by the effect strength ---
sorted=residues.Word_Dict()
res_list=copy.deepcopy(result)
for res in range(len(res_list)):
    cur_res=res_list.min(0)
    sorted.add(cur_res,["%.2f"%res_list[cur_res][0]])
    del res_list[cur_res]
sorted.set_name(P1.get_name()+" vs "+P2.get_name()+
                "\nSorted by '(err_P1+err_P2)/abs(R_P1-R_P2)' value.")
sorted.writeout("compare_protocols.dat", verbose="on")


#---- presentation --------

ratio_dat=xmgr.Dataset(result.to_xmgr("ResNum",0),"error/difference")

ratio_Graph=xmgr.Graph("Resolution enhancement analysis", P1.get_name()+" vs "+P2.get_name())
ratio_Graph.add_dataset(ratio_dat)


ratio_Graph.set_xaxis("Residue",0,125)
ratio_Graph.set_yaxis("err/diff",0,1.5)
ratio_Graph.writeout("compare_protocols.xmgr", legend="off", verbose="on")
   


