#!/usr/bin/env python
import string, mylib, glob, os, sys
doc="""

=========== concatenate_intensity_data.py ===================
by Evgenii Kovrigin   6/13/08
Medical College of Wisconsin
v.1.2

This program concatenates datasets obtained in different rcCPMG experiments
but at the same field, temperature, etc. It allows fitting together data obtained
at with different CPMG frequency tables.

The program relies on data structures build by es2m_full_dispersion_v1.py and
es2m_full_dispersion_v2.py.

First, the program normalizes intensities to have average intensity of reference 
experiment =1 and then scales all CPMG-frequency datasets. As data gets normalized
they may be fit together now.

8/05/08 - In case you have huge list of residues - use help_concatenate_intensity_data.py
          to generate a code for resname_dict{} (simply cut-and-paste) its output here.

============================================================================



"""

#===== USER INPUT SECTION =====
#-- dictionary of residues with corresponding usable dataset names
resname_dict={
'T2' : [ 'odd','even' ],
'E3' : [ 'odd','even' ],
'Y4' : [ 'odd','even' ],
'K5' : [ 'odd','even' ],
'L6' : [ 'odd','even' ],
'V7' : [ 'odd','even' ],
'V8' : [ 'odd','even' ],
'V9' : [ 'odd','even' ],
'V14' : [ 'odd','even' ],
'G15' : [ 'odd','even' ],
'S17' : [ 'odd','even' ],
'L19' : [ 'odd','even' ],
'T20' : [ 'odd','even' ],
'Q22' : [ 'odd','even' ],
'L23' : [ 'odd','even' ],
'I24' : [ 'odd','even' ],
'Q25' : [ 'odd','even' ],
'N26' : [ 'odd','even' ],
'H27' : [ 'odd','even' ],
'R41' : [ 'odd','even' ],
'K42' : [ 'odd','even' ],
'Q43' : [ 'odd','even' ],
'I46' : [ 'odd','even' ],
'G48' : [ 'odd','even' ],
'E49' : [ 'odd','even' ],
'T50' : [ 'odd','even' ],
'C51' : [ 'odd','even' ],
'L52' : [ 'odd','even' ],
'L53' : [ 'odd','even' ],
'I55' : [ 'odd','even' ],
'L56' : [ 'odd','even' ],
'M72' : [ 'odd','even' ],
'T74' : [ 'odd','even' ],
'G75' : [ 'odd','even' ],
'F78' : [ 'odd','even' ],
'V81' : [ 'odd','even' ],
'F82' : [ 'odd','even' ],
'A83' : [ 'odd','even' ],
'I84' : [ 'odd','even' ],
'N85' : [ 'odd','even' ],
'N86' : [ 'odd','even' ],
'K88' : [ 'odd','even' ],
'S89' : [ 'odd','even' ],
'F90' : [ 'odd','even' ],
'E91' : [ 'odd','even' ],
'N92' : [ 'odd','even' ],
'I93' : [ 'odd','even' ],
'H94' : [ 'odd','even' ],
'Q95' : [ 'odd','even' ],
'Y96' : [ 'odd','even' ],
'R97' : [ 'odd','even' ],
'E98' : [ 'odd','even' ],
'V103' : [ 'odd','even' ],
'K104' : [ 'odd','even' ],
'D105' : [ 'odd','even' ],
'S106' : [ 'odd','even' ],
'D107' : [ 'odd','even' ],
'D108' : [ 'odd','even' ],
'V109' : [ 'odd','even' ],
'M111' : [ 'odd','even' ],
'V112' : [ 'odd','even' ],
'L113' : [ 'odd','even' ],
'V114' : [ 'odd','even' ],
'G115' : [ 'odd','even' ],
'N116' : [ 'odd','even' ],
'K117' : [ 'odd','even' ],
'C118' : [ 'odd','even' ],
'D119' : [ 'odd','even' ],
'L120' : [ 'odd','even' ],
'A121' : [ 'odd','even' ],
'A122' : [ 'odd','even' ],
'R123' : [ 'odd','even' ],
'T124' : [ 'odd','even' ],
'V125' : [ 'odd','even' ],
'E126' : [ 'odd','even' ],
'S127' : [ 'odd','even' ],
'R128' : [ 'odd','even' ],
'Q129' : [ 'odd','even' ],
'A130' : [ 'odd','even' ],
'Q131' : [ 'odd','even' ],
'D132' : [ 'odd','even' ],
'L133' : [ 'odd','even' ],
'A134' : [ 'odd','even' ],
'R135' : [ 'odd','even' ],
'S136' : [ 'odd','even' ],
'Y137' : [ 'odd','even' ],
'G138' : [ 'odd','even' ],
'I139' : [ 'odd','even' ],
'Y141' : [ 'odd','even' ],
'I142' : [ 'odd','even' ],
'E143' : [ 'odd','even' ],
'T144' : [ 'odd','even' ],
'S145' : [ 'odd','even' ],
'A146' : [ 'odd','even' ],
'K147' : [ 'odd','even' ],
'T148' : [ 'odd','even' ],
'R149' : [ 'odd','even' ],
'Q150' : [ 'odd','even' ],
'G151' : [ 'odd','even' ],
'V152' : [ 'odd','even' ],
'E153' : [ 'odd','even' ],
'D154' : [ 'odd','even' ],
'A155' : [ 'odd','even' ],
'F156' : [ 'odd','even' ],
'Y157' : [ 'odd','even' ],
'T158' : [ 'odd','even' ],
'L159' : [ 'odd','even' ],
'V160' : [ 'odd','even' ],
'R161' : [ 'odd','even' ]
}





#-- put in paths to your multiple experiments 
paths= {
    'odd' : '/Users/kovrigin/Documents/Workspace/Data/Data.XIII/EKN42.rcCPMG.RasGNP166.RT.500/data/rcCPMG.118ppm/added_odd_half/sp/Sparky/eS.500.22C.118ppm.odd_half.txt.parsed.v1/' ,
	'even' :  '/Users/kovrigin/Documents/Workspace/Data/Data.XIII/EKN42.rcCPMG.RasGNP166.RT.500/data/rcCPMG.118ppm/added_even_half/sp/Sparky/eS.500.22C.118ppm.even_half.txt.parsed.v1/'
        }

#-- column in ?.txt files where peak intensity is (counted as 1,2,3,...)
intensity_column=2

#-- output folder  
outdir="Data"

#===== END OF USER INPUT SECTION =====



#======= CLASS DEFINITIONS =======

class Dataset:
    """ 
    This is a relaxation dispersion data set for one resname. 
    
    Usage:
    
    #-- create
    new_dataset=Dataset('V9','test')
    
    #-- load data from disk
    new_dataset.load_data('es_list.parsed/')
    
    #-- display
    print new_dataset
    
    
    """
    #====== GENERATE EMPTY OBJECT =====
    def __init__(self, resname,comment=''):
        self.full_datapath=''
        self.resname=resname #-- actual name of the folder where intensity data are
        self.comment=comment #-- some identifier, for example: offset or number of points
        self.frequency_list_fname='frequencies.txt'

        #--- list of frequencies (integers)
        self.frequencies=[]
        #-- dictionary of data arrays indexed by frequencies. Data arrays are floating point type.
        self.data={}
        
        
        
    #--- TO BE ABLE TO print x    
    def __repr__(self):
        'representation on print operation'
        out= '='*15 + 'Dataset %s (%s)' % (self.resname, self.comment) + '='*15
        out+= '\nFrequencies: '
        for freq in self.frequencies: out+=' %d  ' % freq
        out+= '\nData: %d arrays\n' % len(self.data.keys())
        sorted_keys=self.data.keys()
        sorted_keys.sort()
        for freq in sorted_keys:
            out+='\nCPMG frequency %d Hz\n' % (freq)
            for row in self.data[freq]:
                for word in row:
                    out+='%e\t' % word
                out+='\n'
            
            
        return out





            
    #------ LOAD_DATA ---------    
    def load_data(self,full_datapath):
        'loading of the data from disk'
        self.full_datapath=full_datapath
        #--- load frequencies
        fname= os.path.join(self.full_datapath,resname,self.frequency_list_fname)
        self.frequencies=map(int,map(string.strip,mylib.Load_Text_File_NonEmpty_Lines(fname)))
        
        #-- load reference experiment
        fname=os.path.join(self.full_datapath,resname,'0.txt')
        datafile=mylib.Load_Text_File_NonEmpty_Lines(fname)
        self.data[0]=datafile
        
        #-- load frequency-specific data
        for frequency in self.frequencies:
            fname=os.path.join(self.full_datapath,resname,'%d.txt' % frequency)
            datafile=mylib.Load_Text_File_NonEmpty_Lines(fname)
            #-- store in main dictionary
            self.data[frequency]=datafile

        #-- convert to floating point
        for frequency in self.data.keys():
            
            datafile=self.data[frequency]
            #--- convert to floating point
            datastrings=map(string.split,datafile)
            datafloat=[]
            for row in datastrings:
                rowfloat=map(float,row)
                datafloat.append(rowfloat)
                
            #-- store in main dictionary
            self.data[frequency]=datafloat
            
            
    #============ WRITE DATA AS NEW DATASET ===========
    def writeout(self,full_datapath):
        'export data for fitting in MatLab'
        self.full_datapath=os.path.join(full_datapath,self.resname)
        
        #--- create folder or clean it up if exists
        if os.path.isdir(self.full_datapath): 
            os.system('rm -r %s'% os.path.join(self.full_datapath,'*'))
        else: 
            mylib.deep_mkdir(self.full_datapath)
        
        #--- export frequencies
        
        stringarray= map(str, self.frequencies)
        outstr=string.join( stringarray ,'\n')
        mylib.Write_Text_File(outstr, 
               os.path.join(self.full_datapath, self.frequency_list_fname))

        #--- export intensities
        for freq in self.data.keys():
            fname=os.path.join(self.full_datapath,'%d.txt'%freq)
            mylib.write_lists_out(self.data[freq], fname)    
    
    
    #============== NORMALIZE_INTENSITY ================
    def normalize_intensity(self, intensity_column):
        """normalize all datasets by average intensity in the first dataset
        Intensity column is given in human notation 1,2,3.... etc"""
        #-- average intensity in the reference dataset
        #----- get the column
        I0_average=0
        N=len(self.data[0])
        for count in range(N):
            intensity=self.data[0][count][intensity_column-1]
            I0_average+=intensity/N
        
        #--- normalize all data sets intensity using this value
        freqs=self.data.keys()
        for freq in freqs:
            N=len(self.data[freq])
            for count in range(N):
                intensity=self.data[freq][count][intensity_column-1]
                self.data[freq][count][intensity_column-1]=intensity/I0_average
            
        
        
def strip_1st_letter(x):
    return x[1:]
        
#===== END OF CLASS DEFINITIONS ===============



#=== PROGRAM BEGINS =====

#--- make  output directory
if not os.path.isdir(outdir): os.mkdir(outdir)



#--- go through dictionary of residues-datasets
for resname in resname_dict.keys():
        print 'Residue %s' % resname
        data_paths={}
        for dataset_ID in resname_dict[resname]:
            data_paths[dataset_ID]=paths[dataset_ID]
            print 'Use "%s" dataset.' % dataset_ID
            


        #-- load data for this residue
        experiment_list=data_paths.keys()
        Data={}
        for experiment in experiment_list:
            dataset=Dataset(resname,experiment) #-- create empty dataset
            dataset.load_data(data_paths[experiment]) #-- load data from disk
            dataset.normalize_intensity(intensity_column) #--- normalize data by reference experiment
            Data[experiment]=dataset #-- store data
          
            

        #-- make a combined list with all poissible frequencies (in case of different frequency tables
        full_frequency_list=Data[experiment_list[0]].frequencies
        for experiment in experiment_list[1:]:
            for frequency in Data[experiment].frequencies:
                if frequency in full_frequency_list: continue
                else: full_frequency_list.append(frequency)
                

        #--- generate a new data set
        combined_data=Dataset(resname,string.join(experiment_list,'_'))
        combined_data.frequencies=full_frequency_list

        #-- combine reference experiments
        new_array=[]
        for experiment in experiment_list:
            for row in Data[experiment].data[0]:
                new_array.append(row)
        combined_data.data[0]=new_array

        #--- combine CPMG frequencies

        for frequency in full_frequency_list:
            new_array=[]
            for experiment in experiment_list:
                #-- skip if frequency was not used in this experiment
                if frequency not in Data[experiment].data.keys(): continue
                #-- concatenate data
                for row in Data[experiment].data[frequency]:
                    new_array.append(row)
            combined_data.data[frequency]=new_array

            
        #    concatenate and write out as new data sets.

        combined_data.writeout(outdir)

#--- save residue list
residue_list=resname_dict.keys()
outstr1=string.join(residue_list,'\n')
#---- save names
mylib.Write_Text_File(outstr1, 
               os.path.join(outdir, 'reslist.txt'))
#---- save number for Matlab
outstr2=map(strip_1st_letter,residue_list)
outstr2=string.join(outstr2,'\n')
mylib.Write_Text_File(outstr2, 
               os.path.join(outdir, 'resnums.txt'))



