#!/usr/bin/env python
doc=""" ====== CONVERT_DK_OUTPUT.py =========
version 2
Evgenii Kovrigin, 7/24/08
Medical College of Wisconsin

This program converts output of cpmg_fitd8 program into MATLAB-readable
format.The program parses DAT file for the specified dataset and creates graphic files

Usage:
conver_dk_output.py dataset_name [CPMG_freq_max R2_min R2_max]

    dataset_name - name without extension of the DAT file

    Optional arguments: limits for graphic representations
    CPMG_freq_max, R2_min, R2_max - must be either dropped or provided all three.

"""

import mylib, grace
import os,sys,string

#====== USER INPUT =========
#-- limits for plotting
CPMG_max=600
R2_min=15
R2_max=40


if len(sys.argv)==2:
    dataset=sys.argv[1]
else:
    if len(sys.argv)==5:
        dataset=sys.argv[1]
        CPMG_max=float(sys.argv[2])
        R2_min=float(sys.argv[3])
        R2_max=float(sys.argv[4])
    else:
        #-- error
        print doc
        sys.exit()



#====== END OF USER INPUT =========


#-- DAT file columns names in 'cpmg_fitd8' output
residue_name_colname='res'
B0_frequency_colname='sfrq'
temp_colname='temp/P'
CPMG_frequency_colname='cpfrq/B1'
R2exp_colname='val'
R2exp_error_colname='err'
R2fit_colname='cval'


#-- open data and parameter files

raw_data=mylib.Load_Text_File_NonEmpty_Lines(dataset+'.dat')
raw_parameters=mylib.Load_Text_File_NonEmpty_Lines(dataset+'.par')


#-- depository of all data
All_Data={}


#-- find column numbers
RES_NUM_COL=raw_data[0].split().index(residue_name_colname)-1
TEMP_COL=raw_data[0].split().index(temp_colname)-1
FIELD_COL=raw_data[0].split().index(B0_frequency_colname)-1
CPMG_FREQ_COL=raw_data[0].split().index(CPMG_frequency_colname)-1
R2_EXP_COL=raw_data[0].split().index(R2exp_colname)-1
R2_ERR_COL=raw_data[0].split().index(R2exp_error_colname)-1
R2_FIT_COL=raw_data[0].split().index(R2fit_colname)-1


#--- Extract data by residues first
for line in raw_data[1:]:
    words=string.split(line)

    #-- obtain and analyze residue number on the line
    residue_number=words[RES_NUM_COL]
    known_residues=All_Data.keys()
    if residue_number not in known_residues:
        All_Data[residue_number]={}

    #-- obtain and analyze temperature
    temperature=words[TEMP_COL]
    known_temps=All_Data[residue_number].keys()
    if temperature not in known_temps:
        All_Data[residue_number][temperature]={}

    #-- obtain and analyze B0 field strength
    B0_field=words[FIELD_COL]
    known_fields=All_Data[residue_number][temperature].keys()
    if B0_field not in known_fields:
        All_Data[residue_number][temperature][B0_field]={'exp':[], 'fit':[]}

    #-- obtain experimental CPMG frequency, R2, its error and fitted R2
    CPMGfreq=words[CPMG_FREQ_COL]
    R2exp=words[R2_EXP_COL]
    dR2exp=words[R2_ERR_COL]
    R2fit=words[R2_FIT_COL]

    #-- save the data
    All_Data[residue_number][temperature][B0_field]['exp'].append([CPMGfreq,R2exp,dR2exp])
    All_Data[residue_number][temperature][B0_field]['fit'].append([CPMGfreq,R2fit])


#-- make figures
all_residues=All_Data.keys()
for residue in all_residues:
    all_temperatures=All_Data[residue].keys()

    #-- create one graph per temperature
    for temperature in all_temperatures:

        graph=grace.Graph("Residue %s" % residue,  "%s Deg C" % temperature)
        
        all_fields=All_Data[residue][temperature].keys()

        #-- create datasets
        symbol=1
        for field in all_fields:
            expdata=All_Data[residue][temperature][field]['exp']
            fitdata=All_Data[residue][temperature][field]['fit']
            
            expdataset=grace.Dataset(expdata, "%s" % field)
            expdataset.symbol=symbol
            expdataset.no_line()
            expdataset.empty_sym()

            fitdataset=grace.Dataset(fitdata, "%s" % field)
            fitdataset.solid_line()
            fitdataset.no_sym()

            graph.add_dataset(expdataset)
            graph.add_dataset(fitdataset)

            symbol+=1

        graph.set_xaxis("CPMG frequency, Hz",0,CPMG_max,100, 5)
        graph.set_yaxis("R2, /s",R2_min, R2_max, 2, 5)
        graph.writeout("Res_%s.%sC" % (residue, temperature))


            
