#!/usr/bin/env python
doc="""
----------------------------------------
 'copy_assignments.py'

  v.3 by Evgenii Kovrigin           11/17/02

  Copying Assignments and Spectrum parameters
  This macro allows you to adjust all the parameters you like 
  on one spectrum and then use its '.save' file as a template
  to create '.save' files for a number of .sp files

    Usage: copy_assignments_2.py  template_file_path
            - original .sp files are expected to be in the current directory
            - .save files are saved in /Sparky folder in the current directory

----------------------------------------

"""
import os, sys, string,  glob 


if len(sys.argv) !=2:
    print doc
    sys.exit()

template_fname=sys.argv[1]
if not os.path.isfile(template_fname):
    print "\a\a\a\nError! '%s' file doesn't exist!!!\n\n" % template_fname
    sys.exit()

# '.sp' data files location
path_to_sp_files="./"


# where to save generated '.save.sp' file for Sparky
path_to_save_files="Sparky"



#--------- What to look for in a Sparky template file ----
spectrum_section='<spectrum>'    
spectrum_name_keyword='name'	# first field on the spectrum name string
spectrum_name_offset=1		# offset from <spectrum>

spectrum_pathname_keyword='pathname'	# first field on the spectrum pathname string
spectrum_pathname_offset=2		# offset from <spectrum>

view_section='<view>'
view_name_keyword='name'	# first field on the spectrum name string
view_name_offset=1 		# offset from <view>
#------------------------------------------------------------

   
    
#------------------ Load_Text_File  --------------
#---- a function to read a text file -------------

#-------------------------------------------------
def Load_Text_File (fname) :
    if not os.path.isfile(fname): 
        print "\n\n\nError!\nCannot open '%s' to read!!!\n\n\a\a\a" % fname
	sys.exit()
    input=open(fname,'r')	
    L=input.readlines()
    input.close()
    return L

#------------------ String_2_Words_2D ----------------------
#----- Convert a list of string into a 2D list of words ------
#-------------------------------------------------
def String_2_Words_2D(list) :
    words_2D_list=[]
    for string in list :
        words_2D_list.append(string.split(string))        
    return words_2D_list


#-------------------- Find_Position ---------------------------
#----- search for a word in a list 
#----- of strings. Returns string number, word number 
#----- or empty list if not found
#-------------------------------------------------
def Find_Position(list, word) :
    for str in list :
        words=string.split(str)
        if word in words : 
           return list.index(str), words.index(word)
    else : 
        return []

#-------------------- Find_String_Num ---------------------------
#----- search for a word in a list 
#----- of strings. Returns string number
#----- or None if not found
#-------------------------------------------------
def Find_String_Num(list, word) :
    for str in list :
        words=string.split(str)
        if word in words : 
           return list.index(str)
    else : 
        return None

#---------------------- Check_If_Line_Is_Right ----------------------------
#---------- checking for presence of a kewword in the beginning of the line
#---------- aborts if keyword doesn't match 
#--------------------------------------------------
def Check_If_Line_Is_Right(fname, file_data_list, line, keyword) :
  tempstr=file_data_list[line]
  tempwords=string.split(tempstr)
  #----- check if it is right line to modify
  if (not tempwords) or (tempwords[0] != keyword) :
      print '\n\nError!\nLine %s of %s should contain "%s" keyword but instead it is:\n"%s"\n\n\a\a\a' % (
               line, fname, keyword, tempstr[:-1])
      sys.exit()
#--------------------------------------------------------------

#---------------  Check_File_For_Keyword --------------------
#-- finds a keyword and returns its string number
#-- aborts if no keyword found
#--------------------------------
def Check_File_For_Keyword(fname, file_data_list, keyword) :
    linenum=Find_String_Num(file_data_list, keyword)
    if not linenum : 
       print "\n\nError!\n%s keyword is not found in %s!\n\n\a\a\a"  %  (keyword, fname)
       sys.exit()
    else : 
       return linenum


#---------------------------------------------------

def get_yes(question):
    'asks question -> 1 if yes, 0 if no'
    print question
    while (1):
        word=raw_input("Y/N:")

        if word in ('Y','y') :  return 1
        
        elif word in ('N', 'n') : return 0
            


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
print """\n----------------------------------------
*** Copying Assignments and Spectrum parameters ***
----------------------------------------"""



#---- check if directory exists, ask and make target directory if don't exist
if not os.path.isdir(path_to_save_files): 
  os.mkdir(path_to_save_files)
else :
    if not get_yes("\b\a\a\aWarning!\n %s direcory exists! Overwrite?"
                   % path_to_save_files):
         sys.exit()



# 1. ------ files 
template_file=Load_Text_File(template_fname)
data_files=glob.glob("*.sp")

# 2. ----- find sections
spectrum_section_pos=Check_File_For_Keyword(template_fname, template_file, spectrum_section)
view_section_pos=Check_File_For_Keyword(template_fname, template_file, view_section)


# 3. ----- modify a template
print "Creating Sparky format '.save' files for:"
for current_fname in data_files :
  sp_file=os.path.basename(current_fname)
  print "%s" % current_fname

  #--- name change 
  line_num=spectrum_section_pos+spectrum_name_offset
  Check_If_Line_Is_Right(template_fname, template_file, line_num, spectrum_name_keyword) 
  template_file[line_num]=spectrum_name_keyword+" "+sp_file+"\n"

  #--- pathname change 
  line_num=spectrum_section_pos+spectrum_pathname_offset
  Check_If_Line_Is_Right(template_fname, template_file, line_num, spectrum_pathname_keyword) 
  template_file[line_num]=spectrum_pathname_keyword+" "+os.path.join(os.getcwd(),current_fname)+"\n"

  #---- window name change
  line_num=view_section_pos+view_name_offset
  Check_If_Line_Is_Right(template_fname, template_file, line_num, view_name_keyword) 
  template_file[line_num]=view_name_keyword+" "+sp_file+"\n"

  #--- write it out
  output=open(os.path.join(path_to_save_files,sp_file)+".save",'w')
  output.writelines(template_file)
  output.close()
print "Done...\n"
