#!/usr/bin/env python
doc="""
Making a macro for molmol colorizing dS2 (minimum - grey or sth) on the sausage diagram
"""
import os, sys, string, pickle
import MFselect_lib_2
import mylib, textfile, residues, xmgr
from MFselect_constants import *
import molmol




#--- input filenames ----

# --- difference in S2 for two forms

temp_1="27C"

clip_mode="yes" #yes/no
clip_by=0.08  # everything less than that is set to S2_min value

#----- result from 'ef' in Sparky on 0 and 1.54eq titration
disappeared_peaks=["N71","T100","F120","N44","V43","F46","S18","T45","V47"]


#---- load data on apo-enzyme ---

apo=residues.Annn_Num("R1.27C.dat.apo").make_empty_copy(temp_1+".apo")
fname_1="%s.apo.fixed_averaged_tensor.log.pickled" % temp_1

db_1=mylib.open_file(fname_1,'r')
data_1=pickle.load(db_1)
data_1.Residue_list.set_name(temp_1+"_MFselect_Data")




#---- load data on holo-enzyme ---
holo=residues.Annn_Num("R1.27C.dat.holo").make_empty_copy(temp_1+".holo")
fname_2="%s.inh.fixed_averaged_tensor.log.pickled" % temp_1

db_2=mylib.open_file(fname_2,'r')
data_2=pickle.load(db_2)
data_2.Residue_list.set_name(temp_1+"_MFselect_Data")



#--- find residues without model assignment by ModelFree from both residue lists
apo_unassigned=data_1.show_unassigned()
print "Unassigned in apo protein: ", apo_unassigned
holo_unassigned=data_2.show_unassigned()
print "Unassigned in holo protein: ", holo_unassigned


#-- make residue lists --
apo.get_ModelFree_results(data_1,S2Scurr,S2Ssimerr)
holo.get_ModelFree_results(data_2,S2Scurr,S2Ssimerr)


#--- remove unassigned by ModelFree from both of them (not to have them at all)---
apo.del_keys(apo_unassigned)
apo.del_keys(apo.get_intersection(holo_unassigned).get_keys())

holo.del_keys(holo_unassigned)
holo.del_keys(holo.get_intersection(holo_unassigned).get_keys())


#---  find common and different residues ----
common_res, apo_only, holo_only=residues.keys_analysis(apo,holo)

#-- get dS2: create a new residue list ---
values=residues.Annn_Num()
colors=residues.Annn_Num()

for res in common_res:
##    
##    if s2<S2_min: # clip outliers
##        s2=S2_min

    dS2=holo[res][0]-apo[res][0]
    dS2err=holo[res][1]+apo[res][1]
    
    #---- these are RAW differences not suitable for colorizing ---
    values.add(res,[dS2,dS2err])

    #---- Analysis: paint proportional to excees of value over error
    #   and only for those whose errors less than value
    if abs(dS2)>abs(dS2err):
        x=dS2-dS2err

        #-------- clip outliers ---------------
        if clip_mode=="yes":
            
            if x>clip_by: x=clip_by
    else:
        x=0
        
    colors.add(res,[x])

#print apo
#print holo
#print values
valuesDataset=xmgr.Dataset(values.to_xmgr("ResNum",0,1),"dS2")
valuesDataset.bar()
valuesDataset.blue_sym()
valuesDataset.symbol_size(0.5)

valuesGraph=xmgr.Graph("S2 holo - apo","(positive - residue gets less mobile)")
valuesGraph.add_dataset(valuesDataset)
valuesGraph.set_xaxis("Residue",0,125)
valuesGraph.set_yaxis("dS2(holo-apo)",-0.2,0.2)
valuesGraph.writeout("dS2_raw.xmgr")


#--- this graph is just for control ----
colorsDataset=xmgr.Dataset(colors.to_xmgr("ResNum",0),"dS2-error")
colorsDataset.bar()
colorsDataset.blue_sym()
colorsDataset.symbol_size(0.5)

colorsGraph=xmgr.Graph("S2 holo - apo","(higher value = residue gets less mobile)")
colorsGraph.add_dataset(colorsDataset)
colorsGraph.set_xaxis("Residue",0,125)
colorsGraph.set_yaxis("dS2-error",0,0.13)
colorsGraph.writeout("dS2_above_error.xmgr")



#-- generate a MolMol macro --

import molmol

Rnase=molmol.Protein(124, colors)
Rnase.set_assigned_colors(molmol.White, molmol.Yellow)
Rnase.set_background_color(molmol.DarkGrey)
Rnase.colorize_by_parameter(0)
Rnase.highlight(disappeared_peaks, molmol.Red)
#-- display dS2 ---
Rnase.write_macro("molmol.mac")
