"""
Sparky module to detect potentially overlapping peaks

"""

import residues, keyfile, mylib
import sys,os
import sparky
  

ini_file_text="""# Parameter file for 'detect_overlap' extension
# usu: 1-Nitrogen, 2-proton but check your spectrum yourself

<dw1_ppm> %s

<dw2_ppm> %s

"""

def run(session):

    print "\n\n\n\n******************* Detect Overlap **********************"
    print "Detection of potentially overlapping peaks in 2D spectra"
    session.command_characters("py")
    cur_view=session.selected_view()
    #--- check if window is open ----
    if not cur_view:
        print "\n\n!!! Error!  Need a spectrum selected !!!\n\n"
        return

    cur_spec=cur_view.spectrum
    #-- check if it is 2D !!!! ---
    if cur_spec.dimension!=2:
        print "\n\n!!! Error!  Cannot detect overlaps on %dD spectra !!!\n\n" % cur_spec.dimension
        return
    

    #-- search for .ini file ---
    fname=os.path.join(sparky.user_sparky_directory, 'Python', 'detect_overlap.ini')
    if os.path.isfile(fname):
        ini_file=keyfile.Ini_File(fname)
        dw2min=ini_file.get_float("<dw2_ppm>")
        dw1min=ini_file.get_float("<dw1_ppm>")
        print "Minimal ppm distances used are:"
        
    else:
        dw2min=0.05
        dw1min=0.4
        mylib.open_file(fname,'w').write(ini_file_text % (dw1min, dw2min))
        print "!!! File '%s' not found!\n File created with default values:" % fname
    
    print "delta w1= %s" % dw1min
    print "delta w2= %s" % dw2min      
   
    peak_list=cur_spec.peak_list()
     
    OL_list=[]

    N=len(peak_list)
    
    for i in range(N):
        
        peakA=peak_list[i]

        A_w1=peakA.position[0]
        A_w2=peakA.position[1]

        i=peak_list.index(peakA)
            
        for j in range(i+1, N):
            
            peakB=peak_list[j]

            B_w1=peakB.position[0]
            B_w2=peakB.position[1]
          
            dw1=abs(A_w1-B_w1)
            dw2=abs(A_w2-B_w2)

            if dw1<dw1min and dw2<dw2min:

                if peakA not in OL_list:
                   OL_list.append(peakA)
                   #print resA,

                if peakB not in OL_list:
                   OL_list.append(peakB)
                   #print resB,
            
    session.show_peak_list(OL_list)
    
    print "Done."   
