doc="""
displaying and exporting peak heights w/o fitting

-------------------------------------

This extension is based on Tom's 'relax.py'.

"""



import curvefit3_functions
import curvefit3

import sputil
import pyutil
import relax_dialog

import string
import tkFileDialog
import os, copy
#=================================================================
class no_real_fit(curvefit3.least_squares_fit):
  'just to create the fit object without fitting'
   
  # ---------------------------------------------------------------------------
  def find_fit(self):
     return 1
  # ---------------------------------------------------------------------------
  def calculate_error(self, params):
     return 1
  # ---------------------------------------------------------------------------
  def estimate_parameter_error(self, trials):
     pass
#=================================================================

class data_display(curvefit3.fit_display):
  'just to show the array of data'
  # ---------------------------------------------------------------------------
  #
  def plot_fit(self, heading, fit,  relative_scale, callback = None):

    mark_height= relative_scale # simple measure of height
    
    self.heading['text'] = heading

    x_range, y_range = self.xy_ranges(fit.xy_pairs)

    c = self.canvas
    c.delete('all')
    c.update_idletasks()   # Make sure geometry has been calculated
    w = c.winfo_width()
    h = c.winfo_height()
    r = 5
    
  
    for k in range(len(fit.xy_pairs)):
      x, y, dy = fit.xy_pairs[k]

      px = pyutil.map_interval(x, x_range, (0,w))
      py = pyutil.map_interval(y, y_range, (h,0))
      id = c.create_oval(px-r, py-r, px+r, py+r, fill = 'black')
      

     
      if callback:
        cb = lambda event, fit=fit, k=k, callback=callback: callback(fit, k)
        c.tag_bind(id, '<ButtonPress>', cb)

    yzero=pyutil.map_interval(0, y_range, (h,0))

    #-- prevent the marker going out of range
    if mark_height>y_range[1]:
      ymarkplus=0
    else:
      ymarkplus=pyutil.map_interval(mark_height, y_range, (h,0))

    if mark_height<y_range[0]:
      ymarkminus=h
    else:
      ymarkminus=pyutil.map_interval(-mark_height, y_range, (h,0))

    markplus=c.create_rectangle(0, yzero, 0+2*r, ymarkplus, fill = 'red')
    markminus=c.create_rectangle(0, ymarkminus, 0+2*r, yzero, fill = 'blue')

    c.update_idletasks()
    
  # ---------------------------------------------------------------------------
  #


#=================================================================

def show_fit(session, heading, fit, relative_scale, callback = None):
  d = sputil.the_dialog(data_display, session)
  d.show_window(1)
  d.plot_fit(heading, fit, relative_scale, callback)

#=================================================================

class dummy_function(curvefit3_functions.Any_Func):
  'this is a fake function to allow just data display'
  def fit_data(self,error_estimate_trials=10, epsilon=1e-6):

      self.fit = no_real_fit(self.data, self, self.init_params, epsilon)
  def parameter_count(self):
    return 1
  
  def value(self, params, x):
    return params

#=================================================================


class display_peak_heights_dialog(relax_dialog.relaxation_dialog):
   def __init__(self, session,
                title='Displaying the peak heights vs. time parameter'):
     relax_dialog.relaxation_dialog.__init__(self, session, title)
     
 # ---------------------------------------------------------------------------
  #
   def fit_function(self, th_pairs, assignment):

    h_values = map(lambda th: th[1], th_pairs)
    h_middle = .25 * (max(h_values) + min(h_values))

    
    params = (h_middle,-1,5*h_middle,-50)
    trials = self.error_estimate_trials
    epsilon=1e-10

    #-- run my Any_Function object now -----
    func = dummy_function()
    func.set_data(th_pairs)
    func.set_init_params(params)

    #-- try fitting ---   
    func.fit_data(trials,epsilon)
    return func.fit
  # ---------------------------------------------------------------------------
  #
   def show_fit_cb(self, event):

    fit = self.peak_list.event_line_data(event)
    if fit:
      show_fit(self.session, fit.name, fit, self.relative_scale_on_plots, self.show_peak_cb)
      self.current_fit_index=self.my_fit_list.index(fit)
  # ---------------------------------------------------------------------------
  #
   def show_next_fit(self):

    if self.current_fit_index==-1: return # not initialized yet
    
    if not self.current_fit_index<(len(self.my_fit_list)-1): return # end reached

    self.current_fit_index+=1
    fit=self.my_fit_list[self.current_fit_index]
    show_fit(self.session, fit.name, fit, self.relative_scale_on_plots,  self.show_peak_cb)
  
  # ---------------------------------------------------------------------------
  #
   def show_previous_fit(self):

    if self.current_fit_index==-1: return  # no fit chosen yet

    
    if not self.current_fit_index>0: return # start reached

    self.current_fit_index-=1
    fit=self.my_fit_list[self.current_fit_index]
    show_fit(self.session, fit.name, fit, self.relative_scale_on_plots,  self.show_peak_cb)

  # ---------------------------------------------------------------------------
  #


   def save_brief(self):
    fname=tkFileDialog.asksaveasfilename(initialfile="project.heights",
                                         initialdir=os.path.dirname(self.session.project.save_path),
                                         title="Save as Residue-Rate-Error list")
    if not fname: return
    temp=copy.copy(self.brief_peak_list)
    fit=self.my_fit_list[0]
    temp.name=('ResNum '+string.join(map(str,list(map(lambda xy: xy[0], fit.xy_pairs))),' '))
    temp.writeout(fname, header='ON', format="%.4e", verbose=0)
  
 

#==========================================================================================
   def peak_line(self, peak, fit):

     htext = ''
     for t, h, dh in fit.xy_pairs:
       htext = htext + '%10.4g ' % h
 
     line = ('%3s' % (peak.assignment[:-3])  + htext)
     return line
#==========================================================================================  
   def show_heading(self, spectrum_times):

     self.peak_list.clear()
     heading = '%3s' % ('Assignment')
     for s, t, f in spectrum_times:
#      st = '%s/%.3g' % (s.name, t)
       st = '%g s' %  t 
       heading = heading + '%10s ' % st
     self.peak_list.heading['text'] = heading
#==========================================================================================
   def show_peaks(self, peaks, spectrum_times):

     single_position = self.single_peak_position.get()
     self.stoppable_loop(' peaks', 1)
     for p in self.session.selected_peaks():
       self.check_for_stop()
       if single_position or not p.is_assigned:
         assignment = None
       else:
	 assignment = p.assignment
       th_pairs = self.time_height_pairs(p.position, assignment, spectrum_times)
       fit = self.fit_function(th_pairs, assignment)
       fit.name = p.assignment
       fit.peak_position = p.position
       fit.spectrum_times = spectrum_times
       line = self.peak_line(p, fit)
       self.peak_list.append(line, fit)
       self.peak_list.listbox.see('end')
       #-- append short list for saving and for viewing
       self.my_fit_list.append(fit)
      
       self.brief_peak_list.add(p.assignment[:-3],list(map(lambda xy: xy[1], fit.xy_pairs)))

  # ---------------------------------------------------------------------------
#==========================================================================================


def run(session):
  sputil.the_dialog(display_peak_heights_dialog,session).show_peak_heights()
