#!/usr/bin/env python

doc="""
=================== Extract peak heights to separate files ======================

The utility to convert 'brief lists' from Sparky to files of x,y,dy structure

('es' in Sparky: 'Displaying the peak heights vs. time parameter')

Usage:  es2text.py es_brief_list noise_RMSD [-norm]

            - es_brief_list is a 'Brief list' saved from 'es' command
            - noise_RMSD is assigned as an error bar to the data points
            - -norm flag directs the program to normalize data by
                    value of the first data point

'es' peak list format:
ResNum t1 t2 t3 ....
------------
x1 h1 h2 h3 ...
.
.
.
XNNN

=============================================================
"""

import os, sys, string
import xmgr, mylib, residues

if len(sys.argv)<3 :
    print doc
    sys.exit()
else:
    inpfname=sys.argv[1]
    noise_RMSD=float(sys.argv[2])

outdirname=inpfname+".dat"

infile=mylib.open_file(inpfname,'r').readlines()
if not os.path.isdir(outdirname): os.mkdir(outdirname)

#--- extract times ---
times=map(float,infile[0].split()[1:])

#--- extract heights ---
lenres=len(infile)
#--- create errors dataset ---
errors=[noise_RMSD]*len(times)

for i in range(1,lenres):
    line=infile[i].split()
    resname=line[0]
    print "Extract data for %s" % resname
    heights=map(float,line[1:])

    #---- Normalize --
    if '-norm' in sys.argv:
        for i in range(len(heights)):
            heights[i]/=heights[0]
    
    #--- make XMGR object ---
    xx=xmgr.Dataset(xmgr.make_xydy_matrix(times,heights,errors))
    xx.export_data(os.path.join(outdirname, resname))
##    xx.symbol_size(3)
##    xx.empty_sym()
##    xx.empty_sym_outlinewidth(3)
##    yy=xmgr.Graph(resname,inpfname)
##    yy.add_dataset(xx)
##    yy.add_dataset(x0)
##
##    #-- set axis
##    yy.set_xaxis("time",0,max(times)*1.1,majticksnum=3,charsize=2)
##
##    yrange=max(heights)-min(heights)
##
##    if max(heights)<0:    ymax=noise_RMSD*2
##    else:                 ymax=max(heights)+noise_RMSD*2
##
##    if min(heights)>0:    ymin=-noise_RMSD*2
##    else:                 ymin=min(heights)-noise_RMSD*2
##
##    
##    yy.set_yaxis("I",ymin,ymax,majticksnum=3,minticksnum=2,charsize=2)
##    
##    yy.writeout(os.path.join(outdirname,resname+'.xmgr'))
