#!/usr/bin/env python2

doc="""
=================== peak heights to XMGR ======================

The utility to convert 'brief lists' from Sparky to XMGR files
(in Sparky: 'Displaying the peak heights vs. time parameter')

Usage:  peakheights2xmgr.py peak_height_list noise_RMSD

Peak list format:
ResNum t1 t2 t3 ....
------------
x1 h1 h2 h3 ...
.
.
.
XNNN

=============================================================
"""

import os, sys, string
import xmgr, mylib, residues

if len(sys.argv)!=3 :
    print doc
    sys.exit()
else:
    inpfname=sys.argv[1]
    noise_RMSD=float(sys.argv[2])

outdirname=inpfname+".xmgrs"

infile=mylib.open_file(inpfname,'r').readlines()
if not os.path.isdir(outdirname): os.mkdir(outdirname)

#--- extract times ---
times=map(float,infile[0].split()[1:])

#--- make a 'zero-line' dataset ----
num0s=20
x0times=[0]*num0s
x0step=(max(times)*1.1)/20
for i in range(num0s):
    x0times[i]=x0step*i

x0=xmgr.Dataset(xmgr.make_xydy_matrix(x0times,[0]*len(x0times),[noise_RMSD*1.5]*len(x0times)))
x0.symbol_size(1)
x0.square()

#--- extract heights ---
lenres=len(infile)

for i in range(2,lenres):
    line=infile[i].split()
    resname=line[0]
    print "Extract data for %s" % resname
    heights=map(float,line[1:])


    #--- make XMGR object ---
#    xx=xmgr.Dataset(xmgr.make_xydy_matrix(times,heights,[noise_RMSD*1.5]*len(times)))
    xx=xmgr.Dataset(xmgr.make_xy_matrix(times,heights))
    xx.symbol_size(3)
    xx.empty_sym()
    xx.empty_sym_outlinewidth(3)
    yy=xmgr.Graph(resname,inpfname)
    yy.add_dataset(xx)
    yy.add_dataset(x0)

    #-- set axis
    yy.set_xaxis("time",0,max(times)*1.1,majticksnum=3,charsize=2)

    yrange=max(heights)-min(heights)

    if max(heights)<0:    ymax=noise_RMSD*2
    else:                 ymax=max(heights)+noise_RMSD*2

    if min(heights)>0:    ymin=-noise_RMSD*2
    else:                 ymin=min(heights)-noise_RMSD*2

    
    yy.set_yaxis("I",ymin,ymax,majticksnum=3,minticksnum=2,charsize=2)
    
    yy.writeout(os.path.join(outdirname,resname+'.xmgr'))
