#!/usr/bin/env python
import string,  glob, os, sys
doc="""===== Generate an HTML page for a list of residues with corresponding graphics ====
by Evgenii Kovrgin,
Medical College of Wisconsin
2/18/09

Usage:
1. Generate an HTML line with one row with all pictures
   from appropriate locations so replacement of the residue name
   would correctly access pictures for a new residue. DO NOT USE
   LISTS OR TABLES!!! THEY CONSUME TOO MUCH MEMORY !!!!
   
2. Copy HTML code for this row in this program below and add appropriate 
   short title (what appears on a page tab), page title and subtitle.

3. Find-Replace your specific residue name you had with the generic word RESIDUE

4. Run:   generate_page.py  residue list  
    where
            residue list contains a column of residue names in a single-letter
            amino acid code  like "T2". No header or footer lines!

        
5. Move HTML page into the folder where you created an HTML template so 
   paths are correct.
   
6. Use Safari to view the page (Firefox uses too much memory).


"""

#================ USER INPUT =============================
short_title="EKN60 R2 graphs, 600 MHz, 20C, two protocols"

title="Comparison of R2 obtained from fitting of relaxation data processed with two alternative protocols"

subtitle="600 MHz, 20C"

output_fname="EKN60.Intensity_Fitting.HighRes_LowRres_comparison.htm"


template_word='RESIDUE'


HTML_template="""
<h2><strong>RESIDUE</strong></h2>
<h3>Low resolution, high S/N&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; High resolution, low S/N<br />
  <img src="../../Data.XV/EKN59.15N_KRas_relaxation.20C/data/rcCPMG/added.LowRes/sp/Sparky/55.ekn59.KRasGNP.added.20C.TCI.500.sp.save.peaks/PNG/RESIDUE.png" width="400" height="300" />
  <img src="../../Data.XV/EKN59.15N_KRas_relaxation.20C/data/rcCPMG/added.HighRes/sp/Sparky/55.ekn59.KRasGNP.added.20C.TCI.500.sp.save.peaks/PNG/RESIDUE.png" width="400" height="300" /><br />
  <img src="../../Data.XV/EKN59.15N_KRas_relaxation.20C/data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.500.20C.120ppm.txt.data/RESIDUE/RESIDUE_RelaxationDispersion.small.png" width="400" height="300" /><img src="../../Data.XV/EKN59.15N_KRas_relaxation.20C/data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.500.20C.120ppm.txt.data/RESIDUE/RESIDUE_RelaxationDispersion.small.png" width="400" height="300" />
  <img src="../../Data.XV/EKN59.15N_KRas_relaxation.20C/data/rcCPMG/two-protocol-analysis/comparison/PNGsmall/RESIDUE.png" width="400" height="300" /></h3>
<hr />
<h3>Low resolution, high S/N&nbsp;<br />
  <img src="../../Data.XV/EKN59.15N_KRas_relaxation.20C/data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.500.20C.120ppm.txt.data/RESIDUE/RESIDUE_RelaxationDispersion.small.png" width="400" height="300" /><img src="data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.600.20C.120ppm.txt.data/RESIDUE/R2fits/83.png" width="400" height="300" /><img src="data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.600.20C.120ppm.txt.data/RESIDUE/R2fits/100.png" width="400" height="300" /><img src="data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.600.20C.120ppm.txt.data/RESIDUE/R2fits/125.png" width="400" height="300" /><img src="data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.600.20C.120ppm.txt.data/RESIDUE/R2fits/151.png" width="400" height="300" /><img src="data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.600.20C.120ppm.txt.data/RESIDUE/R2fits/175.png" width="400" height="300" /><img src="data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.600.20C.120ppm.txt.data/RESIDUE/R2fits/200.png" width="400" height="300" /><img src="data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.600.20C.120ppm.txt.data/RESIDUE/R2fits/250.png" width="400" height="300" /><img src="data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.600.20C.120ppm.txt.data/RESIDUE/R2fits/384.png" width="400" height="300" /><img src="data/rcCPMG/added.LowRes/sp/Sparky/eS.LR.600.20C.120ppm.txt.data/RESIDUE/R2fits/555.png" width="400" height="300" /></h3>
<hr />
<h3>&nbsp; High resolution, low S/N<br />
<img src="../../Data.XV/EKN59.15N_KRas_relaxation.20C/data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.500.20C.120ppm.txt.data/RESIDUE/RESIDUE_RelaxationDispersion.small.png" width="400" height="300" /><img src="data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.600.20C.120ppm.txt.data/RESIDUE/R2fits/83.png" width="400" height="300" /><img src="data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.600.20C.120ppm.txt.data/RESIDUE/R2fits/100.png" width="400" height="300" /><img src="data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.600.20C.120ppm.txt.data/RESIDUE/R2fits/125.png" width="400" height="300" /><img src="data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.600.20C.120ppm.txt.data/RESIDUE/R2fits/151.png" width="400" height="300" /><img src="data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.600.20C.120ppm.txt.data/RESIDUE/R2fits/175.png" width="400" height="300" /><img src="data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.600.20C.120ppm.txt.data/RESIDUE/R2fits/200.png" width="400" height="300" /><img src="data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.600.20C.120ppm.txt.data/RESIDUE/R2fits/250.png" width="400" height="300" /><img src="data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.600.20C.120ppm.txt.data/RESIDUE/R2fits/384.png" width="400" height="300" /><img src="data/rcCPMG/added.HighRes/sp/Sparky/eS.HR.600.20C.120ppm.txt.data/RESIDUE/R2fits/555.png" width="400" height="300" /></h3>
<hr />

<hr />

"""






#================= END OF USER INPUT =====================



HTML_header="""
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title>%s</title>
<style type="text/css">
<!--
body {
	background-color: #CCCCCC;
}
body,td,th {
	font-family: Verdana, Arial, Helvetica, sans-serif;
}
-->
</style></head>

<body>
<h1>%s</h1>
<h2>%s</h2>
<p>&nbsp;</p>
<hr />
<p>&nbsp;</p>
<p>&nbsp;</p>
""" % (short_title, title, subtitle)



HTML_footer="""
<p>&nbsp;</p>
<p>&nbsp;</p>
<hr />
<p>Generated by <strong>GENERATE_PAGE.PY</strong> </p>
<p>&nbsp;</p>
<p>&nbsp;</p>
</body>
</html>
"""



if len(sys.argv)<2:
    print doc
    sys.exit()

    
reslist_fname=sys.argv[1]

finp=open(reslist_fname,'r')
foutp=open(output_fname,'w')

residues=finp.readlines()


#-- make an new HTML row
print "Making HTML for: "
counter=1
HTML_body=""
for line in residues:
  #-- extract residue name
  resname=string.strip(line)
  if not resname : continue  #- skip empty lines
  
  print "%d:   %s" % (counter, resname)
  
  
  #-- replace template word with a residue name
  HTML_body+= string.join(HTML_template.split(template_word),resname)
  
  counter+=1
  
  
foutp.write(HTML_header+HTML_body+HTML_footer)
  
finp.close()
foutp.close()

print '\n"%s" written.\n' % output_fname
  
  
  
  
  
  
  
  
  
  
  
  































