#!/usr/bin/env python
import os,sys,string, math, thread
import residues
import mylib, xmgr,  keyfile
from run_parallel import *

class Get_Exp(SFTP):
    'downloading experiments from the spectrometer'

    #-----------------------------------------------------


    
    def shell(self, exit_process_command="exit\n", stop_shell_command="bye"):
        'interactive shell for tuning the interface to the process'
        
        print "Download of experiments from '%s'" % self.proc_name
        print '-'*40
        HELP_REQUEST="?"
        LOGFILE_REQUEST="log"
        CHANGE_LOCAL_DIR="cd"
        DIR_LIST='ls'

        help_text="""
        Your input is from this window. 
        --------------------
        Available commands:

        get N - 1. download experiment N and save
                2. save into a subdirectory in the current dir
                   given after <file> keyword in the 'text' file
                   of the experiment. If no keyword found - just save
                   in the directory named 'expNN'
                3. save a logfile from 'watch_magnet.py' named 'expNN.log'
                   to the same directory if exist
                4. show it and show the 'log' file of the experiment
        
        %s      - exit the program 
        %s      - print this text
        %s      - get log file name
        %s      - change local directory
        %s      - list current directory
        
        
        """ % (stop_shell_command, HELP_REQUEST,
               LOGFILE_REQUEST, CHANGE_LOCAL_DIR,
               DIR_LIST)
        print help_text

        for line in self.outp_list:
            print line
            
        thread.start_new(self._shell_input_reader,(stop_shell_command,))
        self.input_prompt()
        mylock.acquire()
        while self.exit_requested==NO:
            # check if new commands were received: only first one is considered!!!
            self._flush_outp()
            self._flush_user_input()
            
            mylock.release()
            time.sleep(1)
            mylock.acquire()

            if len(self.user_input)>0:
                #-- look for request to stop program --
                if stop_shell_command == self.user_input[0].strip():
                    self.exit_requested=YES
                #-- help requested
                elif HELP_REQUEST == self.user_input[0].strip():
                    print help_text
                    
                #-- log file name
                elif LOGFILE_REQUEST == self.user_input[0].strip():
                    print os.path.join(os.getcwd(),self.log_fname)

                #-- change local directory ---
                elif CHANGE_LOCAL_DIR in self.user_input[0].split():
                    if len(self.user_input[0].split())!=2:
                        print "\a\a\nProvide 'cd path' to change directory!\n"
                    else:
                        to_dir=self.user_input[0][2:].strip()
                        if os.path.isdir(to_dir):
                            #-- remove 'cd' from input line --
                            os.chdir(to_dir)
                            self.command("lcd "+to_dir)
                            print "\nCurrent directory now: %s\n" % os.getcwd()
                        else:
                            print "\a\a\a\nDirectory '%s' doesn't exist!!!\n" % to_dir

                #-- list local directory ---
                elif DIR_LIST == self.user_input[0].strip():
                        dir_list=os.popen("ls -l").readlines()
                        print
                        print "-"*20
                        print "Current directory: \n%s\n" % os.getcwd(),
                        print "-"*20
                        for line in dir_list:
                            print line[:-1]
                        print "-"*20


                    
                #-- execute it ---
                elif (len(self.user_input[0].split()) >0) and (self.user_input[0].split()[0]=="get"):
                    #-- get experiment ---

                    if len(self.user_input[0].split())!=2:
                        print "Usage: get NN\n where NN is experiment number"
                        self.input_prompt()
                        continue
                    expnum=self.user_input[0].split()[1]


                    
                    print '\n','*'*60    
                    print "\n--- Getting experiment %s ---\n" % expnum
                    path="vnmrsys/exp%s/acqfil\n" % expnum
                    self.command("cd %s" % path)
                    self.command("pwd")

                    self.command("version")
                    self.wait_for_keyword(mylock,"version")

                    #--- check for error messages --
                    if mylib.Find_String_Num(self.outp_list,"Couldn't")!=-1:
                        print "\n\n\a\aNo such experiment '%s'!!!\n" % path
                        self.input_prompt()
                        continue


                    self._flush_outp()                     
                    self.command("get *")
                    self.command("cd ../../..")
                    self.command("version")
                    self.wait_for_keyword(mylock,"version")


                    # --- was it all right ? --------
                    if not(os.path.isfile("fid") and
                           os.path.isfile("log") and
                           os.path.isfile("procpar") and
                           os.path.isfile("text")):
                        print "\a\aWarning! Download is incomplete!!!"
                        self.input_prompt()
                        continue
                    print "Download complete\n"



                    # ---- get the pulse sequence file ----
                    procpar_text=mylib.Load_Text_File('procpar')
                    seqfil_word='seqfil'
                    sequence_name="None"

                    for linenum in range(len(procpar_text)):
                        firstWord=string.split(procpar_text[linenum],' ')[0]
                        if firstWord==seqfil_word:
                            sequence_name=string.split(
                                            procpar_text[linenum+1])[1][1:-1]+".c"
                            print "Pulse sequence used: '%s', downloading it..." % sequence_name

                            self._flush_outp()
                            self.command("get vnmrsys/psglib/%s" % sequence_name)
                            self.command("version")
                            self.wait_for_keyword(mylock,"version")

                            #--- check for error messages --
                            if mylib.Find_String_Num(self.outp_list,"Couldn't")!=-1:
                                print "\n\n\a\aSequence '%s' not found in vnmrsys/psglib!!!\n" % sequence_name
                            else:
                                #--- All right, sequence found ---
                                #--- Check if download is complete
                                if not(os.path.isfile(sequence_name)):
                                    print "\n\n\a\aWarning! Sequence download is incomplete!!!\n"
                                    sequence_name="None"
                            #-- sequence downloaded or None
                            break
                    else:
                        print "The word '%s' was not found in the 'procpar' file!!!\n" % seqfil_word 
                        print "Pulse sequence NOT downloaded!!!\a\a\a\n\n"
                        sequence_name="None"
                        



                    #-- determine the destination dir name --
                    try:
                        localdir=keyfile.Keyword_File(os.curdir,"text").get_string("<file>")
                    except UserWarning:
                        localdir="exp%s" % expnum
                    print "Saving data to the local directory:\n%s\n -> %s/\n" % (os.getcwd(),localdir)
        

                    #--- checking for existence of this directory (in case when parameters
                    #    were simply moved from one experiment to another without checking the
                    #    text file
                    if os.path.isdir(localdir):
                        oldlocaldir=localdir
                        localdir="exp%s" % expnum   # save in just 'expN' directory
                        direxistflag=1
                    else:
                        direxistflag=0

                    #-- move files to a requested local directory ---
                    mylib.deep_mkdir(localdir)
                    fid=os.path.join(localdir,"fid")
                    procpar=os.path.join(localdir,"procpar")
                    log=os.path.join(localdir,"log")
                    text=os.path.join(localdir,"text")
                    os.rename("fid",fid)
                    os.rename("text",text)
                    os.rename("log",log)
                    os.rename("procpar",procpar)
                    #--- move pulse sequence file if exists ----
                    if sequence_name!="None":
                        sequence_final_name=os.path.join(localdir,sequence_name)
                        os.rename(sequence_name,sequence_final_name)
                        sequence_name="None"  #-- recycle it for the next experiment

                    #-- 'watch_magnet.py' log if exists ----
                    wm_log="exp%s.log" % expnum
                    moved_wm_log=os.path.join(localdir,wm_log)
                    if os.path.isfile(wm_log):
                        os.rename(wm_log,moved_wm_log)



                    #--- displaying logs -------
                    print "\n-------------------- TEXT ------------------------"
                    os.system("cat " + text)
                    print "\n-------------------- LOG -------------------------"
                    os.system("cat " + log)

                    if os.path.isfile(moved_wm_log):
                        print "\n-------------- WATCH_MAGNET LOG -------------------"
                        os.system("cat " + moved_wm_log)
                    print "*"*60

                    #--- Notifying a user about new directory name 
                    if direxistflag:
                        print "-"*40
                        print "Warning! Directory '%s' exists!\n" % oldlocaldir
                        print "Saving data to '%s' directory!!!\n" % localdir
                        print "-"*40



                    #---- command executed. Delete it ----
                    print "\n%s>>>" % self.proc_name,
                    del self.user_input[0]
                    
                elif not len(self.user_input[0].strip()): pass

                else:
                    print "\n\a\a\aUnknown command '%s'!\n" % self.user_input[0][:-1]
                    
                self.input_prompt() 
           
        else:
            self.command(exit_process_command)
            print "Request to stop the program was received.\n Process '%s' has been requested to exit..." % self.proc_name
#------------------------------------------------------------------------

doc="""************** Get Experiments *******************

v.2.1 by Evgenii Kovrigin         6/11/03

The SFTP-based shell for downloading experimental data from
the specified spectrometer.

Usage:  get_exp.py   host_name  seconds_to_enter_password

        (example: i600  60)

        - seconds_to_enter_password  is an interval my program
          sleeps allowing you to talk straight to 'sftp'. After
          this interval is over my program begins to interscept
          user input.

        - if the response of the server was too slow and the main
           thread of the program began working - just type 'bye'
           and restart the program with increased sleeping time

        - if the program crashs or stopped by Ctrl-Z - you need
          to kill 'python' and 'sftp' manually


*******************************************************
"""

import sys,os, string
import mylib, keyfile

if len(sys.argv)<3:
	print doc
	sys.exit()


host=sys.argv[1]
sleep_time=int(sys.argv[2])

a=Get_Exp(host, sleep_time)
a.shell()
