doc="""grace.py  v.1   A simple GRACE file builder (GRACE is current version of XMGR) 

  There are to classes:
  Dataset - creates an object to accomodate data and set their presentation
            properties
            x=Dataset(array, legend)
               where array is either (x,y),(x2,y2)... or (x,y,dy),(x2,y2,dy2)...
            if you have x, y and dy arrays -
            use make_xy_matrix(x,y) and  make_xydy_matrix(x,y)
            functions to transpose your data


  Graph - creates a file for GRACE. Many datasets can be added.
            If you are lazy to set x,y axis - just click Autoscale
            in GRACE then
"""

def make_xy_matrix(x_list,y_list):
    '(x1,x2,..) and (y1,y2,...) -> ((x1,y1),(x2,y2),...)'
    if len(x_list)!=len(y_list):
        raise UserWarning, (
            "Size of X data (%d elements) is unequal to Y data (%d elements)!" %
            (len(x_list), len(y_list)))
    new_list=[]
    for i in range(len(x_list)):
        new_list.append([x_list[i],y_list[i]])
    return new_list

def make_xydy_matrix(x_list, y_list, dy_list):
    '(x1,x2..),(y1,y2..),(dy1,dy2..)->((x1,y1),(x2,y2),...)'
    if len(x_list)!=len(y_list) or len(y_list)!=len(dy_list):
        raise UserWarning, (
            "Unequal data sizes:\n\
X data - %d elements, Y data - %d elements), dY data - %d elements)!" %
            (len(x_list), len(y_list), len(dy_list)))
    new_list=[]
    for i in range(len(x_list)):
        new_list.append([x_list[i],y_list[i],dy_list[i]])
    return new_list




import os, sys, string

class Dataset:
    'keeps experimental data and their presentation parameters: (x,y),(x2,y2)... or (x,y,dy),(x2,y2,dy2)'        
    # constructor for matrix xy data
    def __init__(self,data_list, legend=""):
        # acquire data
        self.data=data_list
        self.legend=legend
        # default presentation parameters
        self.symbol=2
        self.size=0.5
        self.fill=1
        self.symb_color=1
        self.symb_linewidth=1
        self.symb_linestyle=1

        self.line_type=05
        self.linestyle=1
        self.linewidth=1
        self.linecolor=1


#------------------------------
    def set_legend(self,new_legend):
        self.legend=new_legend
        
#-------- properties -----------
    # symbol
    def no_sym(self): self.symbol=0
    def dot(self): self.symbol=1
    def circle(self): self.symbol=2
    def square(self): self.symbol=3
    def diamond(self): self.symbol=4
    def triangle_up(self): self.symbol=5
    def star(self): self.symbol=11
    def bar(self): self.symbol=20
    # symbol color
    def black_sym(self): self.symb_color=1
    def red_sym(self): self.symb_color=2
    def green_sym(self): self.symb_color=3
    def blue_sym(self): self.symb_color=4
    # symbol type
    def filled_sym(self): self.fill=1
    def empty_sym(self): self.fill=0
    def empty_sym_outlinewidth(self,width): self.symb_linewidth=width
    # symbol size
    def symbol_size(self,factor):
        '1,2,3,... or fraction'
        self.size=self.size*factor
    
    # line
    def no_line(self): self.line_type=0
    def solid_line(self): self.line_type=1
    def dotted_line(self): self.line_type=2
    def dash_line(self): self.line_type=3
    def long_dash_line(self): self.line_type=4
    def dot_dash_line(self): self.line_type=5
    # line properties
    def line_width(self, factor):
        '1,2,3,...'
        self.linewidth=self.linewidth*factor
    # line color
    def black_line(self): self.linecolor=1
    def red_line(self): self.linecolor=2
    def green_line(self): self.linecolor=3
    def blue_line(self): self.linecolor=4

     
    #--- Exporting numerical data -------
    def export_data(self, fname=""):
        'output the numeric data as a text file'
        if not fname:   filename=self.legend
        else:           filename=fname

        outfile=open(filename,'w')
        for line in self.data:
            outline=string.join(map(str,line),' ')
            outfile.write(outline+'\n')
        outfile.close()
        
        
class Graph:
    'GRACE file builder'
#-----------------------------------------------------        
    def __init__(self, title="", subtitle=""):
        'creates GRACE graph object'
        self.path_to_gracebat="""/Applications/Grace.app/Contents/Resources/grace/bin/"""
        # titles
        self.title=title
        self.subtitle=subtitle
        # coordinates
        self.xmin=0.0
        self.xmax=1.0
        self.ymin=0.0
        self.ymax=1.0
        self.xaxis_label="X"
        self.yaxis_label="Y"
        self.xmajticks=0.2
        self.xminticks=0.1
        self.ymajticks=0.2
        self.yminticks=0.1
        self.y_charsize=1
        self.x_charsize=1
        #error bar defaults
        self.err_type="BOTH"
        self.err_len=0.5
        self.err_lnwd=1
        self.err_lnst=1
        self.err_riser="on"

        self.xscale='Normal'
        self.yscale='Normal'

        # empty
        self.datasets=[]
        # legend position
        self.legend_x=0.9
        self.legend_y=0.8
#----------------------------------------------------
    def add_dataset(self, dataset):
        self.datasets.append(dataset)
        
    def remove_dataset(self, dataset):
        self.datasets.remove(dataset)

#--------------------------------------------------
    def header(self):
        'some mandatory stuff'
        return """# Grace project file
#
@version 50116
@page size 792, 612
@page scroll 5%
@page inout 5%
@link page off
@map font 0 to "Times-Roman", "Times-Roman"
@map font 1 to "Times-Italic", "Times-Italic"
@map font 2 to "Times-Bold", "Times-Bold"
@map font 3 to "Times-BoldItalic", "Times-BoldItalic"
@map font 4 to "Helvetica", "Helvetica"
@map font 5 to "Helvetica-Oblique", "Helvetica-Oblique"
@map font 6 to "Helvetica-Bold", "Helvetica-Bold"
@map font 7 to "Helvetica-BoldOblique", "Helvetica-BoldOblique"
@map font 8 to "Courier", "Courier"
@map font 9 to "Courier-Oblique", "Courier-Oblique"
@map font 10 to "Courier-Bold", "Courier-Bold"
@map font 11 to "Courier-BoldOblique", "Courier-BoldOblique"
@map font 12 to "Symbol", "Symbol"
@map font 13 to "ZapfDingbats", "ZapfDingbats"
@map color 0 to (255, 255, 255), "white"
@map color 1 to (0, 0, 0), "black"
@map color 2 to (255, 0, 0), "red"
@map color 3 to (0, 255, 0), "green"
@map color 4 to (0, 0, 255), "blue"
@map color 5 to (255, 255, 0), "yellow"
@map color 6 to (188, 143, 143), "brown"
@map color 7 to (220, 220, 220), "grey"
@map color 8 to (148, 0, 211), "violet"
@map color 9 to (0, 255, 255), "cyan"
@map color 10 to (255, 0, 255), "magenta"
@map color 11 to (255, 165, 0), "orange"
@map color 12 to (114, 33, 188), "indigo"
@map color 13 to (103, 7, 72), "maroon"
@map color 14 to (64, 224, 208), "turquoise"
@map color 15 to (0, 139, 0), "green4"
@reference date 0
@date wrap off
@date wrap year 1950
@default linewidth 1.0
@default linestyle 1
@default color 1
@default pattern 1
@default font 0
@default char size 1.000000
@default symbol size 1.000000
@default sformat "%.8g"
@background color 0
@page background fill on
@timestamp off
@timestamp 0.03, 0.03
@timestamp color 1
@timestamp rot 0
@timestamp font 0
@timestamp char size 1.000000
@timestamp def "Thu Jul 24 08:09:01 2008"
@r0 off
@link r0 to g0
@r0 type above
@r0 linestyle 1
@r0 linewidth 1.0
@r0 color 1
@r0 line 0, 0, 0, 0
@r1 off
@link r1 to g0
@r1 type above
@r1 linestyle 1
@r1 linewidth 1.0
@r1 color 1
@r1 line 0, 0, 0, 0
@r2 off
@link r2 to g0
@r2 type above
@r2 linestyle 1
@r2 linewidth 1.0
@r2 color 1
@r2 line 0, 0, 0, 0
@r3 off
@link r3 to g0
@r3 type above
@r3 linestyle 1
@r3 linewidth 1.0
@r3 color 1
@r3 line 0, 0, 0, 0
@r4 off
@link r4 to g0
@r4 type above
@r4 linestyle 1
@r4 linewidth 1.0
@r4 color 1
@r4 line 0, 0, 0, 0
@g0 on
@g0 hidden false
@g0 type XY
@g0 stacked false
@g0 bar hgap 0.000000
@g0 fixedpoint off
@g0 fixedpoint type 0
@g0 fixedpoint xy 0.000000, 0.000000
@g0 fixedpoint format general general
@g0 fixedpoint prec 6, 6
"""
#------------------------------------------------------
    def _get_position(self):
        return """@with g0
@    world %f, %f, %f, %f 
@    stack world 0, 0, 0, 0
@    znorm 1
@    view 0.150000, 0.150000, 1.150000, 0.850000
""" % (self.xmin, self.ymin, self.xmax, self.ymax)



#------------------------------------------------------
    def set_title(self,title="", subtitle=""):
        self.title=title
        self.subtitle=subtitle
#-------------------------------------------------------
    def _get_title(self):
        return ("""@    title "%s"
@    title font 4
@    title size 1.1
@    title color 1
@    subtitle "%s"
@    subtitle font 4
@    subtitle size 0.8
@    subtitle color 1
""" % (self.title, self.subtitle))

        
#-----------------------------------------------------------------
    def _get_legend(self):
        return """@    legend on
@    legend loctype view
@    legend %f, %f
@    legend box color 1
@    legend box pattern 1
@    legend box linewidth 1.0
@    legend box linestyle 1
@    legend box fill color 0
@    legend box fill pattern 1
@    legend font 4
@    legend char size 1.000000
@    legend color 1
@    legend length 4
@    legend vgap 1
@    legend hgap 1
@    legend invert false
""" % (self.legend_x,self.legend_y)

#--------------------------------------------------

    def _get_scales(self):
        return """@    xaxes scale %s
@    yaxes scale %s
@    xaxes invert off
@    yaxes invert off
""" % (self.xscale, self.yscale)

#---------------------------------------------------------
    def set_xaxis(self,label, xmin, xmax, majticks_step=-1, minticksnum=2, charsize=1,scale='Normal'):
        self.xaxis_label=label
        self.xmin=float(xmin)
        self.xmax=float(xmax)
        
        if majticks_step==-1:
            self.xmajticks_step=(self.xmax-self.xmin)/6
        else:
            self.xmajticks_step=majticks_step
            
        self.xminticksnum=minticksnum
        self.x_charsize=charsize
        self.xscale=scale
        
    def _get_xaxis(self):
        return ("""@    xaxis  on
@    xaxis  type zero false
@    xaxis  offset 0.000000 , 0.000000
@    xaxis  bar on
@    xaxis  bar color 1
@    xaxis  bar linestyle 1
@    xaxis  bar linewidth 1.0
@    xaxis  label "%s"
@    xaxis  label layout para
@    xaxis  label place auto
@    xaxis  label char size 1.000000
@    xaxis  label font 4
@    xaxis  label color 1
@    xaxis  label place normal
@    xaxis  tick on
@    xaxis  tick major %f
@    xaxis  tick minor ticks %d
@    xaxis  tick default 6
@    xaxis  tick place rounded true
@    xaxis  tick in
@    xaxis  tick major size 1.000000
@    xaxis  tick major color 1
@    xaxis  tick major linewidth 1.0
@    xaxis  tick major linestyle 1
@    xaxis  tick major grid off
@    xaxis  tick minor color 1
@    xaxis  tick minor linewidth 1.0
@    xaxis  tick minor linestyle 1
@    xaxis  tick minor grid off
@    xaxis  tick minor size 0.500000
@    xaxis  ticklabel on
@    xaxis  ticklabel format general
@    xaxis  ticklabel prec 5
@    xaxis  ticklabel formula ""
@    xaxis  ticklabel append ""
@    xaxis  ticklabel prepend ""
@    xaxis  ticklabel angle 0
@    xaxis  ticklabel skip 0
@    xaxis  ticklabel stagger 0
@    xaxis  ticklabel place normal
@    xaxis  ticklabel offset auto
@    xaxis  ticklabel offset 0.000000 , 0.010000
@    xaxis  ticklabel start type auto
@    xaxis  ticklabel start 0.000000
@    xaxis  ticklabel stop type auto
@    xaxis  ticklabel stop 0.000000
@    xaxis  ticklabel char size %f
@    xaxis  ticklabel font 4
@    xaxis  ticklabel color 1
@    xaxis  tick place both
@    xaxis  tick spec type none
"""        % ( self.xaxis_label, self.xmajticks_step, self.xminticksnum, self.x_charsize))
#---------------------------------------------------------

#---------------------------------------------------------
    def set_yaxis(self,label, ymin, ymax, majticks_step=-1, minticksnum=2, charsize=1,scale='Normal'):
        self.yaxis_label=label
        self.ymin=float(ymin)
        self.ymax=float(ymax)
        
        if majticks_step==-1:
            self.ymajticks_step=(self.ymax-self.ymin)/6
        else:
            self.ymajticks_step=majticks_step
            
        self.yminticksnum=minticksnum
        self.y_charsize=charsize
        self.yscale=scale
        
    def _get_yaxis(self):
        return ("""@    yaxis  on
@    yaxis  type zero false
@    yaxis  offset 0.000000 , 0.000000
@    yaxis  bar on
@    yaxis  bar color 1
@    yaxis  bar linestyle 1
@    yaxis  bar linewidth 1.0
@    yaxis  label "%s"
@    yaxis  label layout para
@    yaxis  label place auto
@    yaxis  label char size 1.000000
@    yaxis  label font 4
@    yaxis  label color 1
@    yaxis  label place normal
@    yaxis  tick on
@    yaxis  tick major %f
@    yaxis  tick minor ticks %d
@    yaxis  tick default 6
@    yaxis  tick place rounded true
@    yaxis  tick in
@    yaxis  tick major size 1.000000
@    yaxis  tick major color 1
@    yaxis  tick major linewidth 1.0
@    yaxis  tick major linestyle 1
@    yaxis  tick major grid off
@    yaxis  tick minor color 1
@    yaxis  tick minor linewidth 1.0
@    yaxis  tick minor linestyle 1
@    yaxis  tick minor grid off
@    yaxis  tick minor size 0.500000
@    yaxis  ticklabel on
@    yaxis  ticklabel format general
@    yaxis  ticklabel prec 5
@    yaxis  ticklabel formula ""
@    yaxis  ticklabel append ""
@    yaxis  ticklabel prepend ""
@    yaxis  ticklabel angle 0
@    yaxis  ticklabel skip 0
@    yaxis  ticklabel stagger 0
@    yaxis  ticklabel place normal
@    yaxis  ticklabel offset auto
@    yaxis  ticklabel offset 0.000000 , 0.010000
@    yaxis  ticklabel start type auto
@    yaxis  ticklabel start 0.000000
@    yaxis  ticklabel stop type auto
@    yaxis  ticklabel stop 0.000000
@    yaxis  ticklabel char size %f
@    yaxis  ticklabel font 4
@    yaxis  ticklabel color 1
@    yaxis  tick place both
@    yaxis  tick spec type none
@    altxaxis  off
@    altyaxis  off
"""        % ( self.yaxis_label, self.ymajticks_step, self.yminticksnum, self.y_charsize))

#----------
    def _get_frame(self):
        return """@    frame type 0
@    frame linestyle 1
@    frame linewidth 1.0
@    frame color 1
@    frame pattern 1
@    frame background color 0
@    frame background pattern 0
"""

#---------------------------------------------------------

    def place_legend(self,x,y):
        'x and y as fractions: 0,0 is lower left corner'
        self.legend_x=x
        self.legend_y=y



#---------------------------------------------------------
    def set_errorbars(self, type, length, line_width=1, line_style=1, riser="on"):
        'set error bars'
        self.err_type=type
        self.err_len=length
        self.err_lnwd=line_width
        self.err_lnst=line_style
        self.err_riser=riser
        
#--------------------------------------------------------------------


    def _get_datasets(self):
        ' exports datasets presentation parameters and data'
        out_string=""

        if self.datasets : # there is something to print
            # export parameters
            dataset_number=0
            for set in self.datasets:
                out_string=(out_string+"""@    s%s hidden false
@    s%s type xydy
@    s%s symbol %d
@    s%s symbol size %f
@    s%s symbol color %d
@    s%s symbol pattern 1
@    s%s symbol fill color %d
@    s%s symbol fill pattern %d
@    s%s symbol linewidth %f
@    s%s symbol linestyle %d
@    s%s symbol char 65
@    s%s symbol char font 0
@    s%s symbol skip 0
@    s%s line type %d
@    s%s line linestyle %d
@    s%s line linewidth %f
@    s%s line color %d
@    s%s line pattern 1
@    s%s baseline type 0
@    s%s baseline off
@    s%s dropline off
@    s%s fill type 0
@    s%s fill rule 0
@    s%s fill color 1
@    s%s fill pattern 1
@    s%s avalue off
@    s%s avalue type 2
@    s%s avalue char size 1.000000
@    s%s avalue font 0
@    s%s avalue color 1
@    s%s avalue rot 0
@    s%s avalue format general
@    s%s avalue prec 3
@    s%s avalue prepend ""
@    s%s avalue append ""
@    s%s avalue offset 0.000000 , 0.000000
@    s%s errorbar on
@    s%s errorbar place both
@    s%s errorbar color %d
@    s%s errorbar pattern 1
@    s%s errorbar size %f
@    s%s errorbar linewidth %f
@    s%s errorbar linestyle 1
@    s%s errorbar riser linewidth 1.0
@    s%s errorbar riser linestyle 1
@    s%s errorbar riser clip off
@    s%s errorbar riser clip length 0.100000
@    s%s comment "Editor"
@    s%s legend  "%s"
""" % (
                dataset_number,
                dataset_number,
                dataset_number,set.symbol,
                dataset_number, set.size,
                dataset_number, set.symb_color,
                dataset_number,
                dataset_number, set.symb_color,
                dataset_number, set.fill,
                dataset_number, set.symb_linewidth,
                dataset_number, set.symb_linestyle,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number, set.line_type,
                dataset_number,  set.linestyle,
                dataset_number,  set.linewidth,
                dataset_number, set.linecolor,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number, set.symb_color,
                dataset_number,
                dataset_number, self.err_len,
                dataset_number, self.err_lnwd,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number,
                dataset_number, set.legend))
                                
                #-- increase counter
                dataset_number=dataset_number+1

                                
                                
                #------------------ export numeric data ---------------------
               
            dataset_number=0
            for set in self.datasets:
                # accept (x,y) and (x,y,dy)
                if len(set.data[0])>2:
                    a="dy\n"
                else:
                    a="\n"
                out_string=(out_string+("""@target G0.S%d\n@type xy"""
                            % dataset_number) +a)

                for line in set.data:
                    short_string=""
                    for word in line:
                        short_string=short_string+str(word)+"\t"
                    out_string=out_string+short_string+"\n"
                out_string=out_string+"&\n"
                dataset_number=dataset_number+1

        else:
            raise UserWarning, "No datasets to export!"
        
        return out_string


#---------------------------------------------------------------------
        
    def writeout(self, out_fname, legend="on", graphic_formats=['EPS','PNG'], verbose="on"):
        'creates the XMGR file'
        outfile=open(out_fname+'.agr',"w")
        if not outfile: raise UserWarning, "Cannot create file '%s'!" % out_fname
        outfile.write(self.header())
        outfile.write(self._get_position())
        outfile.write(self._get_title())
        outfile.write( self._get_scales())
        outfile.write(self._get_xaxis())
        outfile.write(self._get_yaxis())
        if legend=="on":  outfile.write(self._get_legend())
        outfile.write(self._get_frame())
        outfile.write(self._get_datasets())
        outfile.close()
        if verbose=='on': print "File '%s.agr' has been written." % out_fname

        #-- export PNG and EPS files
        gracebat_command=os.path.join(self.path_to_gracebat,'gracebat')
        
        JPEG_output='%s %s.agr -printfile %s.jpg -hdevice JPEG'
        PNG_output='%s %s.agr -printfile %s.png -hdevice PNG'
        EPS_output='%s %s.agr -printfile %s.eps -hdevice EPS'

        if 'PNG' in graphic_formats:
            os.system(PNG_output % (gracebat_command, out_fname, out_fname))
            if verbose=='on': print "File '%s.png' has been written." % out_fname
        if 'EPS' in graphic_formats:
            os.system(EPS_output % (gracebat_command, out_fname, out_fname))
            if verbose=='on': print "File '%s.eps' has been written." % out_fname
        if 'JPEG' in graphic_formats:
            os.system(JPEG_output % (gracebat_command, out_fname, out_fname))
            if verbose=='on': print "File '%s.jpg' has been written." % out_fname
       

#********************************************************************
# self testing
if __name__=="__main__":
        

    xdata1=[1,2,3]
    ydata1=[3,3,4]
    ydata2=[3.4,3.0,2.9]
    dy=[.3,.4,0.2]
    data2=[[1,1,0.2],[2,2,0.2],[3,4,0.2]]


    b1=Dataset(make_xydy_matrix(xdata1,ydata1,dy), "exp data")
    b1.circle()
    b1.no_line()
    #b1.dotted_line()
    #b1.blue_sym()

    b3=Dataset(make_xy_matrix(xdata1,ydata2), "curve")
    b3.solid_line()
    b3.no_sym()

#    b2=Dataset(data2, "data2")
#    b2.circle()
#    b2.red_sym()
#    b2.filled_sym()
#    b2.solid_line()


    a=Graph("my_title","my_subtitle")
    a.add_dataset(b1)
#    a.add_dataset(b2)
    a.add_dataset(b3)
    a.set_xaxis("X axis",-1,4,1.1,2)
    a.set_yaxis("Y axis",-1,6,1,0.5)
    a.writeout("test_out")





