#!/usr/bin/env python
doc1="""
================== Import Assignments from GARANT in XEasy format =================
by Evgenii Kovrigin,  10/31/08
v.2

"""

doc2="""

This is a utility to extract assignments from XEASY files
and generate Sparky .save file

Usage import_xeasy.py  SEQ_file PROT_file XEASY_file Sparky_.save_template  new_.save_filename H_axis C/H_axis N_axis
     where

         SEQ_file  - a sequence file from XEASY
         
         PROT_file - an atom list from XEASY
         
         XEASY_file - a resonance list from XEASY (GARANT)
         
         Sparky_.save_template - a .save file for the spectrum you will import
                             assignments into with all colors and levels set
                            (stays unchanged,
                            used only as a template to make a NEW save file)
                            

         new_.save_filename - a Sparky .save file to be created using new assignments

  
   arguments setting order of axis in Sparky .sp dataset (for 2D datasets set 0 for a unused nucleus).
    
         H_axis - serial number of direct proton axis in the Sparky dataset

         C/H_axis - serial number of carbon/indirect proton axis in the Sparky dataset

         N_axis - serial number of nitrogen axis in the Sparky dataset
         
  
      
Examples:

2D dataset:
import_xeasy.py  SEQ_file PROT_file 2D_XEASY_file 2D_Sparky_.save_template  new_.save_filename 2 0 1

3D dataset:
import_xeasy.py  SEQ_file PROT_file 3D_XEASY_file 3D_Sparky_.save_template  new_.save_filename 3 1 2


NOTE: In Sparky the assignment labels will be overlapping. Use 'lu' to unoverlap labels, 'lr' to shift them 
to the right. 'eu' undoes the move.




=========================================================


Bugs and fixes:
12/22/08 - empty lines in PROT file are disregarded



"""

import os, sys, string,  glob 

#--- amino acid abbreviation conversion tables
aa_1to3_transation={
 'R': 'ARG',
 'Q': 'GLN',
 'F': 'PHE',
 'Y': 'TYR',
 'W': 'TRP',
 'K': 'LYS',
 'G': 'GLY',
 'A': 'ALA',
 'H': 'HIS',
 'S': 'SER',
 'P': 'PRO',
 'E': 'GLU',
 'D': 'ASP',
 'T': 'THR',
 'C': 'CYS',
 'M': 'MET',
 'L': 'LEU',
 'N': 'ASN',
 'I': 'ILE',
 'V': 'VAL',
}

aa_3to1_transation={
'ARG' : 'R',
'GLN' : 'Q',
'PHE' : 'F',
'TYR' : 'Y',
'TRP' : 'W',
'LYS' : 'K',
'GLY' : 'G',
'ALA' : 'A',
'HIS' : 'H',
'SER' : 'S',
'PRO' : 'P',
'GLU' : 'E',
'ASP' : 'D',
'THR' : 'T',
'CYS' : 'C',
'MET' : 'M',
'LEU' : 'L',
'ASN' : 'N',
'ILE' : 'I',
'VAL' : 'V'
}

 
#--- templates for save files --
tmp1="""<ornament>
"""

tmp2_2D="""type peak
color 0 2 0 white
flags 0 0 0 0
id %d
pos %.3f %.3f 
height 0.000 %f
rs |%s|%s| |%s|%s| 
[
type label
color 2 0 0 white
flags 0 0 0 1
mode 2
pos %.3f %.3f 
label %s%s-%s%s
xy %.3f,%.3f %.3f,%.3f  
]
""" 

tmp2_3D="""type peak
color 0 2 0 white
flags 1 0 0 0
id %d
pos %.3f %.3f %.3f
height 0.000 %f
rs |%s|%s| |%s|%s| |%s|%s|
[
type label
color 2 0 0 white
flags 0 0 0 1
mode 2
pos %.3f %.3f %.3f
label %s%s-%s%s-%s%s
xy %.3f,%.3f %.3f,%.3f %.3f,%.3f %.3f,%.3f %.3f,%.3f %.3f,%.3f 
]
""" 
tmp3="""<end ornament>"""


#- visual peak label shift
d=[.1, .1, .1]


#--- PROGRAM BEGINS


print doc1
if len(sys.argv)< 9:
    print doc2
    sys.exit()



#--- read the data
seq_file=open(sys.argv[1],'r').readlines()
prot_file=open(sys.argv[2],'r').readlines()
xeasy_file=open(sys.argv[3],'r').readlines()
sparky_template_file=open(sys.argv[4],'r').readlines()
new_save_file=sys.argv[5]
H_axis=int(sys.argv[6])
C_axis=int(sys.argv[7])
N_axis=int(sys.argv[8])


#-- is this 3D dataset?
Dimensions=int(xeasy_file[0].split()[-1])

if Dimensions==2:
    dimension_names=['','']
    #-- find axis names
    if H_axis==0:
       print '\n\n\nError!\n I cannot do C-N HSQC 2D planes! One axis must be proton!\n'
       sys.exit()
    else:
        if C_axis==0:
            dimension_names[N_axis-1]='N'
            dimension_names[H_axis-1]='H'
            X_axis=N_axis
        else:
            if N_axis==0:
                dimension_names[C_axis-1]='C/H'
                dimension_names[H_axis-1]='H'
                X_axis=C_axis
            else:
                print '\n\n\nError!\nOne of the three axis assignments for 2D dataset must be 0. Instead, I have: %d %d %d.\n\n' %(
                        H_axis, C_axis, N_axis)
                sys.exit()
     
    print 'Processing 2D data: [%s]-[%s]' % (dimension_names[0],dimension_names[1])
    
if Dimensions>2:
    if Dimensions==3:
        dimension_names=['','','']
        dimension_names[H_axis-1]='H'
        dimension_names[N_axis-1]='N'
        dimension_names[C_axis-1]='C/H'
        print 'Processing 3D data: [%s]-[%s]-[%s]' % (dimension_names[0],dimension_names[1],dimension_names[2])
    else:
        print "\n\n\nError!\nWrong number of dimensions: %d in '%s'!\n\n" % (Dimensions, sys.argv[3])
        sys.exit()

#--- create a lookup table for single-letter residue names

sequence={}
sequence[0]='?'
for line in seq_file:
    words=string.split(line)
    if len(words)==0:
        continue
    resnum=int(words[1])
    resname=aa_3to1_transation[words[0]]
    #-- save
    sequence[resnum]=resname

#--- create a lookup table for atom names
atoms={}
atoms[0]=[0,'?']
for line in prot_file:
    words=string.split(line)
    resnum=int(words[-1])
    atomname=words[-2]
    atomnum=int(words[0])
    #--- save
    atoms[atomnum]=[resnum, atomname]


    
#--- create a list of assignments with frequencies
#-- drop lines starting with '#'
xeasy=[]
for counter in range(len(xeasy_file)):
    
    line=xeasy_file[counter]
    if line[0]=='#':
        print 'skip'
        continue
    xeasy.append(line)
    
#--- generate Sparky code
ornament=tmp1
sparky_peak_id=1

for line in xeasy:


    #---- 2D DATA ------------
    if Dimensions==2:
        #-- extract info
        words=string.split(line)
        H_freq=float(words[1])
        X_freq=float(words[2])
        Height=float(words[5])
        H_atomnum=int(words[9])
        X_atomnum=int(words[10])

        #-- obtain names and residues
        H_atom_resnum=atoms[H_atomnum][0]
        H_atom_resname=sequence[H_atom_resnum]
        H_atom_name=atoms[H_atomnum][1]
                
        X_atom_resnum=atoms[X_atomnum][0]
        X_atom_resname=sequence[X_atom_resnum]
        X_atom_name=atoms[X_atomnum][1]
        
        #--- make Sparky code for the peak
        
        #-- insert frequencies in the order of Sparky file
        w=[0,0]
        w[H_axis-1]=H_freq
        w[X_axis-1]=X_freq

        #-- insert resnames in the order of Sparky file
        resname=['','']
        resname[H_axis-1]=H_atom_resname+str(H_atom_resnum)
        resname[X_axis-1]=X_atom_resname+str(X_atom_resnum)

        #--- insert atom names
        atomname=['','']
        atomname[H_axis-1]=H_atom_name
        atomname[X_axis-1]=X_atom_name

        

        ornament+=tmp2_2D % (sparky_peak_id,
                          w[0], w[1], 
                          Height,
                          resname[0], atomname[0], resname[1], atomname[1],  
                          w[0], w[1], 
                          resname[0], atomname[0], resname[1], atomname[1], 
                          w[0],w[1],w[1]+d[1],w[0]- d[0]
                          )
    #---- END OF 2D DATA ------------


    #---- 3D DATA ------------
    if Dimensions==3:
        #-- extract info
        words=string.split(line)
        H_freq=float(words[1])
        C_freq=float(words[2])
        N_freq=float(words[3])
        Height=float(words[6])
        H_atomnum=int(words[10])
        C_atomnum=int(words[11])
        N_atomnum=int(words[12])

        #-- obtain names and residues
        H_atom_resnum=atoms[H_atomnum][0]
        H_atom_resname=sequence[H_atom_resnum]
        H_atom_name=atoms[H_atomnum][1]
        
        C_atom_resnum=atoms[C_atomnum][0]
        C_atom_resname=sequence[C_atom_resnum]
        C_atom_name=atoms[C_atomnum][1]
        
        N_atom_resnum=atoms[N_atomnum][0]
        N_atom_resname=sequence[N_atom_resnum]
        N_atom_name=atoms[N_atomnum][1]
        
        #--- make Sparky code for the peak
        
        #-- insert frequencies in the order of Sparky file
        w=[0,0,0]
        w[H_axis-1]=H_freq
        w[C_axis-1]=C_freq
        w[N_axis-1]=N_freq

        #-- insert resnames in the order of Sparky file
        resname=['','','']
        resname[H_axis-1]=H_atom_resname+str(H_atom_resnum)
        resname[C_axis-1]=C_atom_resname+str(C_atom_resnum)
        resname[N_axis-1]=N_atom_resname+str(N_atom_resnum)

        #--- insert atom names
        atomname=['','','']
        atomname[H_axis-1]=H_atom_name
        atomname[C_axis-1]=C_atom_name
        atomname[N_axis-1]=N_atom_name

        

        ornament+=tmp2_3D % (sparky_peak_id,
                          w[0], w[1], w[2],
                          Height,
                          resname[0], atomname[0], resname[1], atomname[1], resname[2], atomname[2], 
                          w[0], w[1], w[2],
                          resname[0], atomname[0], resname[1], atomname[1], resname[2], atomname[2],
                          w[0],w[1],w[0]+d[0],w[2]- d[0],
                          w[1]+d[1],w[0]+d[0],w[1]-d[1],w[2]+d[2],
                          w[2]+d[2],w[0]+d[0],w[2]-d[2],w[1]+d[1]
                          )
    #---- END OF 3D DATA ------------


ornament+=tmp3


#--- insert it in Sparky .save file
sparky_template_file=string.join(sparky_template_file,'')
part1=string.split(sparky_template_file, '<ornament>')[0]
part3=string.split(sparky_template_file, '<end ornament>')[1]
new_text=string.join([part1,ornament,part3],sep='')


outfile=open(new_save_file,'w')
outfile.write(new_text)
outfile.close
print "Output file '%s' written. " % new_save_file 
