# -----------------------------------------------------------------------------
# 'Integration of the peak along different dimensions'



import sputil
import tkutil
import tkMessageBox, tkFileDialog

import curvefit
import xmgr, os
from math import *
# -----------------------------------------------------------------------------
#
class region_rmsd_dialog(tkutil.Dialog):
  # storage for mean RMSD value
  _current_spectrum=None
  _mean_rmsd=0
  _number_of_runs=0
  def __init__(self, session):

    self.session = session
    self.drag_notice = None

    self.my_region=None
    
    tkutil.Dialog.__init__(self, session.tk, 'Integrate Region')

    self.top.columnconfigure(0, weight = 1)

    import Tkinter
    explain = ('Dragging a box exports the region')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 0, column = 0, sticky = 'nw')

    r = Tkinter.Label(self.top, justify = 'left')
    r.grid(row = 1, column = 0, sticky = 'nw')
    self.result_label = r

    br = tkutil.button_row(self.top,
                           ('Close', self.close_cb),
                           ('Int w2', self.region_w2),
                           ('Int w1', self.region_w1),                           
                           ('Help', sputil.help_cb(session, 'RegionRMSD')),
			   )
    br.frame.grid(row = 2, column = 0, sticky = 'nw')

    self.register_callback(1)
    self.top.bind('<Map>', lambda e, s=self: s.register_callback(1))
    self.top.bind('<Unmap>', lambda e, s=self: s.register_callback(0))


  # ---------------------------------------------------------------------------
  #
  def register_callback(self, register):

    if register:
      if self.drag_notice == None:
        self.drag_notice = self.session.notify_me('drag region',
                                                  self.region_selected_cb)
    else:
      if self.drag_notice:
        self.session.dont_notify_me(self.drag_notice)
        self.drag_notice = None

  # ---------------------------------------------------------------------------
  #
  def region_selected_cb(self, region):

    if self.is_window_destroyed():
      self.register_callback(0)
      return

    #--ek
    self.my_region=region

    s = self.session.selected_spectrum()
    if s == None:
      self.result_label['text'] = 'No spectrum selected'
      return
    #print region
    rmin, rmax = region
    ranges = map(lambda vmin, vmax: '%.4g-%.4g' % (vmin, vmax), rmin, rmax)
    import string
    rtext = '[ ' + string.join(ranges, ', ') + ' ]'

        
    #-----------------------------------------------------------------------
    
    msg = ('  Spectrum    %s\n' % s.name +
           '  Region      %s\n\n' % rtext)
    self.result_label['text'] = msg



    
#######################################################################################
#######################################################################################
#######################################################################################
#######################################################################################


  # ---------------------------------------------------------------------------
  #
  def region_w1(self):
    'integrate dimension w1'
    print self.integrate_region(self.session.selected_spectrum(),self.my_region, 'w2')
  def region_w2(self):
    'integrate dimension w2'
    print self.integrate_region(self.session.selected_spectrum(),self.my_region, 'w1')

  # -----------------------------------------------------------------------------
  #
  def integrate_region(self,spectrum, region, X_axis='w1'):
    '->integrated spectrum along "x" or "y"'

    if spectrum.dimension!=2:
        tkMessageBox.showwarning("Integrate Region","Cannot work with %d dimensions!" % spectrum.dimension)
    
    ppm_per_index = map(lambda w,s: float(w)/s,
                        spectrum.spectrum_width, spectrum.data_size)
    print ppm_per_index

    dims={ 'w1':0,  # numerical order of dimensions in arrays
           'w2':1
           }
    #--- assign dimensions
    primary_dimension=dims[X_axis]
    secondary_dimension= not primary_dimension

#    print spectrum.data_height([120,8])

    
    min_X=region[0][primary_dimension]
    min_Y=region[0][secondary_dimension]

    max_X=region[1][primary_dimension]
    max_Y=region[1][secondary_dimension]
    
    curr_point=[0,0]

    #--- go along main dimension, integrate secondary
    
    integrated=[]
    max_y=0 # just for xmgr scaling

    X_steps=(max_X-min_X)/ppm_per_index[primary_dimension]
    Y_steps=(max_Y-min_Y)/ppm_per_index[secondary_dimension]

    for i in range(X_steps):
      
      curr_X=min_X + ppm_per_index[primary_dimension]*i
      curr_point[primary_dimension]=curr_X
      
      sum=0

      for j in range(Y_steps):

          curr_Y=min_Y + ppm_per_index[secondary_dimension]*j
          curr_point[secondary_dimension]=curr_Y
                     
          height=spectrum.data_height(curr_point)
                     
          sum+=height

      Y_point= sum/Y_steps
      integrated.append([curr_X,Y_point])
                     
      if Y_point>max_y: max_y=Y_point  # just for xmgr scaling



    fname=tkFileDialog.asksaveasfilename()
    if not fname: return

    #--- plain output ----
    file=open(fname,'w')
    for line in integrated:
      file.write("%f\t%f\n" % (line[0], line[1]))
    file.close()

    #-------- make XMGR file to look at -------
    int_dataset=xmgr.Dataset(integrated,str(region))
    int_dataset.solid_line()
    int_dataset.no_sym()
    int_graph=xmgr.Graph("Integrated 2D")
    int_graph.add_dataset(int_dataset)
    int_graph.set_xaxis(X_axis,min_X, max_X)
    int_graph.set_yaxis("Data height",0, max_y)
    int_graph.writeout(fname+".xmgr")
    return integrated

#-------------------------------------------------------------
  def fit_integrated(self,integrated_region, fit_func):
    'fit region to two peaks'
    pass
    
  
#######################################################################################
#######################################################################################
#######################################################################################
#######################################################################################

  
# -----------------------------------------------------------------------------
#
class lorentzian_func:
  'lorentzian lineshape peak'

  def parameter_count(self):
    return 3
  
  def value(self, params, x):
    a0, dv, v0 = params
  
    A1=a0*dv*dv
    
    A2=dv*dv-4*(x-v0)*(x-v0)

    return A1/A2


  def parameter_derivatives(self, params, x):

    epsilon=1e-6
    derivatives=[]
    
    for i in range(params):
      #-- step up ---
      temp_params2=params[:]
      temp_params2[i]=params[i]+params[i]*epsilon
      #-- step down ---      
      temp_params1=params[:]
      temp_params1[i]=params[i]-params[i]*epsilon

      
      dfda=(self.value(temp_params2,x)-self.value(temp_params1,x))/params[i]*epsilon
      
      derivatives.append(dfda)

  
    return derivatives

#  

def run(session):

  sputil.the_dialog(region_rmsd_dialog,session).show_window(1)
