"""v.2 Class definitions to handle files with text data indexed by keywords in the first column

     Use Ini_File class to parse your text files
"""
import sys
import os
import string
import mylib

class Keyword_File:
    """text file handling (general puprose)
    - opens files and do checks
    - separates files into lines and into words
    - keeps a text file as a list of lists of words from each line
    - indexing: 0,1,...,N
    - 1st word in each line considered to be a 'keyword'
    - number of identical keyword is not limited but only 1st will be found
       the rest of line is a 'list value'
    """
    filename=None       # source data file
    iterator_counter=0  
    

#---------- constructor -----------------------------
    def __init__(self, path_to_data, file_to_load, verb="silent"):
        'loads a file as a list of lists'
        self.text=[]             # list of list of words from each string        
        # check path
        path_to_data=string.rstrip(path_to_data)
        if not os.path.isdir(path_to_data) :
             raise IOError, "\n\nDirectory '%s' doesn't exist!" % path_to_data
        
        # check a file
        self.filename=os.path.join(path_to_data,file_to_load)
        if not os.path.isfile(self.filename) :
            raise IOError, "\n\nFile %s doesn't exist!" % self.filename
        inputstream=open(self.filename, 'r')
        if not inputstream :
            raise IOError, "\n\nCannot read file '%s'!" % self.filename

        # read all, make a list of lists
        raw_strings=inputstream.readlines()

        # remove empty lines
        raw_strings=filter(lambda test_string : test_string.strip(),raw_strings)
        if len(raw_strings) < 1 :
                raise UserWarning, "\n\nThe data file '%s' is empty!" % self.filename

        # fill in an array
        for str in raw_strings:
            list=string.split(str)
            self.text.append(list)
        # report
        if verb=="verbose" :
            mydir=os.path.dirname(self.filename)
            print "\nFile '%s' loaded" % os.path.basename(self.filename),
            if mydir!=os.curdir : print "from\n%s" % mydir
            else: print ":",
            print "%s non-empty lines read."  % len(self.text)

        #-- close the file
        inputstream.close()
#--------------------------------------------------------

#--------- print and 'x' -----
    def __repr__(self):
        fulltext=["File:\n"+self.filename+"\n-----------------\n"]
        for key in self.text:
            fulltext.append(string.join( map(str,key),'\t')+"\n")
        return string.join(fulltext)

#--------- write data to a file as simple table ------
    def writeout(self, out_fname, mode="silent"):
        'creates a table in the file'
        outfile=open(out_fname,"w")
        if not outfile: raise UserWarning, "Cannot create file '%s'!" % out_fname
        fulltext=[]
        for key in self.text:
            fulltext.append(string.join( map(str,key),"\t")+"\n")
        outfile.writelines(string.join(fulltext,""))
        outfile.close()
        if mode=="verbose" :
            print "File '%s' successfully written: %d lines." % (out_fname, len(self))

    
#--- length, truth------------
    def __len__(self):
        return len(self.text)


#-------- a[ind] -------
    def __getitem__(self, line_number):
        'list by a number'
        return self.text[line_number]


#------------- getting a line by keyword in the first column -----------
    def get_all_line(self, keyword):
        'find a line by keyword, returns the found line or raises exception'
        for line in self.text:
            if keyword==line[0]: break
        else:
            raise UserWarning, (
            "\n\nError!\nKeyword '%s' is not found in '%s'!" % (keyword, self.filename))
        return line
#--------------- determining a position of a keyword in the datalist -------
    def get_linenum(self, keyword):
        'find a keyword -> line number'
        for i in range(len(self.text)):
            if keyword==self.text[i][0]: return i
        else:
            raise UserWarning, (
            "\n\nError!\nKeyword '%s' is not found in '%s'!" % (keyword, self.filename))



#------------- getting the first word after a keyword (second column word)-----------
    def get_string(self, keyword):
        'find a line by keyword, returns the next in line word or raises exception'
        return self.get_all_line(keyword)[1]

#------------- assigning one string to a keyword --------------
    def assign_string(self, keyword, new_string):
        'replaces the contents after existing keyword (keeps order!)'
        list_ref=self.get_all_line(keyword)
        # remove old list of values
        for i in range(1,len(list_ref)):
            del list_ref[-1]
        # append new string
        list_ref.append(new_string)

    def get_float(self, keyword):
        'find a line by keyword, returns the next found float or raises exception'
        return string.atof(self.get_all_line(keyword)[1])
 
    def get_integer(self, keyword):
        'find a line by keyword, returns the next found integer or raises exception'
        return string.atoi(self.get_all_line(keyword)[1])
  
    # replaced by get_string(). Keep for compatibility
    def get_value(self, keyword):
        'Do not use! Find a line by keyword, returns the next in line value or raises exception'
        return self.get_all_line(keyword)[1]


#------------- dealing with a list after a keyword (slow dictionary)--------------
    def get_list(self, keyword):
        '-> list of words found after a keyword'
        return self.get_all_line(keyword)[1:]

    def assign_list(self, keyword, new_list):
        'replaces the line after existing keyword (keeps order!)  (slow dictionary'
        list_ref=self.get_all_line(keyword)
        # remove old list of values
        for i in range(1,len(list_ref)):
            del list_ref[-1]
        # append new list of values
        for i in range(len(new_list)):
            list_ref.append(new_list[i])
        





#---- iterator: return a line------------------------------
    def __iter__(self):
        return self
    
    def next(self):
        'iterating with RIGHT index order!!!'
        current=self.iterator_counter

        if not current<len(self):
            self.iterator_counter=0
            raise StopIteration
        
        self.iterator_counter=self.iterator_counter+1
        return self[current]


#--------- Deleting entries ------------------------------------    

    def remove_index(self, line_number):
        'remove line by number'
        x=self.text[line_number]
        self.text.remove(x)

    def remove_value(self, list_value):
        'remove line by its list value (keyword, val1, val2 ...)'
        self.text.remove(list_value)

    def remove_keyword(self, keyword):
        'removes a line with this keyword'
        line=self.get_all_line(keyword)
        self.remove_value(line)
        
        
#---------- Export dictionary using first word as a keyword -------
#           as a tool to a very quick search for lines 
    def dict(self):
        'Export dictionary using first word as a keyword and "file" for filename'
        new_dict={}
        for line in self.text:
            new_dict[line[0]]=line[1:]
        new_dict["file"]=self.filename
        return new_dict





#-------------- Initialization files (improved initial fname rules) ----------------
class Ini_File(Keyword_File):
    'just more natural paths usage'
    def __init__(self, ini_fname, mode="silent"):
        path,fname= os.path.split(ini_fname)
        if path=="":  path=os.getcwd()
        Keyword_File.__init__(self,path,fname,mode)
#--------------------------------------------------------------        

if __name__=="__main__" :
#    test=Keyword_File(os.getcwd(),"37C.replace_resolved.ini", verb="verbose").export_dict()
    print "test"
    test=Ini_File("watch_magnet.ini","verbose")
    print test
 #   print test
    #test.assign_string("<Replacement_table_file>","ddd")
    #print test
    #test.writeout("testkey"," ")
   
    
