#!/usr/bin/env python
import string, glob, os, sys
doc="""
------------------- LineshapeKin Simulation Summary Report Generator -----------------------------
 version 1.0 

5/8/09
LineShapeKin Simulation
Copyright (C) 2009 Evgenii Kovrigin
All Rights Reserved

This program looks in the directory supplied as a parameter and 
generates LineShapeKin Simulation HTML report for every sub-folder found.

"""

# Check parameters 
if len(sys.argv)!=2:
  print "Error in 'lineshapekin_simulation_report.py': I need a folder name with simulation results supplied as a parameter!"
  sys.exit()
  
# get folder and check if exists
target_folder=sys.argv[1]
if not os.path.isdir(target_folder):
   print "ERROR: Folder '%s' does not exist!!!" % target_folder
   sys.exit()
print "Working folder: '%s'" % target_folder

# get folder names
simulations=glob.glob(os.path.join(target_folder,'*'))
if not len(simulations):
  print "Error: Folder '%s' is empty!" % target_folder



#--- HTML pieces ---
HTML_part1="""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title>Title - LineShapeKin Simulation</title>
<style type="text/css">
<!--
body {
	background-color: #CCCCCC;
}
body,td,th {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: medium;
}
.style1 {font-size: small}
-->
</style></head>

<body>
""" 
HTML_part1=string.join(string.split(HTML_part1,'Title'),target_folder)

HTML_part2="""
<h2>LineShapeKin Simulation Report</h2>
<h3>Location: <span class="style2">Title</span></h3>
<p>&nbsp;</p>
""" 
HTML_part2=string.join(string.split(HTML_part2,'Title'),target_folder)
 


_HTML_part3="""
<p>&nbsp;</p>
<p>LineShapeKin Simulation<br />
  Copyright (C) 2009 Evgenii Kovrigin<br />
All Rights Reserved</p>
<p>&nbsp;</p>
</body>
</html>"""

HTML_part3="""
<p>&nbsp;</p>
<p>LineShapeKin Simulation<br />
Evgenii Kovrigin, Medical College of Wisconsin, 2009<br />
</p>
<p>&nbsp;</p>
</body>
</html>"""




#--- specific HTML code for the entry
HTML_template="""
<hr />
<h2><a href="U_AWF/">U_AWF</a></h2>
<h3>description</h3>
<p><img src="../images/model.png"  " /></p>
<table width="100%" border="2" cellspacing="2" cellpadding="2">
  <tr>
    <td valign="top"><p><em><strong>Equilibrium molar fractions of species</strong></em><br />
        <br />
        <img src="U_AWF/U_AWF_populations.png" width="521" height="391" /><br />
        <span class="style1">Other graphic formats: <a href="U_AWF/U_AWF_populations.eps">EPS</a>, <a href="U_AWF/U_AWF_populations.fig">MATLAB Figure</a></span></p>
    <p class="style1">Points correspond to steps in titration series corresponding to  Rtotal and LRratio used in NMR line shape simulations below. Curves are smooth functions simulated at constant Rtotal. Populations of multimeric species are shown per monomer. Population of free ligand is normalized to total receptor concentration.</p></td>
    <td valign="top"><p><em><strong>Isothermal Titration Calorimetry profile</strong></em><br />
        <br />
        <img src="U_AWF/U_AWF_ITC.png" alt="" width="521" height="391" /><br />
        <span class="style1">Other graphic formats: <a href="U_AWF/U_AWF_ITC.eps">EPS</a>, <a href="U_AWF/U_AWF_ITC.fig">MATLAB Figure</a></span></p>
    <p class="style1">Heat uptake curve is simulated as a derivative of the population of species multiplied by specific dH.</p></td>
    <td rowspan="3" valign="top"><p>parameters</p></td>
  </tr>
  <tr>
    <td valign="top"><p><em><strong>1D-spectra of the titration series</strong></em><br />
      <br />
      <img src="U_AWF/U_AWF_spectra.png" width="521" height="391" /><br />
        <span class="style1">Other graphic formats: <a href="U_AWF/U_AWF_spectra.eps">EPS</a>, <a href="U_AWF/U_AWF_spectra.fig">MATLAB Figure</a></span></p>
    <p class="style1">Traces (bottom to top) correspond to L/R ratios indicated by points in the Populations graph (above). Red dots indicate peak maxima for a visual aid (also plotted as the chemical shift titration curves on the right). Dashed lines indicate chemical shifts of individual species.</p></td>
    <td valign="top"><p><em><strong>Chemical shift titration curves</strong></em><br />
      <br />
      <img src="U_AWF/U_AWF_titration_curve.png" width="521" height="391" /><br />
        <span class="style1">Other graphic formats: <a href="U_AWF/U_AWF_titration_curve.eps">EPS</a>, <a href="U_AWF/U_AWF_titration_curve.fig">MATLAB Figure</a></span></p>
    <p class="style1">Points correspond to positions of peak maxima numerically detected in the spectral traces. Sometimes not all peaks will be detected and signal/noise here is not considered.</p></td>
  </tr>
  <tr>
    <td valign="top"><strong><em>Plot of the line widths (full width at a half height)</em></strong>
      <p><img src="U_AWF/U_AWF_line_width_curve.png" width="521" height="390" /></p>
    <p><span class="style1">Plain text data for comparison plotting: <a href="U_AWF/U_AWF_line_width_curve.txt">TXT</a><br />
      Other graphic formats: <a href="U_AWF/U_AWF_line_width_curve.eps">EPS</a>, <a href="U_AWF/U_AWF_line_width_curve.fig">MATLAB Figure</a></span></p>
    <p class="style1">The full width at a half height (FWHH) is determined numerically for the detected peaks. For the lorentzian line shape  FWHH = 2*R<sub>2</sub>. <br /> At the points of peak coalescence the measurement fails, you should discard these points. The plain text data for re-plotting the graph is saved with the graphics.</p></td>
    <td valign="top">&nbsp;</td>
    <td valign="top">&nbsp;</td>
  </tr>

</table>
"""
test_case_name='U_AWF' # what to replace to make an HTML code for a new case



outstring=HTML_part1+HTML_part2

for folder in simulations:
  # skip files
  if not os.path.isdir(folder):
    continue
                
  # generate a report for the entry
  case_name=os.path.basename(folder)
  print "Generating a report for '%s'." % case_name
  new_HTML=string.join(string.split(HTML_template,test_case_name),case_name)
  # get description
  fid=open(os.path.join(folder,'%s.txt'%case_name),'r')
  parameters_text=map(string.strip,fid.readlines())
  description=parameters_text[1] # description line
  fid.close()
  
  # add description
  new_HTML= string.join(string.split(new_HTML,'description'),description)
  # add parameters text
  
  new_HTML= string.join(string.split(new_HTML,'parameters'),string.join(parameters_text,'<br />'))
  
  # save
  outstring+=new_HTML 

# finalize
outstring+=HTML_part3


# save HTML file
filename=os.path.join(target_folder,'summary_report.htm')
fid=open(filename,'w')
fid.write(outstring)
fid.close()
print '\n\nSummary report "%s" generated.\n' % filename
  
  
