#!/usr/bin/env python2
import residues, mylib
import os, sys, math
doc="""
============== Luciferase assay data processing ===================

Three light readings are averaged.
Three total protein weights are averaged.
Light readings are divided by total protein = specific light output

All non-control results are divided by control light output =
= enhancement. Data are output in normal and logarithmic scale


Input data format:
(name) (three light readings) (three total protein measurements)
example:
-----------------------------
control	1 2 3  10 20 30

#1 10 20 30 10 20 30
#2 10 20 30 100 200 300
----------------------------- 

Usage: luciferase_assay.py inputfile1.txt inputfile2.txt inputfile3.txt ...
        (unlimited number of files)
====================================================================
"""
#---------------------------------------------
class Luciferase_Data(residues.Word_Dict):
    'stores specific luciferase data structure'
    
    final_result=None #--- processing results
    control_name="control"
    
    def __init__(self,fname):
        residues.Word_Dict.__init__(self,fname)
        self.convert_values_to_float()
        self.final_result=self.make_empty_copy(self.get_name()+'.processed')
        
    #-------- little statistics routine ----
    def _StdErr095(self,list):
        'sigma(x_mean)*t_student(0.95%)'
        N=len(list)
        mean,StdDev=mylib.statistics(list)
        if N!=3:
            print "Error! Statistics for N!=3 is not implemented!"
            sys.exit()
        t_student_3_095=4.3
        return mean, t_student_3_095*StdDev/math.sqrt(N)

    def _x_div_y(self,x,dx,y,dy):
        'result, result_error'
        z=x/y
        dz=(abs(dx/y) + abs(x*dy/(y*y)))
        return z,dz
    
    #------- normalizing 
    def _normal_value(self,key):
        'needs three values in two groups!!!'
        #--- check data format
        if len(self[key])!=6:
            print "Data format error for line '%s'! I need two group of three values!\n\
Got instead: %s" % (key, self,key)
            sys.exit()

        #-- OK !
        MEAN=0
        ERROR=1
        light=  self._StdErr095(self[key][:3])
        protein=self._StdErr095(self[key][3:])

        return self._x_div_y(x=light[MEAN],dx=light[ERROR],y=protein[MEAN],dy=protein[ERROR])

        

    #--------- processing ------------
    def process(self):
        'all calculations'
        
        #--- check if control is there
        if not self.has_key(self.control_name):
            print "ERROR! '%s' not found in '%s'" %(self.control_name, self.get_name())
            sys.exit()

        control=self._normal_value(self.control_name)
        print "Control:", control
        
        for key in self.get_keys():
            if key==self.control_name: continue

            #--- normalize lights by protein amount ----
            sample=self._normal_value(key)
            print key, "Normalized light: ", sample
            #--- evaluate the effect relative to control with error propagation ---
            ratio=self._x_div_y(x=sample[0],dx=sample[1],y=control[0],dy=control[1])
            #--- use control as just a scaling factor ----
            ratio2=sample[0]/control[0], sample[1]/control[0]
            print "Ratio with proper error: ",ratio
            print "Ratio without control error: ",ratio2
            


    
#---------------------------------------------
        
##if len(sys.argv)<2:
##    print doc
##    sys.exit()
fname='liza_test.py'


print "-------------------------------------"
data= Luciferase_Data(fname)
#data.process()
print data._x_div_y(x=2.,dx=3.,y=5.,dy=7.)

