#!/usr/bin/env python
import string, sys
doc1=""" ========= make_prot_seq_files.py ========
by Evgenii Kovrigin,  10/23/08
v.1.2

"""


doc2="""
This utility prepares the PROT and SEQ files in XEASY format
for GARANT and PINE assignment program.

Usage: make_prot_seq_files.py sequence_file first_residue_number > out.txt
    where
    
        'sequence_file' contains plain text, single-letter aminoacid
                        code sequence (no numbers allowed). Lines starting
                        with '>' symbol are ignored
                        
        'first_residue_number' - serial number of the first aminoacid
                                residue in the supplied sequence
                                
        'out.txt' - will contain formatted sequence and statistics
                                
Output files contain amino-acid data in XEASY format and a sequence file .pineseq
for PINE submission.

=================================================================

Updates and fixes:
10/23/08 - numbering of residues in the PROT file corrected
10/31/08 - sequence format for PINE input added
"""

if len(sys.argv)!=3:
    print doc1
    print doc2
    sys.exit()


sequence_file=sys.argv[1]
starting_number=int(sys.argv[2])

#-- all amino acids
aminoacids='RQFYWKGAHSPEDTCMLNIV'

##- AA code translation
aa_transation={
 'R': 'ARG',
 'Q': 'GLN',
 'F': 'PHE',
 'Y': 'TYR',
 'W': 'TRP',
 'K': 'LYS',
 'G': 'GLY',
 'A': 'ALA',
 'H': 'HIS',
 'S': 'SER',
 'P': 'PRO',
 'E': 'GLU',
 'D': 'ASP',
 'T': 'THR',
 'C': 'CYS',
 'M': 'MET',
 'L': 'LEU',
 'N': 'ASN',
 'I': 'ILE',
 'V': 'VAL',
}

#-- atom names in residues
aa_atom_lists={
 'R': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'HG2', 'HG3', 'QG', 'CD', 'HD2', 'HD3', 'QD', 'NE', 'HE', 'CZ', 'NH1', 'HH1', 'NH2', 'HH21', 'HH22', 'QH2', 'C'],
 'Q': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'HG2', 'HG3', 'QG', 'CD', 'NE2', 'HE21', 'HE22', 'QE2', 'C'],
 'F': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'QD', 'QE', 'QR', 'CG', 'CD1', 'HD1', 'CE1', 'HE1', 'CZ', 'HZ', 'CE2', 'HE2', 'CD2', 'HD2', 'C'],
 'Y': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'QD', 'QE', 'QR', 'CG', 'CD1', 'HD1', 'CE1', 'HE1', 'CZ', 'CE2', 'HE2', 'CD2', 'HD2', 'HH', 'C'],
 'W': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'CD1', 'CD2', 'CE3', 'CE2', 'NE1', 'HD1', 'HE3', 'CZ3', 'CZ2', 'HE1', 'HZ3', 'CH2', 'HZ2', 'HH2', 'C'],
 'K': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'HG2', 'HG3', 'QG', 'CD', 'HD2', 'HD3', 'QD', 'CE', 'HE2', 'HE3', 'QE', 'NZ', 'HZ1', 'HZ2', 'QZ', 'C'],
 'G': ['N', 'H', 'CA', 'HA3', 'HA2', 'QA', 'C'],
 'A': ['N', 'H', 'CA', 'HA', 'QB', 'CB', 'HB1', 'HB2', 'HB3', 'C'],
 'H': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'ND1', 'CD2', 'HD1', 'CE1', 'NE2', 'HD2', 'HE1', 'C'],
 'S': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'HG', 'C'],
 'P': ['N', 'CD', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'HG2', 'HG3', 'QG', 'HD2', 'HD3', 'QD', 'C'],
 'E': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'HG2', 'HG3', 'QG', 'CD', 'HE2', 'C'],
 'D': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'HD2', 'C'],
 'T': ['N', 'H', 'CA', 'HA', 'CB', 'HB', 'QG2', 'HG1', 'CG2', 'HG21', 'HG22', 'HG23', 'C'],
 'C': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'HG', 'C'],
 'M': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'HG2', 'HG3', 'QG', 'QE', 'CE', 'HE1', 'HE2', 'HE3', 'C'],
 'L': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'HG', 'QD1', 'QD2', 'CD1', 'HD11', 'HD12', 'HD13', 'CD2', 'HD21', 'HD22', 'HD23', 'QQD', 'C'],
 'N': ['N', 'H', 'CA', 'HA', 'CB', 'HB2', 'HB3', 'QB', 'CG', 'ND2', 'HD21', 'HD22', 'QD2', 'C'],
 'I': ['N', 'H', 'CA', 'HA', 'CB', 'HB', 'QG2', 'CG2', 'HG21', 'HG22', 'HG23', 'CG1', 'HG12', 'HG13', 'QG1', 'QD1', 'CD1', 'HD11', 'HD12', 'HD13', 'C'],
 'V': ['N', 'H', 'CA', 'HA', 'CB', 'HB', 'QG1', 'QG2', 'CG1', 'HG11', 'HG12', 'HG13', 'CG2', 'HG21', 'HG22', 'HG23', 'QQG', 'C'],
}

#-- default text in PROT file
default_text="999.000 0.000"

print doc1

#--- load sequence
sequence_list=open(sequence_file,'r').readlines()
#--- drop lines starting with >
sequence=""
for counter in range(len(sequence_list)):
    line=sequence_list[counter]
    if line[0]=='>':
        print line
        continue
    sequence+=line
                     

#--- remove spaces
sequence=string.upper(string.join( string.split(sequence)  ,''))
#--- check sequence, abort if non-aminoacid symbols are found
print 'Checking sequence:'

for counter in range(len(sequence)):
    letter=sequence[counter]
    if letter not in aminoacids:
        print "\n\nERROR!\n\n"
        print "Unknown symbol '%s' is found in the protein sequence. Please, check '%s'!\n\n" % (
            letter, sequence_file)
        sys.exit()
    print '%s' %letter,
    if (counter+1)%10==0: print ' - %d' % (counter+starting_number)
    
print '\n%d residues.\n' % (counter+1)



#---- Generate PROT file
prot_file=""
seq_file=""
pineseq_file=""
global_atom_counter=0;
for residue_counter in range(len(sequence)):
    residue_name1=sequence[residue_counter]
    residue_name3=aa_transation[residue_name1]
    residue_number=residue_counter+starting_number
    atom_list=aa_atom_lists[residue_name1]

    #-- add line to sequence files
    seq_file+="%s\t\t%d\n" % (residue_name3, residue_number)
    pineseq_file+="%s\n" % residue_name1

    #-- add lines to PROT file
    for local_atom_counter in range(len(atom_list)):
        global_atom_counter+=1
        atom_name=atom_list[local_atom_counter]
        prot_file+="%d\t%s\t%-6s%4d\n" % (global_atom_counter,default_text,
                                         atom_name,residue_number)


        


#--- write files out
outfname_seq=sequence_file+'.seq'
outfname_pineseq=sequence_file+'.pineseq'

outfname_prot=sequence_file+'.prot'

outfile=open(outfname_seq,'w')
outfile.write(seq_file)
outfile.close()
print '%d amino acids.\nFile "%s" written.\n' % (len(sequence), outfname_seq)

outfile=open(outfname_pineseq,'w')
outfile.write(pineseq_file)
outfile.close()
print '%d amino acids.\nFile "%s" written.\n' % (len(sequence), outfname_pineseq)

outfile=open(outfname_prot,'w')
outfile.write(prot_file)
outfile.close()
print '%d atoms.\nFile "%s" written.\n' % (global_atom_counter-1, outfname_prot)









    

