#!/usr/bin/env python
import os,sys,string
import MFselect_lib_2
import residues
import mylib
import keyfile
import copy
from MFselect_constants import *


doc="""
************************************************
The script to compare and analyse ModelFree output files

Usage: mfout_diff.py  -s file
            calculates mean S2S, mean S2S error and Gamma,
            finds max S2S error and Gamma

       mfout_diff.py -c file1 file2
           compare models assignments
           compare S2S-s to a sum of their errors
           not matching residues are ignored

"""

#--------------------------------------------------------
def analyze_one(filename):
    mfout= MFselect_lib_2.Modelfree_Output(filename, TENSOR_OPT)
    mfout.load(conf_level=0.95)

    res_list=mfout.reslist.get_keys()
    res_list.sort()

    S2S_list=[]
    for res in res_list:
        S2S_list.append(float(mfout.reslist[res][S2S]))

    S2Sstddev_list=[]
    for res in res_list:
        S2Sstddev_list.append(float(mfout.reslist[res][S2Sstddev]))

    Gamma_list=[]
    for res in res_list:
        Gamma_list.append(float(mfout.reslist[res][GAMMA]))



    S2Sstddev_mean, S2Sstddev_sigma =mylib.statistics(S2Sstddev_list)
    S2Sstddev_max = max(S2Sstddev_list)

    Gamma_mean, Gamma_sigma =mylib.statistics(Gamma_list)
    Gamma_max = max(Gamma_list)



    print "*******************************\nfile: %s\n" % filename
    print "Mean (S2S simulated standard deviation): %.4f" % S2Sstddev_mean
    print "rmsd : \t\t%.4f" % S2Sstddev_sigma
    print "Max:  residue \t%s\t%s \n" %  (res_list[S2Sstddev_list.index(S2Sstddev_max)], S2Sstddev_max)

    print "Mean Gamma: \t%.4f" % Gamma_mean
    print "rmsd Gamma: \t%.4f" % Gamma_sigma
    print "Max:  residue \t%s\t%s " %  (res_list[Gamma_list.index(Gamma_max)], Gamma_max)
    print
    print

#--------------------------------------------------------
def print_columns(mfout, res, *columns):
    'printing columns'
    for col in columns:
        print "%s" % (str(mfout.reslist[res][col]).ljust(7)),
    print 
                                    

def print_residue(res,mfout,  print_header="yes"):

    if print_header=="yes":
        print "Columns        S2S    S2Serr S2F   S2Ferr     TE      TEerr   REX     REXerr    G      Gcrit"     
    print "Res: %4s  M%s" % (res, mfout.reslist[res][MODEL_USED]),
    print_columns(mfout, res, S2S, S2Sstddev, S2F, S2Fstddev,TE, TEstddev, REX, REXstddev, 
                     GAMMA, GAMMAcritical)

def compare_two(filename1, filename2):
    print "\nWarning! Residues with S2 less than some cutoff are not included in mfout-s from optimization!\n"
    
    mfout1= MFselect_lib_2.Modelfree_Output(filename1, TENSOR_OPT)
    mfout1.load(conf_level=0.95)

    res_list1=mfout1.reslist.get_keys()
    res_list1.sort()

    mfout2= MFselect_lib_2.Modelfree_Output(filename2, TENSOR_OPT)
    mfout2.load(conf_level=0.95)

    res_list2=mfout2.reslist.get_keys()
    res_list2.sort()


    #-- check that lists are the same -------
    for res in res_list1:
        if res not in res_list2:
            print "Residue %s of '%s' not found in '%s'" % (res, mfout1.reslist.name, mfout2.reslist.name)
            res_list1.remove(res)
            
    for res in res_list2:
        if res not in res_list1:
            print "Residue %s of '%s' not found in '%s'" % (res, mfout2.reslist.name, mfout1.reslist.name)
            res_list2.remove(res)

    # --------- tensors ---------------------------
    print "Tensors:\n--------------------"
    print mfout1.reslist.name
    print mfout1.print_tensor()
    print

    print mfout2.reslist.name
    print mfout2.print_tensor()
    print


    #---------- Different model assignments ---------------
    same_model_list=res_list1[:]
    print "\nDifferent model assignments :\n%s\n%s\n----" % (mfout1.reslist.name,mfout2.reslist.name)
 
    header="yes"
    for res in res_list1:
        if mfout1.reslist[res][MODEL_USED]!=mfout2.reslist[res][MODEL_USED]:
            print_residue(res, mfout1,header)
            header="no"
            print_residue(res, mfout2, header)
            same_model_list.remove(res)
            print 



    #---------- Different S2S ---------------
    print "\nDiffering S2S:\n%s\n%s\n----" % (mfout1.reslist.name,mfout2.reslist.name)
 
    header="yes"
    for res in res_list1:
        s1=float(mfout1.reslist[res][S2S])
        e1=float(mfout1.reslist[res][S2Sstddev])
        
        s2=float(mfout2.reslist[res][S2S])
        e2=float(mfout2.reslist[res][S2Sstddev])
        
        if abs(s1-s2) > (e1+e2):
            
            print_residue(res, mfout1,header)
            header="no"
            print_residue(res, mfout2, header)
            print 







#--------------------------------------------------------


if len(sys.argv) <3 :
    print doc
    sys.exit()


if '-s' == sys.argv[1]: analyze_one(sys.argv[2])
elif '-c'== sys.argv[1]: compare_two(sys.argv[2], sys.argv[3])
else:
    print "\a\a\a\n\nError!!!  Wrong option: '%s'" % sys.argv[1]
    print doc
    
