#!/usr/bin/env python
"""molmol.py
Classes for creating piece of a macro for MolMol to paint residues in
colors corresponding to the value of some parameter
Makes also XMGR file to check

07/19/05 - plain output resnum-rgb is added to allow import to Pymol

"""

init_lines="""
BackColor %s 
DefPropAtom 'prev_sel' 'selected'
DefPropBond 'prev_sel' 'selected'
ColorAtom %s 
"""

res_lines="""
SelectRes 'prev_sel & num=%s'
ColorAtom %s 
"""

final_lines="""
SelectAtom 'prev_sel'
SelectBond 'prev_sel'
"""

import mylib, residues, xmgr





##    
##def mix_color(par, minpar, maxpar, increments):
##    'creates an incremented color spectrum starting from base R:G:B color'
##
##    d_color=1.0/increments
##
##    
##    d_param=(maxpar-minpar)/increments
##
##    Red=d_color*(par-minpar)/d_param
##    Blue=1-d_color*(par-minpar)/d_param
##    return Color(Red,0.1,Blue)

#----------------------------------------------------------------------

class Color:
    'defines a color'
    def __init__(self, *values):
        
        if len(values)==3:
            self.Red=float(values[0])
            self.Green=float(values[1])
            self.Blue=float(values[2])
            
        elif len(values)==1:
            number=values[0]
            self.Red=float(number)
            self.Green=float(number)
            self.Blue=float(number)
            
        else:
            raise UserWarning, "\n\nWrong number of arguments!!! '%s'\n\
                Need one or three!\n" % values


    def __sub__(self, other):
        return Color(self.Red-other.Red,
                     self.Green-other.Green,
                     self.Blue-other.Blue)
    

    def __add__(self, other):
        return Color(self.Red+other.Red,
                     self.Green+other.Green,
                     self.Blue+other.Blue)

    def __mul__(self, other):
        return Color(self.Red*other.Red,
                     self.Green*other.Green,
                     self.Blue*other.Blue)

    def __div__(self, other):
        return Color(self.Red/other.Red,
                     self.Green/other.Green,
                     self.Blue/other.Blue)

        




    def __repr__(self): return "%s %s %s" % (self.Red, self.Green, self.Blue)

    


#---- Common colors ----
White=Color(1,1,1)
Black=Color(0,0,0)
LightGrey=Color(0.95,0.95,0.95)
Grey=Color(0.75,0.75,0.75)
DarkGrey=Color(0.3,0.3,0.3)

Red=Color(1,0,0)
Green=Color(0,1,0)
Blue=Color(0,0,1)
Yellow=Color(1,1,0)
Cyan=Color(0,1,1)



#----------------------------------------------------------------------

class Protein:
    'handles information about protein structure to show'

    def __init__(self, total_res_num, assigned_residues_Annn_Num):

        #-- total number of residues in the protein
        self.total_res=total_res_num  # !!! not used anywhere!!!!

        #-- storage of data for assigned residues
        self.assigned_residues=assigned_residues_Annn_Num

        self.macro_text=""
	self.resnum_rgb=""
        
        #-- default colors
        self.background_color=White
        self.unassigned_residues_color=Black
        self.max_assigned_residues_color=Green  
        self.min_assigned_residues_color=Grey
#--------------------------------------------------------------------------------------
    def set_assigned_colors(self, min_color, max_color):
        'define color of assigned residues'
        self.max_assigned_residues_color=max_color  
        self.min_assigned_residues_color=min_color

#--------------------------------------------------------------------------------------
    def set_unassigned_color(self, color):
        'define color of unassigned residues'
        self.unassigned_residues_color=color

#--------------------------------------------------------------------------------------
    def set_background_color(self, color):
        'define color of unassigned residues'
        self.background_color=color

#--------------------------------------------------------------------------------------
    def highlight(self, resname_list, color):
        'just re-assign the color to highlight some residues (to use after "colorize_by_parameter()")'
        if type(resname_list)!=type([]):
            raise UserWarning, "\n\nError! Need a list of residue names!!!\nGot instead:\n%s" % resname_list

        for res in resname_list:
            resnum=res[1:]
            rescolor=color
            self.macro_text+=res_lines % (resnum, rescolor)
        
#--------------------------------------------------------------------------------------Cyan
    def colorize_by_parameter(self, parameter_col_num, scaling="auto", min="not set", max="not set", verbose="on"):
        """-> a piece of macro to colorize residues for MolMol
                using a range of a parameter to map the available
                range of residue color (single so far)

        scaling="auto" or "manual" if "manual" - min and max used
        """

        
        self.macro_text=init_lines % (self.background_color, self.unassigned_residues_color)
        
        res_list=self.assigned_residues.sorted_keys()

        

        if scaling=="auto":
            #-- find minimum value of parameter
            min_par_res=self.assigned_residues.min(parameter_col_num)
            min_par=self.assigned_residues[min_par_res][parameter_col_num]

            #-- find maximum value of parameter    
            max_par_res=self.assigned_residues.max(parameter_col_num)
            max_par=self.assigned_residues[max_par_res][parameter_col_num]

        elif scaling=="manual":
            if "not set" in (min,max):
                raise UserWarning, (
                "\n\nManual scaling requires 'min' and 'max' parameters set! Got min='%s', max='%s' instead!"
                % (min, max))
        
            min_par=min
            max_par=max
        else:
            raise UserWarning, "\n\nUnknown scaling mode: '%s'" % scaling
            
        
        dColor_dParam= ((self.max_assigned_residues_color-self.min_assigned_residues_color)/
                        Color(max_par-min_par))

        full_histogram=[]
        clipped_histogram=[]
        for res in res_list:
            resnum=res[1:]
            respar=self.assigned_residues[res][parameter_col_num]
            
            full_histogram.append([resnum,respar])

            #-- clipping ---
            if respar>max_par:
                if verbose=='on':
                    print "clip %s from %s to %s" % (resnum, respar, max_par)
                respar=max_par

            if respar<min_par:
                if verbose=='on':
                    print "clip %s from %s to %s" % (resnum, respar, min_par)
                respar=min_par

            
            rescolor=self.min_assigned_residues_color+dColor_dParam*Color(respar-min_par)
                    
            self.macro_text+=res_lines % (resnum, rescolor)
	    
	    
	    #============ THIS LINE CREATES RESNUM-RGB OUTPUT ===========
	    self.resnum_rgb+= "%s \t%s\n" % (resnum, rescolor)
	    
	    
            visual_offset=0.5 #-- for better viewing
            clipped_histogram.append([int(resnum)+visual_offset,respar])
    

        #--- generate XMGR histogram
        histogram=xmgr.Graph("colorizing by parameter")
        
        clipped_hist_data=xmgr.Dataset(clipped_histogram,"clipped")
        clipped_hist_data.bar()
        clipped_hist_data.blue_sym()
        clipped_hist_data.symbol_size(0.7)
        histogram.add_dataset(clipped_hist_data)

        full_hist_data=xmgr.Dataset(full_histogram,"full dataset")
        full_hist_data.bar()
#        full_hist_data.empty_sym()
        full_hist_data.symbol_size(0.7)
        full_hist_data.red_sym()        
        histogram.add_dataset(full_hist_data)

        histogram.set_xaxis("Residue number",0,1)
        histogram.set_yaxis("Y",0,1)
        histogram.writeout("histogram.xmgr")


#--------------------------------------------------------------------------------------
    def write_macro(self, macro_fname):
        'output the macro for MolMol'
        self.macro_text+=final_lines
        mylib.open_file(macro_fname,'w').write(self.macro_text)
        return self.macro_text
##--------------------------------------------------------------------------------------
    def write_text_resnum_rgb(self, fname):
        'text output '
        mylib.open_file(fname,'w').write(self.resnum_rgb)
        return self.resnum_rgb
#--------------------------------------------------------------------------------------




if __name__=='__main__':

    test_list=residues.Annn_Num('molmol_test.list')
    print test_list
    protein=Protein(124,test_list)
   
    
    print protein.write_macro(0,'molmol.mac')
    
