import os, random
import sys
import string
from math import *
UserWarning="UserWarning"  # for compatibility with Python 1.5
YES,NO=1,0   # conditional flags



####################################################################################
####################################################################################
#            STATISTICS
####################################################################################
####################################################################################


#------------- Simple mean and sigma of a list ----------


def statistics(list):
    '->mean, sigma'
    sum=0
    sumsquare=0
    N=len(list)
    for i in range(N) :
        x=list[i]
        sum=sum+x
        sumsquare=sumsquare+x*x
        #-- abs() prevent appearance of -0
    return sum/N, sqrt(abs((sumsquare-sum*sum/N)/(N-1)))

#-------------------------------------------------------------



#----------- histogram -------------
# creates a histogram of x_list values in the +-interval around 0
# with a num_buckets and normalized to norm
# returns a list of bucket centers and a list of hist values
def histogram(x_list, interval, num_buckets, norm=1) :
    'makes a histogram of list values distributed in the +-interval around 0'
    

    bucket_width=2.0*interval/num_buckets
    buckets=[0]*num_buckets

    # histogram creation
    for i in x_list:
        x_to_bucket=int((interval+i)/bucket_width)
        # reject values falling out of +-interval
        try:
            buckets[x_to_bucket]=buckets[x_to_bucket]+1
        except IndexError:
            pass

    # normalization
    max=0
    for i in buckets:
        if i>max : max=i
    
    ordinate=[]
    for i in range(len(buckets)):
        buckets[i]=float(buckets[i])*norm/max
        #ordinate of a bucket center
        ordinate.append(bucket_width*(i+0.5)-interval)

    # make x,y array
    hist_out=[]
    for i in range(len(buckets)):
        hist_out.append([ordinate[i],buckets[i]])

    return hist_out







####################################################################################
####################################################################################
#            FILE AND DIRECTORY OPERATIONS
####################################################################################
####################################################################################

def open_file(fname,mode):
    'provide a little cosmetics on error message when opening.  -> file object'
    try:
        file=open(fname,mode)
    except IOError:
        raise UserWarning, (
"\n\n\a\a\a-----------------------------------\nError! Cannot open a file with '%s' mode!\n->  %s"
        % (mode,fname))
    return file

#------------------------------------------------------------
def check_file(file, fname):
    'opened OK? - redundant, use "open_file()" instead'
    if not file: 
        raise UserWarning, "\a\a\n\nErorr! Cannot open a file!\n->  %s" % (fname) 

#-------- Makes a list directories of a given directory
def list_dirs_only(path_to_fids):
    'Makes a list directories of a given directory'
    dirlist=os.listdir(path_to_fids)
    # remove file entries
    dirlist_to_iterate=dirlist[:]
    for item in dirlist_to_iterate:
        if not os.path.isdir(os.path.join(path_to_fids,item)):
            dirlist.remove(item)
    return dirlist

#-------------------------------------------------------------------
def deep_mkdir(path):
    'creates a directory creating the path if necessary'
    dirs=filter(None, path.split('/'))
    pathback=""
    for i in range(len(dirs)):
        nextdir=dirs[i]
        if not os.path.isdir(nextdir): os.mkdir(nextdir)
        os.chdir(nextdir)
        pathback+="../"
    #-- go back --
    os.chdir(pathback)






####################################################################################
####################################################################################
#            LIST OPERATIONS
####################################################################################
####################################################################################


#--------- write a list of lists into a file as simple table ------
def write_lists_out(data_list, out_fname="stdout"):
    'write a list of lists into a file as a table'
    if out_fname=="stdout" :
        outfile=sys.stdout
    else:        
        outfile=open(out_fname,"w")
        if not outfile: raise UserWarning, "Cannot create file '%s'!" % out_fname
    fulltext=[]
    for key in data_list:
        fulltext.append(string.join( map(str,key),'\t')+"\n")
    outfile.writelines(string.join(fulltext,""))
    if out_fname!="stdout" :    outfile.close()

###--------- write a list into a file as simple table ------
##def print_list(data_list, out_fname="stdout"):
##    'write a list into a file or to a stdout'
##    if out_fname=="stdout" :
##        outfile=sys.stdout
##    else:        
##        outfile=open(out_fname,"w")
##        if not outfile: raise UserWarning, "Cannot create file '%s'!" % out_fname
##    fulltext=""
##    for key in data_list:
##        fulltext+=string.join(str(key)+"\n")
##    outfile.write(fulltext)
##    if out_fname!="stdout" :    outfile.close()
##

        
#------------------------------------------------------------
def common_key_list(*key_lists):
    'make a non-redundant union of key lists'
    joined_list=[]
    for i in range(len(key_lists)):
        list=key_lists[i]
        for key in list:
            if key not in joined_list:
                joined_list.append(key)
                
    return joined_list


#------------------ Load_Text_File  --------------
#---- a function to read a text file -------------
#     returns list of strings 
#-------------------------------------------------
def Load_Text_File (fname) :
    if not os.path.isfile(fname): 
        print "\n\n\nError!\nCannot open '%s' to read!!!\n\n\a\a\a" % fname
	sys.exit()
    input=open(fname,'r')	
    L=input.readlines()
    input.close()
    return L


#------------------ Load_Text_File_NonEmpty_Lines  --------------
#---- a function to read a text file -------------
#     returns list of strings containing any non-space characters
#-------------------------------------------------
def Load_Text_File_NonEmpty_Lines (fname) :
    if not os.path.isfile(fname): 
        print "\n\n\nError!\nCannot open '%s' to read!!!\n\n\a\a\a" % fname
	sys.exit()
    input=open(fname,'r')	
    L=input.readlines()
    M=[]
    for line in L:
        if line.strip()!='':
            M.append(line)
    input.close()
    return M

#---------- Write Text File --------------
#-- writes a list of strings into a file --
#----------------------------------------
def Write_Text_File (list,fname):
    outfile=open(fname,'w')
    outfile.writelines(list)
    outfile.close()
    




#------------------ String_2_Words_2D ----------------------
#----- Convert a list of string into a 2D list of words ------
#-------------------------------------------------
def String_2_Words_2D(list) :
    words_2D_list=[]
    for line in list :
        words_2D_list.append(string.split(line))        
    return words_2D_list


#-------------------- Find_Position ---------------------------
#----- search for a word in a list 
#----- of strings. Returns string number, word number 
#----- or [-1,-1] if not found
#-------------------------------------------------
def Find_Position(list, word) :
    '-> [str,column] or [-1,-1] if not found'
    for str in list :
        words=string.split(str)
        if word in words : 
           return list.index(str), words.index(word)
    else : 
        return [-1,-1]

#-------------------- Find_String_Num ---------------------------
#----- search for a word in a list 
#----- of strings. Returns string number
#----- or -1 if not found
#-------------------------------------------------
def Find_String_Num(list, word) :
#    print list, word
    for str in list :
        words=string.split(str)
        if word in words : 
           return list.index(str)
    else : 
        return -1
#-------------------- Find_Keyword_First_in_Line ------------
#----- search for a keyword in a list of lists in the FIRST postion
#-----  Returns string number or -1 if not found
#-------------------------------------------------
def Find_Keyword_First_in_Line(list, word) :
    "search for a keyword in a list of lists in the FIRST postion"
    for str in list :
        if len(str)>0:
          if word==str[0] : 
             return list.index(str)
    else : 
        return -1

#-------------------- Find_Keyword_First_in_String ------------
#----- search for a keyword in a list of strings in the FIRST postion
#-----  Returns string number or -1 if not found
#-------------------------------------------------
def Find_Keyword_First_in_String(list, word) :
    "search for a keyword in a list of strings in the FIRST postion"
    for line in list :
        str=line.split()
        if len(str)>0:
          if word==str[0] : 
             return list.index(line)
    else : 
        return -1
#---------------------- Check_If_Line_Is_Right ----------------------------
#---------- checking for presence of a kewword in the beginning of the line
#---------- aborts if keyword doesn't match 
#--------------------------------------------------
def Check_If_Line_Is_Right(fname, file_data_list, line, keyword) :
  tempstr=file_data_list[line]
  tempwords=string.split(tempstr)
  #----- check if it is right line to modify
  if (not tempwords) or (tempwords[0] != keyword) :
      print '\n\nError!\nLine %s of %s should contain "%s" keyword but instead it is:\n"%s"\n\n\a\a\a' % (
               line, fname, keyword, tempstr[:-1])
      sys.exit()
#--------------------------------------------------------------

#---------------  Check_File_For_Keyword --------------------
#-- finds a keyword and returns its string number
#-- aborts if no keyword found
#--------------------------------
def Check_File_For_Keyword(fname, file_data_list, keyword) :
    linenum=Find_String_Num(file_data_list, keyword)
    if not linenum : 
       print "\n\nError!\n%s keyword is not found in %s!\n\n\a\a\a"  %  (keyword, fname)
       sys.exit()
    else : 
       return linenum


    

 

#--------------------------------------------------------

#---- searches the list for a word containing substring ---
#--
def find_column(header_list, substring):
    'searches the list for a word containing substring -> value from 0,1,2,...'
    count=0
    found=0
    found_index=-1
    for i in header_list:
        if string.find(i,substring) > -1 :
            if found:
                raise UserWarning, "Substring '%(substring)s' was found twice in %(header_list)s!!!" % vars()
            found=1
            found_index=count
        count=count+1
    if not found:
        raise UserWarning, "No substring '%s' found in '%s'!" % (substring, header_list)
    else:
        return found_index




#---- !!!! Very strange function !!!! -----
def count_the_word(list_of_strings, the_word):
    'counts strings where "the_word" appears'
    count=0
    for line in list_of_strings:
        line_list=line.split()
        count+=line_list.count(the_word)
    return count













####################################################################################
####################################################################################
#           RUNNING SHELL SCRIPTS WITH PIPES
####################################################################################
####################################################################################

#----- a function to guard against whitespaces after '\'
#      in Unix-pipe using commandlines
def strip_white_space(command):
    flag=0  # last symbol is not '\'
    safe_command=""
    for x in command:
        # after end of line
        if flag==1 :
            if x==' ' or x=='\t' :
                continue
            elif x=='\n' :
                # OK, end of line
                flag=0
                safe_command=safe_command+x
                continue
            else :
                 raise UserWarning, (
"""\n\nError!\nYour command %s... contains characters after some slash!
which break the Unix pipe!!!"""
                 % string.join(command[:20],""))
        # end of line
        if x=="\\" :
            # watch now!
            flag=1
            safe_command=safe_command+x
            continue
        # ordinary symbol           
        safe_command=safe_command+x
    return safe_command
#-------------------------------------------

    
#----------- run a system command
#   caring not to break Unix-pipes by accidental whitespaces after '\'
def safe_run(command):
    safe_command=strip_white_space(command)
    os.system(safe_command)




####################################################################################
####################################################################################
#            MISCELLANEOUS
####################################################################################
####################################################################################
def beep():
    'beeps once'
    print '\a\a\a',

def error(message):
    'Displays error message and exits'
    print "\n\n\nERROR !!!\n"
    print message
    print '\n\n\n'
    sys.exit

#-- checking argument type --
def isfloat(value):
    try:
            value=float(value)
    except ValueError:
            return 0
    return 1

####################################################################################
####################################################################################
#            STRINGS OPERATIONS
####################################################################################
####################################################################################

#---- utility function
def strip_digits(name):
    'remove digits from the name'
    new_name=""
    for symbol in name:
        if not symbol in "1234567890": new_name+=symbol
    return new_name





####################################################################################
####################################################################################
#            OUTDATED STUFF
####################################################################################
####################################################################################

#---- adding a Unix slash to a path
def add_slash(path):
    'outdated...'
    if path[-1:]!='/': return path+'/'
    return path







if __name__=="__main__":

    list=[1.,2.,3.,4.,5.]

    mean,sigma=statistics(list)
    print mean, sigma


