# -----------------------------------------------------------------------------
# 'integrate peak'
# 
#

import sputil
import tkutil
import tkMessageBox, tkFileDialog
import popen2,string, copy

import mylib, keyfile
import os
from math import *
# -----------------------------------------------------------------------------
#
GNUPLOT_CALL="/usr/local/bin/gnuplot\n"
#-- global definition of dimensions
w1=0
w2=1
#-- positions in init_pars[[]]
FREQUENCY=0
LINEWIDTH=1
INTENSITY=2
OFFSET=3
LABEL=4

#--- gnuplot iterations between reporting the progress: don't set to less than 5! Gets stuck!
max_iterations=5

RETURN_ERROR=0
RETURN_OK=1

class integrate_peak_dialog(tkutil.Dialog):

  fname="" #-- file name for gnuplot to fit --
  init_pars=[] #-- tuples of initial parameters for fitting : ((peak_position_w1,...w2),(linewidth_w1,...w2))
  parfname="" #-- The file with gnuplot parameters (before and after fitting)
  gnuplot=None #-- gnuplot I/O pipes

  
  #--- initial numbers for wanted peak maximum ---
  w1_slice=[]
  w2_slice=[]


  
  current_spectrum=None

  def __init__(self, session):

    self.session = session
    self.drag_notice = None

    self.fit_region=None #--- Region of data to fit --
    self.my_region=None  #--- Any current region --
    self.region_baseline_offset=None
    
    tkutil.Dialog.__init__(self, session.tk, '2D Peak Fitting')

    self.top.columnconfigure(0, weight = 1)

    import Tkinter
    explain = ('Dragging a box defines the region.')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 0, column = 0, sticky = 'nw')


    #--- region line ---
    br = tkutil.button_row(self.top,
                           ('Grab the Region to Fit', self.region_to_fit),
			   )
    br.frame.grid(row = 1, column = 0, sticky = 'nw')


    #--- parameters line ---

    br4 = tkutil.button_row(self.top,
                           ('Take Region for Initial Parameters', self.estimate_parameters),
                           ('Set a Peak Label', self.set_label),
                            ('Save File for Fitting', self.save_parameters),
			   )
    br4.frame.grid(row = 2, column = 0, sticky = 'nw')
    
    #--- gnuplot line ---
    br2 = tkutil.button_row(self.top,
                           ('Fit 1 peak', self.fit_1_peak),
                           ('View 1 peak color maps', self.view_1peak_3D),
                           ('View slices', self.view_slices),
        		   )
    br2.frame.grid(row = 3, column = 0, sticky = 'nw')

    #-- main info ---
    r = Tkinter.Label(self.top, justify = 'left')
    r.grid(row = 4, column = 0, sticky = 'nw')
    self.result_label = r

    #-- service buttons ---
    br3 = tkutil.button_row(self.top,
                           ('Reinitialize', self.reset_fit_parameters),
                           ('Close', self.close_cb),
                           ('Help', self.help),
			   )
    br3.frame.grid(row = 5, column = 0, sticky = 'nw')





    self.register_callback(1)
    self.top.bind('<Map>', lambda e, s=self: s.register_callback(1))
    self.top.bind('<Unmap>', lambda e, s=self: s.register_callback(0))


  # ---------------------------------------------------------------------------
  #
  def register_callback(self, register):

    if register:
      if self.drag_notice == None:
        self.drag_notice = self.session.notify_me('drag region',
                                                  self.region_selected_cb)
    else:
      if self.drag_notice:
        self.session.dont_notify_me(self.drag_notice)
        self.drag_notice = None

  # ---------------------------------------------------------------------------
  #
  def region_selected_cb(self, region):

    if self.is_window_destroyed():
      self.register_callback(0)
      return

    #--ek
    self.my_region=region

    self.current_spectrum = self.session.selected_spectrum()

    self.refresh_info()
    

  #----- REFRESH INFO --------
    
  def refresh_info(self,additional_text=""):
    'Refresh message in the main window'

    if not self.my_region:
       tkMessageBox.showerror("2D Peak Fitting","Drag a box to select region first!")
       return
     
    region=self.my_region
    s=self.current_spectrum
    if s == None:
      self.result_label['text'] = 'No spectrum selected'
      return

    rmin, rmax = region
    ranges = map(lambda vmin, vmax: '%.4g-%.4g' % (vmin, vmax), rmin, rmax)
    import string
    rtext = '[ ' + string.join(ranges, ', ') + ' ]'

        

    paresttxt=""
    if self.init_pars:
      paresttxt="Peak parameter estimates:\nPeak  w1,ppm     (lw1,ppm)    w2,ppm     (lw2,ppm)  Intensity  Baseline_offset \n"
      paresttxt+="-"*50+"\n"
      for set in self.init_pars:
        paresttxt+="%s  %.2f     (%.2f)      %.2f      (%.2f)     %.0f     %.0f\n" % (set[LABEL], set[FREQUENCY][w1],
                                                                                  set[LINEWIDTH][w1],
                                                                                  set[FREQUENCY][w2],
                                                                                  set[LINEWIDTH][w2],
                                                                                  set[INTENSITY],
                                                                                  set[OFFSET])
      paresttxt+="\n"

    
    filenametext=""
    if self.fname: filenametext="File name:   %s\n\n" % self.fname
    
    msg = ("\n%s" % filenametext + 
           'Region      %s\n\n' % rtext  + paresttxt)
    
    self.result_label['text'] = msg + additional_text
    
    
  #-------- menu:  My help function ----------
  def help(self):
    tkMessageBox.showinfo("2D Peak Fitting",
    """Written by Evgenii Kovrigin 05/09/05
(1) Define data region: drag the box around the peak, make sure peak gets selected.
(2) Hit 'Take as Region to Fit'.
(3) Estimate initial peak position and the linewidth: drag a smaller box. Its width and height will be taken as estimate of the line widths.
    Middle of the box is taken for initial peak position.
(4) Hit 'Take for Initial Parameters' to save.
Repeating (3),(4) adds another peak estimate for overlapped fitting. To remove all hit 'Reinitialize'.""")
    

  def reset_fit_parameters(self):
    self.init_pars=[]
    self.fname=""
    self.parfname=""
    self.region_baseline_offset=None
    self.my_region=None
    self.w1_slice=[]
    self.w2_slice=[]
    self.fit_region=None
    
    self.result_label['text'] = "Parameters reset. Pick new."


  # --------------menu: region_to_fit ---------------------------------------------------------------
  #
  def region_to_fit(self):
    'integrate dimension w2'    

    #-- check if there was any assignment selected to form a peak name
    if not self.my_region: 
        tkMessageBox.showwarning("2D Peak Fitting","\n\n!!! Select the region first with the mouse !!!\n\n")
        return

    
    if not  self.session.selected_peaks(): 
        tkMessageBox.showwarning("2D Peak Fitting","\n\n!!! Error!  Switch to selection mode (F1)and have at least one peak in a selected region !!!\n\n")
        return

      
    #-- form a file name
    peaks=""
    for peak in self.session.selected_peaks():
      peaks+=peak.assignment[:-3]+"_"
    peaks=peaks[:-1]+".txt"
    
    #path=os.path.dirname(self.session.project.save_path)
    self.fname=peaks #os.path.join(path,peaks)

    #--- output the region for gnuplot --
    if self.current_spectrum.dimension!=2:
        tkMessageBox.showwarning("2D Peak Fitting","Cannot work with %d dimensions!" % spectrum.dimension)
        return

    self.fit_region=copy.deepcopy(self.my_region)
    self.region_baseline_offset=self.output_region_to_fit(self.current_spectrum,self.my_region,self.fname)
    self.refresh_info()



  # -------------- output_region_to_fit ---------------------------------------------------------------
  #
  def output_region_to_fit(self, spectrum, region, fname):
    'outputs region to fit to  a temporary file'

    ppm_per_index = map(lambda w,s: float(w)/s,
                        spectrum.spectrum_width, spectrum.data_size)


    min_X=region[0][w1]
    min_Y=region[0][w2]

    max_X=region[1][w1]
    max_Y=region[1][w2]

    X_steps=(max_X-min_X)/ppm_per_index[w1]
    Y_steps=(max_Y-min_Y)/ppm_per_index[w2]

    #--- go through the spectrum ---
    data=[]
    outp=open(fname,'w')
    for i in range(X_steps):
      
      curr_X=min_X + ppm_per_index[w1]*i

      outp.write("\n") #-- separator of 'scans' for gnuplot

      for j in range(Y_steps):

          curr_Y=min_Y + ppm_per_index[w2]*j
                     
          height=spectrum.data_height([curr_X,curr_Y])
                     
          data.append([curr_X, curr_Y, height])

          outp.write("%e\t%e\t%e\n" % (curr_X, curr_Y, height))
          
    
    outp.close()
    self.refresh_info() #--- refreshes the main window information


    #--- estimate baseline offset for the region using corners and return it

    baseline_offset=(spectrum.data_height([min_X,min_Y])+
                     spectrum.data_height([max_X,min_Y])+
                     spectrum.data_height([min_X,max_Y])+
                     spectrum.data_height([max_X,max_Y]))/4
                
    return baseline_offset


  # ------- menu: estimate parameters for fitting --------------------------------------------------------
  #
  
  def estimate_parameters(self):
    'estimate linewidth and peak position from the rectangle'

    #-- check if there was any assignment selected to form a peak name
    if not self.my_region: 
        tkMessageBox.showerror("2D Peak Fitting","\n\n!!! Select the region first with the mouse !!!\n\n")
        return

    min_w1=self.my_region[0][w1]
    min_w2=self.my_region[0][w2]

    max_w1=self.my_region[1][w1]
    max_w2=self.my_region[1][w2]


    peak_w1=(max_w1+min_w1)/2
    peak_w2=(max_w2+min_w2)/2


    intensity=self.current_spectrum.data_height([peak_w1,peak_w2])

    
    #--- save for display of slices --
    self.w1_slice.append(peak_w1)
    self.w2_slice.append(peak_w2)


    lw_w1=max_w1-min_w1
    lw_w2=max_w2-min_w2

    set=[[peak_w1,peak_w2],[lw_w1,lw_w2],intensity,self.region_baseline_offset,"No_label"]
    self.init_pars.append(set)

    self.refresh_info() #--- refreshes the main window information 
    

  #----- menu: set peak label ----
  def set_label(self):
    'save selected peak name as a label'
    peaklist=self.session.selected_peaks()
    if len(peaklist)>1:
        tkMessageBox.showwarning("2D Peak Fitting","\n\n Select just one peak for the label!\n\n")
        return

    name=peaklist[0].assignment

    #--- assign to a last parameter set
    if not self.init_pars: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nEstimate parameters first!\n\n")
        return

    self.init_pars[-1][LABEL]=name
    self.refresh_info()


  #----- menu: save parameters ----
    
  def save_parameters(self):
    'create start.par file for fitting'

    if not self.init_pars: 
        tkMessageBox.showerror("2D Peak Fitting","\n\nEstimate parameters first!\n\n")
        return

    #-- make start parameters file ---
    extension='.par'
    self.parfname=self.fname+extension
    startpar_file=open(self.parfname,'w')
    outstring="""
    I_max = %f
    
    w0_x= %f
    L_x=  %f
    
    w0_y= %f
    L_y=  %f

    offset= %f
    """ % (self.init_pars[0][INTENSITY],
           self.init_pars[0][FREQUENCY][w1],self.init_pars[0][LINEWIDTH][w1],
           self.init_pars[0][FREQUENCY][w2],self.init_pars[0][LINEWIDTH][w2],
           self.init_pars[0][OFFSET])
    
    startpar_file.write(outstring)
    startpar_file.close()

    self.refresh_info("Initial parameters saved to '%s'.\n Ready to fit.\n" % self.parfname) #--- refreshes the main window information 


#=================== 1 PEAK-SPECIFIC FUNCTIONS ==========================
  #--- fitting functions definitions ---
  one_lorentzian="""
  
      W1(x)=L_x**2/(L_x*L_x + (w0_x-x)**2) 

      W2(y)=L_y**2/(L_y*L_y + (w0_y-y)**2)

      Y(x,y)=I_max*W1(x)*W2(y) + offset

      """ 
    
  # ---------------------menu: FITTING 1 PEAK --------------------------------------------------------
  #
  def fit_1_peak(self):
    'fitting of a single 2D peak with gnuplot'

    #-- check if only one peak is to fit! --
    if len(self.init_pars)!=1:
      tkMessageBox.showerror("2D Peak Fitting","You defined %d initial parameter sets while \
asking to fit 1 peak! Use 'Reinitialize' and define ONE set of initial parameters." % len(self.init_pars))
      return


    if self.fit(self.one_lorentzian)==RETURN_ERROR: return

    
    #---- read in results of fitting ----
    fitted_pars=keyfile.Ini_File(self.parfname)

    self.init_pars[0][INTENSITY]=float(fitted_pars.get_list("I_max")[1])
    self.init_pars[0][OFFSET]=float(fitted_pars.get_list("offset")[1])
    self.init_pars[0][FREQUENCY][w1]=float(fitted_pars.get_list("w0_x")[1])
    self.init_pars[0][LINEWIDTH][w1]=float(fitted_pars.get_list("L_x")[1])
    self.init_pars[0][FREQUENCY][w2]=float(fitted_pars.get_list("w0_y")[1]) 
    self.init_pars[0][LINEWIDTH][w2]=float(fitted_pars.get_list("L_y")[1])
                  

    self.refresh_info("Gnuplot finished. See Python Shell for details.\n Fit parameters saved in '%s'." % self.parfname) #--- refreshes the main window information 

  # ------------ menu: VIEW 1 PEAK -----------------------------------------------------------------
  def view_1peak_3D(self):
    'color map of one peak'

    if len(self.init_pars)!=1:
      tkMessageBox.showerror("2D Peak Fitting","You defined %d initial parameter sets while \
asking to fit 1 peak! Use 'Reinitialize' and define ONE set of initial parameters." % len(self.init_pars))
      return

    set_parameters="""
    I_max =  %f
    w0_x  =  %f
    L_x   =  %f
    w0_y  =  %f
    L_y   =  %f
    offset=  %f
    """ % (self.init_pars[0][INTENSITY],
    self.init_pars[0][FREQUENCY][w1],
    self.init_pars[0][LINEWIDTH][w1],
    self.init_pars[0][FREQUENCY][w2], 
    self.init_pars[0][LINEWIDTH][w2], 
    self.init_pars[0][OFFSET])
    
    
    function=self.one_lorentzian 
  
    self.view_3D(set_parameters+function)



##  def view_1peak_slice(self):
##    'color map of one peak'
##
##    if len(self.init_pars)!=1:
##      tkMessageBox.showerror("2D Peak Fitting","You defined %d initial parameter sets while \
##asking to fit 1 peak! Use 'Reinitialize' and define ONE set of initial parameters." % len(self.init_pars))
##      return
##
##    set_parameters="""
##    I_max =  %f
##    w0_x  =  %f
##    L_x   =  %f
##    w0_y  =  %f
##    L_y   =  %f
##    offset=  %f
##    """ % (self.init_pars[0][INTENSITY],
##    self.init_pars[0][FREQUENCY][w1],
##    self.init_pars[0][LINEWIDTH][w1],
##    self.init_pars[0][FREQUENCY][w2], 
##    self.init_pars[0][LINEWIDTH][w2], 
##    self.init_pars[0][OFFSET])
##    
##    
##    function=self.one_lorentzian 
##  
##    self.view_slices(set_parameters+function,)
##





#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



  #-------------- menu: VIEW 2D SLICE THROUGH THE PEAK ----------------
  def  view_slices(self):
    'generate slices'

    expdata_fnames=[]
    for i in range(len(self.w1_slice)):
      expdata_fnames.append( self.generate_slices(self.w1_slice[i], self.w2_slice[i],str(i)) )

    print expdata_fnames

    finish="\nexit\n"

    gnuplot_command="""
    set term x11
    set multiplot
    """
    

    #----- w1 dimension ----
    w1_graph="""
      set size 1,0.5
      set origin 0,0 
      set ytics
      set xrange [*:*] reverse
      plot """

    #-- add experimental data --
    w1_set="""'%s' title "%s" with points lt -1 pt %d """
    for i in range(len(expdata_fnames)):
        w1_graph+= (w1_set % (expdata_fnames[i][w1],
                                        self.init_pars[i][LABEL], #-- peak name
                                        4+i #-- nice symbol
                                        ) +
                               ",\\\n")
    w1_graph=w1_graph[:-3]+"\n" #-- remove last slash and a comma




    #----- w2 dimension ----
    w2_graph="""
      set size 1,0.5
      set origin 0,0.5 
      set ytics
      set xrange [*:*] reverse
      plot """ 

    #-- add experimental data --
    w2_set="""'%s' title "%s" with points lt -1 pt %d """
    for i in range(len(expdata_fnames)):
        w2_graph+= (w1_set % (expdata_fnames[i][w2],
                                        self.init_pars[i][LABEL], #-- peak name
                                        4+i #-- nice symbol
                                        ) +
                               ",\\\n")
    w2_graph=w2_graph[:-3]+"\n" #-- remove last slash and a comma



##
##    """
##      set origin 0.48,0.5 
##      unset ytics
##      set colorbox vertical noborder
##      set cbtics 100000
##      splot '%s' using 1:2:(Y($1,$2)) title "Fitting function" """ % self.fname +
##    """
##      set origin 0,0
##      set xtics
##      set ytics
##      splot	'%s' using 1:2:($3-Y($1,$2)) title "Exp - Fit: same scale" """ % self.fname +
##    """
##      set origin 0.48,0
##      set cbrange [*:*]
##      set cbtics 10000
##      unset ytics
##      splot	'%s' using 1:2:($3-Y($1,$2)) title "Exp - Fit: magnified" """ % self.fname +
##    """
##
##    """
##
    finish="""
    unset multiplot
    exit\n
    """

    
    gnuplot_command+=w1_graph + w2_graph
    
    print gnuplot_command
    self.gnuplot=popen2.Popen4(GNUPLOT_CALL,0)
    self.gnuplot.tochild.write(gnuplot_command)
    tkMessageBox.showinfo("2D Peak Fitting","Press OK to continue.")
    self.gnuplot.tochild.write(finish)
    gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
    self.gnuplot=None
    print "--- Gnuplot messages ----"
    for line in gnuplot_output:
        print line
    print "--- End of Gnuplot messages ----"
 

#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

  def   generate_slices(self, peak_w1, peak_w2, extension):
    'display orthogonal slices of experimental data and fitted function through the peak maximum'


    #--- find step size ---
    spectrum=self.current_spectrum
    ppm_per_index = map(lambda w,s: float(w)/s,
                        spectrum.spectrum_width, spectrum.data_size)

    #-- ranges --
    min_w2=self.fit_region[0][w2]
    max_w2=self.fit_region[1][w2]
    min_w1=self.fit_region[0][w1]
    max_w1=self.fit_region[1][w1]


    w1_steps=(max_w1-min_w1)/ppm_per_index[w1]
    w2_steps=(max_w2-min_w2)/ppm_per_index[w2]


          
    #--- along direct dimension first (w2) ---
    fname_w2=self.fname+".w2_"+extension+".txt"
    outp=open(fname_w2,'w')

    for i in range(w2_steps):
      
      w=min_w2 + ppm_per_index[w2]*i

      intensity=self.current_spectrum.data_height([peak_w1,w])
      outp.write("%e\t%e\n" % ( w, intensity))
    outp.close()


    #--- along indirect dimension first (w1) ---
    fname_w1=self.fname+".w1_"+extension+".txt"
    outp=open(fname_w1,'w')

    for i in range(w1_steps):
      
      w=min_w1 + ppm_per_index[w1]*i

      intensity=self.current_spectrum.data_height([w,peak_w2])
      outp.write("%e\t%e\n" % (w, intensity))
    outp.close()

    #-- return a file names
    return fname_w1,fname_w2

#=================== END OF 1 PEAK-SPECIFIC FUNCTIONS ==========================





  #----------------- MAIN FITTING PROCEDURE ------------------------------------
  #
  def fit(self, function):
    'general fitting procedure'

    #--- check if the file was defined ---
    if not self.fname:
      tkMessageBox.showerror("2D Peak Fitting","Region to fit is not defined!")
      return RETURN_ERROR

    if not os.path.isfile(self.parfname):
      tkMessageBox.showerror("2D Peak Fitting","Parameters for fitting were not saved!")
      return  RETURN_ERROR
      

    fit_command="""
      FIT_MAXITER=%d
      set fit errorvariables
      fit Y(x,y) '%s'  using 1:2:3:(1) via '%s'
      update '%s'
      """ % (max_iterations,self.fname,self.parfname,self.parfname)


    finish="\nexit\n"
     
    gnuplot_command=function + fit_command

    #--- see gnuplot output
    #self.session.command_characters("py")
    print gnuplot_command

    iterations=0
    while 1:
      iterations+=max_iterations
      #-- run gnuplot fitting --
      self.gnuplot=popen2.Popen4(GNUPLOT_CALL,0)
      self.gnuplot.tochild.write(gnuplot_command + finish)
      gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
      self.gnuplot=None


      #--- Update iterations count in the window --
      self.refresh_info("Iterations: %d" % iterations)
      if mylib.Find_String_Num(gnuplot_output,'converged.')!=-1:
        break
      
    print "--- Gnuplot messages ----"
    for line in gnuplot_output:
        print line
    print "--- End of Gnuplot messages ----"

    return RETURN_OK
    

   

  #-------- MAIN 3D PLOTTING PROCEDURE ------
  #
  def view_3D(self,function):
    'color maps of results'

    #--- check if the file was defined ---
    if not self.fname:
      tkMessageBox.showerror("2D Peak Fitting","Region to fit is not defined!")
      return


    terminal="""
    set term x11

    """
   
    plotting_3D=("""
      
      set pm3d map
      set cbrange [-200000:200000]

      set multiplot
      set size 0.5,0.5
      set origin 0,0.5 
      unset colorbox
      unset xtics
      set ytics 
      splot '%s' title "Experimental data" """ % self.fname +
    """
      set origin 0.48,0.5 
      unset ytics
      set colorbox vertical noborder
      set cbtics 100000
      splot '%s' using 1:2:(Y($1,$2)) title "Fitting function" """ % self.fname +
    """
      set origin 0,0
      set xtics
      set ytics
      splot	'%s' using 1:2:($3-Y($1,$2)) title "Exp - Fit: same scale" """ % self.fname +
    """
      set origin 0.48,0
      set cbrange [*:*]
      set cbtics 10000
      unset ytics
      splot	'%s' using 1:2:($3-Y($1,$2)) title "Exp - Fit: magnified" """ % self.fname +
    """
      unset multiplot
  """ )
    finish="\nexit\n"

    gnuplot_command=terminal+ function + plotting_3D
    
    print gnuplot_command
    self.gnuplot=popen2.Popen4(GNUPLOT_CALL,0)
    self.gnuplot.tochild.write(gnuplot_command)
    tkMessageBox.showinfo("2D Peak Fitting","Press OK to continue.")
    self.gnuplot.tochild.write(finish)
    gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
    self.gnuplot=None
    print "--- Gnuplot messages ----"
    for line in gnuplot_output:
        print line
    print "--- End of Gnuplot messages ----"
      
      
  

  # ------------ EXIT GNUPLOT -----------------------------------------------------------------
  #
##  def exit_fit(self):
##    'send exit command to gnuplot'
##    if self.gnuplot:
##      self.gnuplot.tochild.write("exit\n")
##      gnuplot_output=self.gnuplot.fromchild.readlines() #-- can only be executed after gnuplot exits
##      self.gnuplot=None
##      print "--- Gnuplot messages ----"
##      for line in gnuplot_output:
##        print line
##      print "--- End of Gnuplot messages ----"














   
##
##
##  # -----------------------------------------------------------------------------
##  #
##  def integrate_all_spectra(self, region, X_axis):
##
##    path=os.path.dirname(self.session.project.save_path)
##    #--- make a name of peaks names ---
##    peaks=""
##    peak_names=""
##    for peak in self.session.selected_peaks():
##      peaks+=peak.assignment[:-3]+"."
##      peak_names+=peak.assignment[:-3]+" "
##    peaks+="x_"+X_axis
##    peaks_path=os.path.join(path,peaks)
##    if not os.path.isdir(peaks_path):   os.mkdir(peaks_path)
##    
##    int_graph=xmgr.Graph(peak_names)
##   
##    #--- run integration
##    y_maxs=[]
##    y_mins=[]
##    
##    for spectrum in self.session.project.spectrum_list():
##      integrated, min_X, max_X, min_Y, max_Y = self.integrate_region(spectrum,region, X_axis)
##      y_maxs.append(max_Y)
##      y_mins.append(min_Y)
##      
##      #--- plain output ----
##      fname=spectrum.name+".txt"
##      file=open(os.path.join(peaks_path,fname),'w')
##      for line in integrated:
##        file.write("%f\t%f\n" % (line[0], line[1]))
##      file.close()
##
##      #-------- make XMGR file to look at -------
##      int_dataset=xmgr.Dataset(integrated,spectrum.name)
##      int_dataset.solid_line()
##      int_dataset.no_sym()
##      int_graph.add_dataset(int_dataset)
##      
##    int_graph.set_xaxis(X_axis,min_X, max_X)
##    int_graph.set_yaxis("Data height",min(y_mins), max(y_maxs))
##    int_graph.writeout(os.path.join(peaks_path,peaks)+".xmgr", legend='off')
##
##
###---------------------------------------------------------------------------------------
##  
##  def integrate_region(self,spectrum, region, X_axis='w1'):
##    '->integrated spectrum along "x" or "y"'
##
##    if spectrum.dimension!=2:
##        tkMessageBox.showwarning("Integrate Region","Cannot work with %d dimensions!" % spectrum.dimension)
##    
##
##    ppm_per_index = map(lambda w,s: float(w)/s,
##                        spectrum.spectrum_width, spectrum.data_size)
###    print ppm_per_index
##
##    dims={ 'w1':0,  # numerical order of dimensions in arrays
##           'w2':1
##           }
##    #--- assign dimensions
##    primary_dimension=dims[X_axis]
##    secondary_dimension= not primary_dimension
##
###    print spectrum.data_height([120,8])
##
##    
##    curr_point=[0,0]
##
##    #--- go along main dimension, integrate secondary
##    
##    integrated=[]
##
##    #--- one index ---    
##    min_X=region[0][primary_dimension]
##    min_Y=region[0][secondary_dimension]
##
##    max_X=region[1][primary_dimension]
##    max_Y=region[1][secondary_dimension]
##    
##    X_steps=(max_X-min_X)/ppm_per_index[primary_dimension]
##    Y_steps=(max_Y-min_Y)/ppm_per_index[secondary_dimension]
##
##    # just for xmgr scaling
##    max_y=spectrum.data_height(curr_point)/Y_steps 
##    min_y=max_y
##
##    for i in range(X_steps):
##      
##      curr_X=min_X + ppm_per_index[primary_dimension]*i
##      curr_point[primary_dimension]=curr_X
##      
##      sum=0
##
##      for j in range(Y_steps):
##
##          curr_Y=min_Y + ppm_per_index[secondary_dimension]*j
##          curr_point[secondary_dimension]=curr_Y
##                     
##          height=spectrum.data_height(curr_point)
##                     
##          sum+=height
##
##      Y_point= sum/Y_steps
##      integrated.append([curr_X,Y_point])
##                     
##      if Y_point>max_y: max_y=Y_point
##      if Y_point<min_y: min_y=Y_point
##
##    #--- ranges for plotting ---
##    min_x=min_X
##    max_x=min_x + ppm_per_index[primary_dimension] * (X_steps-1)
##    
##    
##    return integrated, min_x, max_x, min_y, max_y
##
##
##


#---------------------------------------------------------------------------------------


def run(session):

  sputil.the_dialog(integrate_peak_dialog,session).show_window(1)
