#!/usr/bin/env python2
doc="""
================== Plot relaxation data =================
'plot_data.py'
v1. by Evgenii Kovrigin

Takes a list of data files and plots them on the same xmgr graph
versus residue number.



Usage: plot_data.py [-diff] fname1 fname2 ...
-diff  - plot difference in values with summed error for two first files:
        value=1st-2nd, err=1st+2nd

-reldiff - plot relative-to-1st difference: (1st-2nd)/1st, error
            calculated as (err1+err2)/1st   - 1st is just a scaling
            factor in this equation !

Output: comparison.xmgr

Data should be N x dx
========================================================="""

import os, sys, string,  glob 
import mylib, xmgr, residues

if '-diff' in sys.argv:
    diff_flag="abs_diff"
    sys.argv.remove('-diff')
else:
    if '-reldiff' in sys.argv:
        diff_flag="rel_diff"
        sys.argv.remove('-reldiff')
    else:
        diff_flag="no"

if (len(sys.argv)-1) < 1:
    print doc
    sys.exit()

#-- source data filenames
filenames=[]
count=1
while count < len(sys.argv):
    filenames.append(sys.argv[count])
    count+=1

#--- print out of the command line:
print "--- Loading:"


#--- read data lists --------
res_datasets=[]
for fname in filenames:
    res_datasets.append(residues.Word_Dict(fname))
    
#---- calculate difference -------
if diff_flag!='no':
    diff_dataset=res_datasets[0].make_empty_copy()
    dataset0residues=res_datasets[0].sorted_keys()
    dataset1residues=res_datasets[1].sorted_keys()
    for residue in dataset0residues:
        #-- if there is no same residue in one list - ignore it a
        if residue not in dataset1residues:
            diff_dataset.del_keys([residue])
            continue
        value1=float(res_datasets[0][residue][0])
        value2=float(res_datasets[1][residue][0])
        difference=value1 - value2

        error1=float(res_datasets[0][residue][1])
        error2=float(res_datasets[1][residue][1])
        summed_error= error1 + error2
        
        if  diff_flag=='rel_diff':
            #--- relative difference
            #rel_error1=abs(error2/value1)+abs(error1*value2/(value1*value1))
            diff_dataset[residue]=[difference/value1,summed_error/value1] #-- /value1 - is just a scaling factor!
        else: #-- absolute difference calculated 
            diff_dataset[residue]=[difference,summed_error]

    diff_dataset.set_name("%s - %s" % (res_datasets[0].get_name(),res_datasets[1].get_name()))
    diff_dataset.writeout("%s.dat" % diff_flag)
    
                              
#--- prepare xmgr datasets ---
xmgrdatasets=[]
for dataset in res_datasets:
    xmgrdatasets.append(xmgr.Dataset(
        dataset.to_xmgr(Xcol="Key",Ycol=0,dYcol=1),dataset.name
                                      ))

comp_graph=xmgr.Graph("comparison of raw data")


if diff_flag!='no':
    #--- plot differences of the first two ----
    comp_graph.add_dataset(xmgr.Dataset(diff_dataset.to_xmgr(Xcol="Key",Ycol=0,dYcol=1)))
    comp_graph.set_title("%s - %s" % (res_datasets[0].get_name(),res_datasets[1].get_name()))
    
else: #---- plot datasets separately -----
    #--- assign symbols and load the graph ----
    symbol=2
    fill=0
    for dataset in xmgrdatasets:
        dataset.symbol=symbol
        if fill==0:
            fill=1
        else:
            fill=0
            
        dataset.fill=fill
        symbol+=1

        comp_graph.add_dataset(dataset)

#--- graph settings ----
comp_graph.set_xaxis("Residue number",0, len(res_datasets[0])+1)
comp_graph.set_yaxis("value",0, 1)
comp_graph.set_errorbars("BOTH", 0.1,2)

graphfname="comparison.xmgr"
if diff_flag!='no':
    graphfname=diff_flag+'.'+graphfname
comp_graph.writeout(graphfname)
