doc="""
double exponential decay analysis

-------------------------------------

This extension is based on Tom's 'relax.py'.

"""



import curvefit3_functions
reload(curvefit3_functions)
import sputil
import relax_dialog
import string

class double_exponential_decay_dialog(relax_dialog.relaxation_dialog):
   def __init__(self, session,
                title='Fitting of signal intensities to I=a1*exp(-b1*t)+a2*exp(-b2*t)'):
     relax_dialog.relaxation_dialog.__init__(self, session, title)
     
 # ---------------------------------------------------------------------------
  #
   def fit_function(self, th_pairs, assignment):

    h_values = map(lambda th: th[1], th_pairs)
    h_middle = .25 * (max(h_values) + min(h_values))

    
    params = (h_middle,-1,5*h_middle,-50)
    trials = self.error_estimate_trials
    epsilon=1e-10

    #-- run my Any_Function object now -----
    func = curvefit3_functions.Double_Exponent_Func()
    func.set_data(th_pairs)
    func.set_init_params(params)

    #-- try fitting ---   
    try:
      func.fit_data(trials,epsilon)
    except OverflowError:
      self.session.command_characters('py')
      print "\n\nFitting warning!   Assignment %s didn't fit !!!" % assignment
      func.fit.param_sd=[0]*len(func.fit.params)
      return func.fit

    
    #-- try error estimate ---
    try:
      func.set_same_dy(func.fit.point_sd)
      func.fit.estimate_parameter_error(trials)
    except OverflowError:
      self.session.command_characters('py')
      print "\n\nFitting warning!   Error estimate for %s failed !!!" % assignment
      func.fit.param_sd=[0]*len(func.fit.params)

    print 40*"="
    for line in th_pairs:      print string.join(map(str,line), '  ')
    
    print "Successful fit of %s!" % assignment
    print "Parameters: ",func.fit.params
    print "Point SD: %.2e " % func.fit.point_sd
    print "Parameter SD: ", func.params_sd()
    print "Steps taken:", func.steps_taken()
    print "Fit converged:", func.converged()
    return func.fit

#==========================================================================================
   def peak_line(self, peak, fit):

     htext = ''
     for t, h, dh in fit.xy_pairs:
       htext = htext + '%10.4g ' % h
 
     line = ('%15s   R(I)= %.2f +-%.2f   R(II)= %.2f +-%.2f ' % (peak.assignment[:-3], -fit.params[1], fit.param_sd[1],
                                         -fit.params[3], fit.param_sd[3])  + htext)
     return line
#==========================================================================================  
   def show_heading(self, spectrum_times):

     self.peak_list.clear()
     heading = '%15s %s      %s ' % ('Assignment', '    R,1/s', 'SD')
     for s, t, f in spectrum_times:
#      st = '%s/%.3g' % (s.name, t)
       st = '%g ms' %  t 
       heading = heading + '%10s ' % st
     self.peak_list.heading['text'] = heading
#==========================================================================================
   def show_peaks(self, peaks, spectrum_times):

     single_position = self.single_peak_position.get()
     self.stoppable_loop(' peaks', 1)
     for p in self.session.selected_peaks():
       self.check_for_stop()
       if single_position or not p.is_assigned:
         assignment = None
       else:
	 assignment = p.assignment
       th_pairs = self.time_height_pairs(p.position, assignment, spectrum_times)
       fit = self.fit_function(th_pairs, assignment)
       fit.name = p.assignment
       fit.peak_position = p.position
       fit.spectrum_times = spectrum_times
       line = self.peak_line(p, fit)
       self.peak_list.append(line, fit)
       self.peak_list.listbox.see('end')
       #-- append short list for saving and for viewing
       self.my_fit_list.append(fit)
      
       self.brief_peak_list.add(p.assignment[:-3],[-fit.params[1], fit.param_sd[1],-fit.params[3], fit.param_sd[3]])

  # ---------------------------------------------------------------------------
#==========================================================================================


def run(session):
  sputil.the_dialog(double_exponential_decay_dialog,session).show_peak_heights()
