#!/usr/bin/env python
import string, mylib, glob, os, sys, copy
doc="""

=========== select_frequencies.py ===================
by Evgenii Kovrigin   2/9/08
Medical College of Wisconsin
v.1

This program helps to analyze replicate points within pseudo-3D datasets
generated by 'ek_relax15N'. 

It will report on-screen indices of specific frequencies to enable 
[Select by index] in 'es' Sparky extension.

Usage:   analyze_interleaved_replicates.py path1 
             where 
                path - is path to raw spectral data

============================================================================



"""

if len(sys.argv) < 2:
  print doc
  sys.exit()
  
  

#===== USER INPUT SECTION =====

# file with a list of number of rcCPMG cycles (ncyc)
ncyc_fname='Rex_lists/Rex_RTncyc.ek'  
taucp_fname='Rex_lists/Rex_taucp.ek'


#-- put in path to experiments you wish to analyze
data_path=sys.argv[1]

#===== END OF USER INPUT SECTION =====





print "\nAnalyzing dataset: %s/" % data_path

#-- read ncyc list from specific data folder and convert to integer array
ncyc_list=mylib.Load_Text_File_NonEmpty_Lines (os.path.join(data_path,ncyc_fname))

#-- taucp list
taucp_list=mylib.Load_Text_File_NonEmpty_Lines (os.path.join(data_path,taucp_fname))

#-- go through frequency list, collect indices of experiments to average, by frequency
experiments_to_add={}

#--- collect experiments at specific frequencies in a dictionary
#    as  a list of [expt#, Trelax]
frequency_dict={}

for current_expt in range(len(ncyc_list)):
    
    #-- extract experimental parameters
    ncyc=int(ncyc_list[current_expt])
    taucp=float(taucp_list[current_expt])
     
    #-- use  string representation of frequency for accurate indexing
    if taucp==0:
        CPMGfreq='reference'
    else:
        CPMGfreq= int(1.0/(2*taucp))
        
    #--- total relaxation time
    Trelax= 8*taucp*ncyc

    #-- collect data about current experiment
    if CPMGfreq not in frequency_dict.keys():
        frequency_dict[CPMGfreq]=[[current_expt, Trelax]]
    else:
        frequency_dict[CPMGfreq].append([current_expt, Trelax])
    



#--- REPORT FREQUENCY INDICES and build 'fitting arrays' 
#    containing indices of the planes at the same frequency
#    including reference planes
#fitting_dict={}
#- reference experiment first
ref_exp_indices=[]
planes=frequency_dict['reference']
print "Reference = planes: ",
for expt in planes:
    print '\t', expt[0],
    ref_exp_indices.append(expt[0])
print '\n'


sorted_freqs=frequency_dict.keys()
sorted_freqs.sort()
for current_frequency in sorted_freqs:

 
    #-- get experiment indices
    planes=frequency_dict[current_frequency]
    
    #-- total number of planes
    N=len(planes)

    #-- report frequencies
    if current_frequency=='reference': continue
    
    print '%d Hz = planes:   ' % current_frequency
    
    indices=copy.copy(ref_exp_indices)
    for expt in planes:
        print '\t%d (%.2fms)' %(expt[0], expt[1]*1e3)
        indices.append(expt[0])
    print '\n'
    
    #-- save full list of indices for fitting
#    fitting_dict[current_frequency]=copy.copy(indices)



#print '\n\nPrinting lists for fitting (including indices of reference experiment)\n'

#sorted_freqs=fitting_dict.keys()
#sorted_freqs.sort()
#for freq in sorted_freqs:
#    print '%s Hz:  '%freq,
#    for index in fitting_dict[freq]:
#        print '\t', index,
#    print
    


print "\nDone."

        
        
    








