doc="""
=============== Select Group ==========================
Sparky extension to select a group peaks in the spectrum
using peak list in the text file.

Format: first word in every string is considered to be
        a peak name. 'Assignment' header line is ignored
=======================================================
"""
import textfile
import sys,os
import sparky
import string
import tkFileDialog

#-----------------------------------------------------
def strip_N_H(peak_name):
    'removes sparky assignment suffix (last three letters) from peak name if it is there'
    if peak_name[-2:-1]=='-':
        return peak_name[:-3]
    return peak_name

#-----------------------------------------------------

def run(session):
 
    session.command_characters("py")
    print doc
    fname=tkFileDialog.askopenfilename()
    if not fname: return

    print "------ File: --------"
    requested_peaks_file=textfile.Text_File(fname)
    print "--------- Spectrum: --------\n%s" % session.selected_view().spectrum.name   
    requested_peaks=[]
    print "-------- Peaks read: -----------"
    #-- extract names of peaks from peaklist ---
    for line in requested_peaks_file.file:

        if not line: continue  #-- skip empty line

        words=line.split()
        if len(words)<1: continue #-- skip empty line

        peak_name=words[0]
        if peak_name =='Assignment': continue #-- skip header

        requested_peaks.append(peak_name)
        print "%s, " % peak_name, 
    print "\n---------- %d peaks read ------" % len(requested_peaks)

    #--- check current view peak list versus requested list
    peak_list=session.selected_view().spectrum.peak_list()

    selected=[]
    not_found_peaks=0
    for peak in requested_peaks:
        for spec_peak in peak_list:
            if strip_N_H(peak)==strip_N_H(spec_peak.assignment):
                selected.append(spec_peak)
                break
        else:
            print "Warning! Peak '%s' not found in the spectrum!" % peak
            not_found_peaks+=1

    if not_found_peaks:
        print "---------\n%d peaks from requested list were not found!" % not_found_peaks
    else:
        print "---------- %d peaks selected --------" % len(requested_peaks)
    session.show_peak_list(selected)
  
            
