#!/usr/bin/env python2
doc="""
=================== Separate interleaved data =====================

v.2.1 by Evgenii Kovrigin 12/18/03



This program processes and separates 2D data interleaved by
arraying a parameter which is listed FIRST in the 'array' variable
in 'dg'-window (parameter,phase).

--------------------

First, the 'fid' file is processed by fid.separate_fids.com' macro.
Make sure - settings are correct. 'np' and 'ni' are read and
adjusted automatically.

Then the program looks up in the PROCPAR file for the parameter array
values and separates fids based using nmrPipe function QMIX.
Fids are named by names in the corresponding 'timepoints_table.txt'.

Usage: separate_fids.py  timepoints_table.txt

'timepoints_table.txt':
plain text file of the following structure
parameter_value     new_name
23                  2ms
66                  134ms
23                  2ms_2
...

if the duplicate parameter value is - use differing name like '2ms_2'!

====================================================================
"""
import os, sys, string
import mylib

if len(sys.argv) !=2:
    print doc
    sys.exit()


print """
=================== Separate interleaved data =====================
v.2.1 by Evgenii Kovrigin 12/22/03
"""



#------ reading procpar file -----
procpar=mylib.String_2_Words_2D(mylib.Load_Text_File('procpar'))

#--- find arrayed parameter ------------
array_line=mylib.Find_Keyword_First_in_Line(procpar,'array')
array= string.split(
                string.split(
                            string.strip(procpar[array_line+1][1])
                            ,'"')[1]
                  ,',')
#--- check if it is = parameter,phase
if len(array)<2:
    print "\n\nError!\nAcquisition was done with just one parameter arrayed: ", array
    sys.exit()
    
#-- OK, got it!!!
parameter=array[0]
print "Arrayed parameter: %s  \n" % parameter

#---- find parameter location ------
param_line=mylib.Find_Keyword_First_in_Line(procpar,parameter)

#--- check for impossible error:
if param_line==-1:
    print """
ERROR !!!!
Parameter '%s' is listed in the array variable but not found in anywhere in the 'procpar' file!!!
""" % parameter
    sys.exit()

#--- Load parameter values array 
expts=procpar[param_line+1][1:]
print '"%s" values found: %s\n'% (parameter, expts)
series_length=len(expts)


#---- Processing with var2pipe
np_line=mylib.Find_Keyword_First_in_Line(procpar,'np')
np=string.atoi(procpar[np_line+1][1])
print "np=",np

ni_line=mylib.Find_Keyword_First_in_Line(procpar,'ni')
ni=string.atoi(procpar[ni_line+1][1])
print "ni=",ni
#--- window functions of NMRPipe need this number be increased 
fake_ni=ni*series_length
fid_command="fid.separate_fids.com %s %s %s %s" % (
            np, np/2, fake_ni*2, fake_ni)
print '\nExecuting: ===> "%s"\n' % fid_command, 30*'-'
os.system(fid_command)
print 30*'-'

#--- read table of timepoints 
table=mylib.String_2_Words_2D(mylib.Load_Text_File(sys.argv[1]))
name_list=[]
for line in table[1:]:
    if len(line)==2:
        name_list.append([line[0],line[1]])

#--- check if table corresponds to the procpar file parameter array
for count in range(len(expts)):
    if expts[count]!=name_list[count][0]:
        print "\n\nERROR!\n The list of timepoints in the '%s' file doesn't match '%s' array in the procpar file!\n\
        \nprocpar =>  %s\
        \nsupplied table => %s\n\n" % (
            sys.argv[1],parameter,expts[count], name_list[count][0])

#--- Proceed to separation -----

list_of_used_fnames=[]
for expt_num in range(series_length):
    #-- generate extration list
    extraction_string=""
    count=0
    for count in range(len(expts)):
        if count==expt_num:
            extraction_string+=" 1 0 0 1"
        else:
            extraction_string+=" 0 0 0 0"

    output_dirname=name_list[expt_num][1]
    
    #-- check that the duplicates are correctly named (differently from one another)
    if output_dirname in list_of_used_fnames:
        print "\n\nERROR!\nTimepoint '%s' has a duplicate with the same name!!!\n\
Please, add '_2' or sth to it in the '%s'!" % (name_list[expt_num], sys.argv[1])
        sys.exit()

    #--- store used name ---
    list_of_used_fnames.append(output_dirname)

    print "\n---------------------------\nDirectory: ", output_dirname    
    nmrPipe_command="nmrPipe -in ./all_test.fid | nmrPipe -fn QMIX -ic %d -oc 2 -cList %s -time -ov -out test.fid" % (
                    series_length*2, extraction_string)
    print nmrPipe_command
    os.system(nmrPipe_command)
    if os.path.isdir(output_dirname):
        print "\n\nERROR!\nOutput directory exists! Delete it first, please!"
        sys.exit()
    os.system("mkdir "+output_dirname)
    os.system("mv test.fid %s/" % output_dirname)
    print "%s point is done." % expts[expt_num]
    
os.system('rm all_test.fid')
                        
    
