#!/usr/bin/env python
import os,sys,string
import residues, xmgr
import mylib
import keyfile
import copy
import colors
from colors import *
#-- to make it portable: use color only when requested explicitly
if '-color' in sys.argv: colors.COLOR=ON


TRIM_PERCENT=10


doc= """
******************** Averaging of the dataset with trimming **********

'smart_average.py percent% column_to_evaluate dataset.dat [-color]'

    - trim percent
    - column_to_evaluate - the actual column number (usu 2 if 1st - res name)
    - the data from 'column' are trimmed and their mean and rmsd are calculated
    - residue names may be both NNN and Annn format

by Evgenii Kovrigin    1/12/04
**********************************************************************
"""

if len(sys.argv)<4 :
    print doc
    sys.exit()


data_fname=sys.argv[3]
TRIM_PERCENT=sys.argv[1]
if sys.argv[1][-1]!="%":
    print "Percent value should be given as a first option as N%!"
    sys.exit()
TRIM_PERCENT=float(TRIM_PERCENT[:-1])
VALUE_COLUMN=int(sys.argv[2])-2  # column in the datafile to average

SILENT,VERBOSE=0,1
raw_data=residues.Word_Dict(data_fname,mode=VERBOSE)

#--- recognize whether residue format is NNN or ANNN
if  raw_data.get_keys()[0][:1].isalpha():
    format_flag="Annn"
else:
    format_flag="NNN"

#--- replace NNN with xNNN for common processing further
ini_data=residues.Annn_Num()
for res in raw_data.sorted_keys():
    resname=res
    if format_flag=="NNN":  resname="x"+res
    value=float(raw_data[res][VALUE_COLUMN])
    ini_data.add(resname,[value])

#--- begin processing -------------------------------
full_data=copy.deepcopy(ini_data)
trimmed_data=ini_data.trim_data(0,TRIM_PERCENT)

full_mean,full_sigma=full_data.statistics(column_number=0)
trimmed_mean,trimmed_sigma=trimmed_data.statistics(column_number=0)

print "*"*60
cprint(INV, "Dataset: %s" % data_fname)
print "Full mean: %.4f,  sigma   %.4f" % (full_mean,full_sigma)

trmean="%.4f," % trimmed_mean
trsigma="%.4f\n" % trimmed_sigma
print clr(INV,"Trimmed mean: "), clr(GREEN,trmean),"\tsigma ", clr(RED,trsigma)

open(data_fname+".trimmed",'w').write(
    "Source file: %s\nMean: %.4f,  Sigma   %.4f\n%s" %
    (data_fname,trimmed_mean,trimmed_sigma,trimmed_data))
                      

#------------- presentation -------------

full=xmgr.Dataset(full_data.to_xmgr("ResNum",0), "all")
full.circle()
full.empty_sym()
full.symbol_size(2.2)


trimmed=xmgr.Dataset(trimmed_data.to_xmgr("ResNum",0), "trimmed")
trimmed.circle()
trimmed.filled_sym()
trimmed.symbol_size(1)



graph=xmgr.Graph(data_fname,"Trimmed mean: %.4f,  Sigma   %.4f" %
                 (trimmed_mean,trimmed_sigma) )
graph.add_dataset(full)
graph.add_dataset(trimmed)
graph.set_xaxis("Residue number", 0, 125)
graph.set_yaxis("value", 0.3, 1)
graph.writeout(data_fname+".xmgr")


                      



