#!/usr/bin/env python
#----- constants ----------
doc1="""
********** Reduced spectral density calculations ******************************

output - nanoseconds per radian

"""
doc2="""

Usage:
        spectral_density.py R1 R2 NOE outfname


*******************************************************************************
"""
field=600 #-- MHz

c=3.5298e4
d=-7.204e4
gammaH=26.75
gammaN=-2.712

omegaN=-field*gammaN/gammaH




def reduced_J(R1,R2,NOE):
    '-> [ J(0), J(0.87wN), J(wN)]'


    sNH=R1*(NOE-1)*gammaN/gammaH

    J_087wN= 4*sNH/(5*d*d)


    J_wN=(4*R1-5*sNH)/(3*d*d+4*c*c)


    J_0=(6*R2-3*R1-2.72*sNH)/(3*d*d+4*c*c)
    
    return [J_0, J_087wN, J_wN ]


################################################################

import sys, os
import residues


print doc1

if len(sys.argv)<5 :
    print doc2
    sys.exit()

R1_fname=sys.argv[1]
R2_fname=sys.argv[2]
NOE_fname=sys.argv[3]
out_fname=sys.argv[4]

R1_data=residues.Annn_Num(R1_fname)
R2_data=residues.Annn_Num(R2_fname)
NOE_data=residues.Annn_Num(NOE_fname)

print "----------- Initial data ------------------"
print R1_data
print R2_data
print NOE_data

residues_list=R1_data.get_keys()

J_data=R1_data.make_empty_copy()


for res in residues_list:
    #-- convert to ns per rad    
    J_data[res]=map(lambda x: x*1e9, reduced_J(R1_data[res][0],R2_data[res][0],NOE_data[res][0]))

J_data.set_name("Spectral density values calculated from: %s, %s, %s\n\
Residue\t\tJ(0)\t\tJ(0.87wN)\t\tJ(wN)" %
                (R1_fname, R2_fname, NOE_fname))

print "\n---------------- Calculated data -------------------"
print J_data
J_data.writeout(out_fname, format="\t%.5f")












