#!/usr/bin/env python
doc="""
*************************************************************

Classes to work with text files of arbitrary format

    - special cases are Sparky .save and .proj files

*************************************************************
"""
import mylib, residues
import os, sys, string


#-------------------------------------------------------------------
class Text_File:
    """ Arbitrary format text file

        - file is stored as original strings

        Access functions:
        a=Text_File("fname")
        
        print a.file[1]
        a.file[1]= sth else
        line_number=a.find_all_strings("substring")#-- all containing 'substring'
        line_number=a.find_next_string("previous_substring") #-- next line after first encounter
        line_number=a.find_keyword_first_in_line("keyword") #-- looks for a string
                                                    STARTING with this keyword
        a.writeout(fname)
    """
    #-------------------------------------------------------------------
    def __init__(self, infname, verbose=1):
                 
        self.infname=infname
        self.verbose=verbose
        
        self.file=mylib.open_file(infname, 'r').readlines()

        if verbose: print '"%s" loaded.' % infname
        
    #-------------------------------------------------------------------
    def __repr__(self):
        return string.join(self.file, "")

    #-------------------------------------------------------------------
    def writeout(self, outfname):
        mylib.open_file(outfname, 'w').write(str(self))
        if self.verbose: print '"%s" written.' % outfname

    #-------------------------------------------------------------------
    def find_all_strings(self,substring):
        'find all strings where "substring" is found -> list of string numbers or []'
        found=[]
        for i in range(len(self.file)):
            if self.file[i].find(substring)!=-1: found.append(i)
        return found
    
    #-------------------------------------------------------------------
    def find_next_string(self,start,substring):
        'find next strings where "substring" is found starting from "start" -> string absolute numbers or -1'
        N=len(self.file)
        for i in range(start,N):
            line=self.file[i]
            if line.find(substring)!=-1: return self.file.index(line)
        return -1

    #-------------------- Find_Keyword_First_in_Line ------------
    #----- search for a keyword in  the FIRST postion
    #-----  Returns string number or -1 if not found
    #-------------------------------------------------
    def find_keyword_first_in_line(self, word) :
        "search for a keyword in a string in the FIRST postion"
        for str in self.file :
            str_content=str.split()
            if len(str_content)>0:
              if word==str_content[0] : 
                 return self.file.index(str)
        else : 
            return -1




    #---------------------------------------------------------------------            
##    def read_chunk(self,strnum, strings_to_read):
##        'extracts strings_to_read lines of text starting from strnum'
##        return self.file[strnum,strnum+strings_to_read)




#-------------------------------------------------------------------


class Sparky_Data_File(Text_File):
    'specifics of Sparky ".sp.save" files is taken into account here'

    def __init__(self, inifname, verbose=1):
        Text_File.__init__(self, inifname, verbose)
        self.peaks=residues.Annn_Text()

        
        found=self.find_all_strings("type peak")
        for i in found:

            rs_string=self.find_next_string(i,"rs ")
            res_name=self.file[rs_string].split('|')[1]
            self.peaks.add(res_name,[i])

        if verbose: print "%d peaks found." % len(found)
    
    def get_positions(self):
        'find peak positions in the file-> Annn_Text=res,N,H'
        positions=self.peaks.make_empty_copy("positions")

        for res in self.peaks.get_keys():

            pos=self.find_next_string(self.peaks[res][0],'pos')
            N=float(self.file[pos].split()[1])
            H=float(self.file[pos].split()[2])
            positions[res]=[N,H]

        return positions






########### OTLADKA ##########################################
if __name__=='__main__':
    print "-"*50
    
    a=Sparky_Data_File("test.sp.save")
#    a.writeout("test.sp.save1")
#    print a
    print a.get_positions()
    print "Done..."
