#!/usr/bin/env python
import os,sys,string
import residues, xmgr
import mylib
import keyfile
import copy
import colors
from colors import *
#-- to make it portable: use color only when requested explicitly
if '-color' in sys.argv: colors.COLOR=ON


doc= """
******************** Trimming the dataset by the error value **********

'trim_by_errors.py N% dataset.dat [-color]'

    - N% - trim percent of the total number of residues
    - the data from 'column' are trimmed and their mean and rmsd are calculated
    - residue names may be both NNN and Annn format

by Evgenii Kovrigin    1/12/04
**********************************************************************
""" 

if len(sys.argv)<3 :
    print doc
    sys.exit()


data_fname=sys.argv[2]
VALUE_COLUMN=1  # error column in the residue.Annn_Num object 
trim_percent=sys.argv[1][:-1]

print "Data set: ", data_fname
print "Trimming to '%s'%% by column %s" % (trim_percent, VALUE_COLUMN+2)



SILENT,VERBOSE=0,1
raw_data=residues.Word_Dict(data_fname,mode=VERBOSE)
#--- convert data to floating point



#--- recognize whether residue format is NNN or ANNN
if  raw_data.get_keys()[0][:1].isalpha():
    format_flag="Annn"
else:
    format_flag="NNN"

#--- replace NNN with xNNN for common processing further
#    and fill the Annn_Num object for processing
ini_data=residues.Annn_Num()
for res in raw_data.sorted_keys():
    resname=res
    if format_flag=="NNN":  resname="x"+res
    value_list=map(float,raw_data[res])
    ini_data.add(resname,value_list)

#--- begin processing -------------------------------
trimmed_data=copy.deepcopy(ini_data)
#----------- how many residues to exclude? -----------------
NumRes2Trim=int(len(ini_data)*int(trim_percent)/100.0)
print "Remove %d residues" % NumRes2Trim

print "\n--------- Trim %s %% ------------" % trim_percent
print "\nRemove ",

for i in range(NumRes2Trim):
    res=trimmed_data.max(VALUE_COLUMN)
    
    print  res, trimmed_data[res] , '\n'
    del trimmed_data[res]
    #-- recalculate delta's

ini_mean,ini_sigma=ini_data.statistics(column_number=VALUE_COLUMN)
trimmed_mean,trimmed_sigma=trimmed_data.statistics(column_number=VALUE_COLUMN)

print "*"*60
cprint(INV, "Dataset: %s" % data_fname)
print "Initial mean error: %.4f,  sigma   %.4f" % (ini_mean,ini_sigma)

trmean="%.4f," % trimmed_mean
trsigma="%.4f\n" % trimmed_sigma
print clr(INV,"Trimmed mean error: "), clr(GREEN,trmean),"\tsigma ", clr(RED,trsigma)

open(data_fname+".trimmed",'w').write(
    "Source file: %s\nMean: %.4f,  Sigma   %.4f\n%s" %
    (data_fname,trimmed_mean,trimmed_sigma,trimmed_data))

               
#------------- presentation -------------

full=xmgr.Dataset(ini_data.to_xmgr("ResNum",0,1), "   initial data")
full.circle()
full.filled_sym()
full.symbol_size(1)


trimmed=xmgr.Dataset(trimmed_data.to_xmgr("ResNum",0,1), "   left after trimming by %s%%" % trim_percent)
trimmed.circle()
trimmed.empty_sym()
trimmed.symbol_size(2.2)



graph=xmgr.Graph(data_fname,"Trimmed mean: %.4f,  Sigma   %.4f" %
                 (trimmed_mean,trimmed_sigma) )
graph.add_dataset(full)
graph.add_dataset(trimmed)
graph.set_xaxis("Residue number", 0, 125)
graph.set_yaxis("value", 0.3, 1)
graph.writeout(data_fname+".xmgr")

#-------- preparing original vs trimmed data p


                      



