#!/usr/bin/env python2
doc="""
*********************** Value to molmol v.2 ****************************************

This program makes a macro for molmol with color proportional to the value.
(accepts both Unix/Windows and Mac newline characters)

by Evgenii Kovrigin                                                     07/24/03


----------------------------------------------------------------------------------

Usage:

S2_to_molmol2.py [-c] [-m] -d data_file.txt [-min number -max number]

    where:
            data.txt   - the text file in plain format:
                                                            resnum value
                                                            resnum value
                                                            resnum value
                                            -  empty lines ignored
                                            - 'resnum' without 'value' - ignored
                                            
            -m          if the text file comes from Macintosh (newlines are '\\r')
                        by default - '\\r\\n' - Unix convention
    
            -c         - if you want a colorized output
            
            
            -min number  -max number
                       - if you want to set minimal and maximal S2 value
                                  rather than use automatic scaling


***********************************************************************************
"""
import os, sys, string

import mylib, textfile, residues
from MFselect_constants import *
import molmol
import xmgr
import colors
from colors import *
#-- to make it portable: use color only when requested explicitly
if '-c' in sys.argv: colors.COLOR=ON

#------------------------------------------------------------

def load_plain_text(fname):
    'reads formatted data:  resnum value etc'
    print "Reading text file '%s'" % fname
    file=mylib.Load_Text_File(fname)
    data=residues.Annn_Num()
    for line in file:
        
        words=string.split(line)

        if len(words)>1:
            # Yes there is residue number and value!
            print '%d - %g' % (resnum,value)
            resnum=int(words[0])
            value=float(words[1])
            data.add('X%d'%resnum,[value])
    return data
                

#------------------------------------------------------------

def load_plain_text_from_Mac(fname):
    'reads Mac-formatted data:  resnum value etc'
    print "Reading text file '%s' from Macintosh" % fname
    Mac_file=mylib.Load_Text_File(fname)
    
    data=residues.Annn_Num()
    file=string.split(Mac_file[0],'\r')
    for line in file:
        words=string.split(line)
        
        if len(words)>1:
            # Yes there is residue number and value!
            resnum=int(words[0])
            value=float(words[1])
            print '%d - %g' % (resnum,value)
            data.add('X%d'%resnum,[value])
    return data
                




#---- Common colors ----
White=molmol.Color(1,1,1)
Black=molmol.Color(0,0,0)
LightGrey=molmol.Color(0.95,0.95,0.95)
Grey=molmol.Color(0.75,0.75,0.75)
DarkGrey=molmol.Color(0.3,0.3,0.3)

Red=molmol.Color(1,0,0)
Green=molmol.Color(0,1,0)
Blue=molmol.Color(0,0,1)
Yellow=molmol.Color(1,1,0)
Cyan=molmol.Color(0,1,1)




#############################################################################


#--- check command line for minimal parameters ----
if ('-d' not in sys.argv) or (sys.argv.index('-d')==(len(sys.argv)-1)):
    print doc
    sys.exit()

cprint(YELLOW, "\n---------  Mapping value onto the protein structure -------------\n")
    
#--- input filename ----
fname_index=sys.argv.index('-d')
fname=sys.argv[fname_index+1]
cprint(GREEN,"Input filename: '%s'" % fname)

#--- Clipping level on the value: everything less or more than that is set to min or max values supplied
if '-min' in sys.argv and '-max' in sys.argv:
    ind=sys.argv.index('-min')+1
    S2_min=float(sys.argv[ind])
    ind=sys.argv.index('-max')+1
    S2_max=float(sys.argv[ind])
    scaling="manual"
    cprint(GREEN,"Scaling levels (min, max): %s, %s" % (S2_min, S2_max))
else:
    scaling="auto"
    S2_min=S2_max="not_set"  
    cprint(GREEN,"Scaling - automatic")


#-- read data ---

if '-m' in sys.argv:  datafile=load_plain_text_from_Mac(fname)
else: datafile=load_plain_text(fname)


#-- generate a MolMol sub-macro --

import molmol

protein=molmol.Protein(-1, datafile)

protein.set_assigned_colors(min_color=molmol.Red, max_color=molmol.Yellow)

protein.set_unassigned_color(molmol.Color(0.5,0.5,0.5))

protein.set_background_color(molmol.Black)

protein.colorize_by_parameter(parameter_col_num=0, scaling=scaling, min=S2_min, max=S2_max)

#protein.highlight(["X66","X119"],molmol.Red)


protein.write_macro("molmol.mac")

