#!/usr/bin/env python
import time, popen2, sys, os, string, thread, mylib, keyfile
import colors
from colors import *



doc="""\n***************** Watch Magnet ************************

The utility to monitor the parameters of the spectrometer

v.3.3  by Evgenii Kovrigin         

----------------------------------------------------------

Usage:
    watch_magnet.py  file.ini


- Makes a sequence of 'showstat'  requests with a pause
  between each requests. Finds maximal lock level read, finds temperature etc.
  Number of requests in the sequence and a short pause value
  are set in the .ini file

- Repeates these request sequences in the intervals set in .ini file

- Prints the current experiment number and parameters on screen and
  records it to the log file 'watch_magnet.log'

- watches the lock level and temperature and sends an email to the
  specified address (if email notification is set on). Sends an email also
  when new experiment starts - this is to be indication of normal operation.
  !!! Doesn't work (with mail.yale.edu) if the same machine has an open
  browser window logged in!!!!!!!!!!!!!!!

- maximal allowed temperature and lock changes are set in the .ini file

- email notification may be switched on/off during the run by editing the .ini
  file - the program re-reads the file before every spectrometer parameter
  request (server/account may also be changed)

- Note: STOPPING the program must be done through 'stop' instruction in
        the <command> field of the .ini file. Just change 'go' for 'stop' and
        save the file. The program re-reads every 10 seconds.

        This way SSH is killed and connection is released. If the program crashed
        or forced to exit by Ctrl-Z or 'kill' command - you need to kill SSH child
        process manually to release the connection to the server!

        Correspondingly: before starting the program set 'go' in the <command> field

- Includes colorized output and special attention warning (on screen) for
  lock and temperature changes


1/15/04  -  Correction: if there is VT system malfunction than the current temperature
            is not reported to 'showstat'. In this case the program says "NOT REPORTED"
            in the temperature field
2/17/04  -  Final correction of VT malfunction detection

""" 


#--- utility
def count_lines(word,list):
    'counts lines containing "word"'
    count=0
    for line in list:
        if word in line.split(): count+=1
        
    return count



END_SIGNAL=''   # last element in the list read from a closed stream




#------------ Reader ----------------
def reader(stream,list):
    'continously reads the stream and appends simbols to list'
    char=' '
    while char!=END_SIGNAL:
        char=stream.readline()

        mylock.acquire()
        list.append(char)
        mylock.release()


    #---- add end signal -------
    mylock.acquire()
    list.append(char)
    mylock.release()



#-------- Connection test ------------
def test_connection(inputstreamlist):
    if inputstreamlist[-1]==END_SIGNAL:
        cprintnnl(RED, "\n\n\a\a\aConnection broken!!!\n\n")
        sys.exit()
    


#----------------------------------------------------------
def get_locks(locks_number, pause_seconds, inputstreamlist):
    '-> experiment ID, maximal lock value  value in the sequence'
    locks=[]
    prog.tochild.write("showstat\n")
    while len(locks)<locks_number:
        #--- read ten 'showstat' outputs ------
        
        test_connection(inputstreamlist)
        time.sleep(1)

        #-- search for last line in the 'showstat' output ------
        linenum=mylib.Find_String_Num(inputstreamlist, "Decoupler:")
        if linenum!=-1:
            #print '*'*30
            #print string.join(inputstreamlist,'\n')
            #---- get lock -------
            linenum=mylib.Find_String_Num(inputstreamlist, "level:")
            line=inputstreamlist[linenum]
            words=line.split()
            lock=float(words[-1])
            locks.append(lock)

            #-- get experiment number ---
            linenum=mylib.Find_String_Num(inputstreamlist, "ID:")
            line=inputstreamlist[linenum]
            words=line.split()
            expID=words[-1]


            #-- get temperature ---
            linenum=mylib.Find_String_Num(inputstreamlist, "active:")
            line=inputstreamlist[linenum]
            words=line.split()
            try:
                temp=float(words[-1])
            except ValueError:
                temp="NOT REPORTED!"


            #-- get remaining time ---
            linenum=mylib.Find_String_Num(inputstreamlist, "remaining")
            line=inputstreamlist[linenum]
            words=line.split()
            remaining_time=words[-1]


            #-- get FID element ---
            linenum=mylib.Find_String_Num(inputstreamlist, "element:")
            line=inputstreamlist[linenum]
            words=line.split()
            fid_element=words[-1]


            #--- detect overflows -----
            prog.tochild.write("cat vnmrsys/%s/acqfil/log\n" % expID)
            overflows_number=count_lines("overflow", inputstreamlist)

       

            # reset streamlist and line number
            mylock.acquire()
            num=len(inputstreamlist)
            for i in range(num):
                del inputstreamlist[0]
            inputstreamlist.append(' ')
            linenum=None
            mylock.release()
            
            if len(locks)<locks_number:
                time.sleep(pause_seconds)  # at least, to pause
                prog.tochild.write("showstat\n")

    return expID, max(locks), temp, remaining_time, fid_element, overflows_number
#----------------------------------------------------------


def current_time():
    '-> (weekday, month, date, time, zone,year)'
    ostime=os.popen("date")
    mytime=ostime.readline().split()
    return mytime

from smtplib import *
def email(text):
    'sends an email with the text in the subject line'
    message="Subject: "+text
    if email_notification in ('on', 'ON'):
        try:
            server=SMTP(email_server)
            failed=server.sendmail("watch_magnet@%s" % host, email_account, message)
            server.quit()
        except:
            mylib.open_file("failed_email.log",'a').write("Exceptions raised on sending a message to '%s'\nMessage text: %s\n"
                                                    % (email_server, text))
            return
        
        if failed:
            mylib.open_file("failed_email.log",'a').write("Address failed on sending a message to '%s'\nMessage text: %s\n"
                                        % (email_server, text))
#-------------- the program --------------



#--- read command line -----------

if len(sys.argv)!=2:
    print doc
    sys.exit()


ini_fname=sys.argv[1]
ini_pars=keyfile.Ini_File(ini_fname)

#--- read parameters --------------
print '*'*60
command=ini_pars.get_string("<command>")
if command=='stop':
    print "\a\a\a\n!!! Set the <command> field to 'go' in the '%s'!!!" % ini_fname
    sys.exit()
print "Watch_magnet is started with the following parameters:\n------------------"
host=ini_pars.get_string("<host>")
print "Connection requested to: %s " % host
passwordsleep=ini_pars.get_integer("<login_delay>")


LOCKS_IN_SEQUENCE=ini_pars.get_integer("<requests_num_in_sequence>")
PAUSE_BETWEEN_LOCKS=ini_pars.get_integer("<pause_between_requests>")
PAUSE_BETWEEN_CHECKS=ini_pars.get_integer("<sleep_between_checks>")

print "Status request interval is set to %s minutes" % PAUSE_BETWEEN_CHECKS
print "Request sequence: %d 'showstat' requests with %d sec in between" % (LOCKS_IN_SEQUENCE, PAUSE_BETWEEN_LOCKS)


LOCK_CHANGE_TOLERANCE=ini_pars.get_float("<max_lock_change>") 
TEMP_CHANGE_TOLERANCE=ini_pars.get_float("<max_temp_change>")

print "Lock change tolerance: %.1f" % LOCK_CHANGE_TOLERANCE
print "Temperature change tolerance: %.1f" % TEMP_CHANGE_TOLERANCE

email_notification=""

colors.COLOR=OFF
if ini_pars.get_string("<colorized_output>")=="on":
    colors.COLOR=ON
    cprint(GREEN, "Colorized output - Yes")

#--- start connection ---

print "\n------------------- Connecting to %s Sun Workstation -----------------------" % host
prog=popen2.Popen4("ssh %s" % host,0)
print "%s seconds to enter the password!" % passwordsleep
time.sleep(passwordsleep)

inputstreamlist=['\n']   # input stream
#--- begin reading output --------
thread.start_new(reader, (prog.fromchild,inputstreamlist))
prog.tochild.write("s\n")  # choose 'sun' shell
mylock=thread.allocate_lock()


unknown_form="not reported" # to report unknown answer of the server only once 
current_exp="None"
current_lock=-1.0
current_temp=-1.0
overflows_number=0
overflow_reported=''

while 1:


    #-- check for 'email-notification' mode
    email_notification_change=ini_pars.get_string("<email_notification>")
    if email_notification_change!=email_notification:
        email_notification=email_notification_change
        if email_notification in ('on', 'ON'):
            email_server=ini_pars.get_string("<email_server>")
            email_account=ini_pars.get_string("<email_account>")
            print "E-mail notification is ON to <%s>.\n!!! Be sure not to have any browser windows LOGGED IN !!!" % email_account
            thread.start_new(email,("Email notification ON. ",))
        else:
            print "E-mail notification is OFF"




    #--- Spectrometer parameters request
    try:    
       expID, maxlock, VTtemp, remaining_time, fid_element, new_overflows_number =get_locks(locks_number=LOCKS_IN_SEQUENCE,
                                                     pause_seconds=PAUSE_BETWEEN_LOCKS,
                                                     inputstreamlist=inputstreamlist)

    except:
      
        outstring="The server answer has some unanknown form. Cannot read.\n %s" % string.join(inputstreamlist,'\n')
        #--- to report only 1st unknown answer in a series ---
        if (unknown_form=="not reported"):
            thread.start_new(email,("The server answer has some unknown form. Cannot read.\n",))
            unknown_form="reported"

    else:
        #--- the answer was succesfully read at least once since last unknown answer ----
        unknown_form="not reported"
        
        #-- send e-mail about new experiment ----
        if current_exp!=expID:
            thread.start_new(email,("%s started, lock: %s, T: %s, time: %s"
                                    % (expID,maxlock,VTtemp, remaining_time[:-3]),))
            current_exp=expID
            overflows_number=0
            overflow_reported='' 

        #--- watch lock level change, skip startup ----
        if current_lock < 0: # startup !
            current_lock=maxlock
        elif abs(current_lock-maxlock) > LOCK_CHANGE_TOLERANCE:
            thread.start_new(email,("%s - lock change!  %s to %s"  % (expID,current_lock,maxlock),))
            
            cprintnnl(BLINK,"====>>>>> ")
            cprintnnl(BLINK,"   %s - lock change!  %s to %s   "  % (expID,current_lock,maxlock))
            cprint(BLINK,"  <<<<<==== ")
                   
            current_lock=maxlock

        #--- watch temperature change, skip startup ----
        if VTtemp!="NOT REPORTED!": #--- Check for malfunction of the VT system !!! ---
            
            if current_temp < 0 : # startup of the VT system!
                current_temp=VTtemp
            elif abs(current_temp-VTtemp) > TEMP_CHANGE_TOLERANCE:
                thread.start_new(email,("%s - temp change!  %s to %s"  % (expID,current_temp,VTtemp),))
                
                cprintnnl(BLINK,"====>>>>> ")
                cprintnnl(BLINK,"   %s - temperature change!  %s to %s   "  % (expID,current_temp,VTtemp))
                cprint(BLINK,"  <<<<<==== ")
                
                current_temp=VTtemp

            

        #--- check for overflows ----
        if new_overflows_number>overflows_number:
            #--- NEW OVERFLOWS !!! ----
            overflows_number=new_overflows_number
            overflow_reported='!!!=> OVERFLOW!'
            cprintnnl(BLINK,"====>>>>> ")
            cprintnnl(INV_RED,"   OVERFLOW !!!  ")
            cprint(BLINK,"  <<<<<==== ")
            thread.start_new(email,("%s - OVERFLOW! " % current_exp,))
        else:
            overflow_reported=''
            
        
        #-- make a time record like: 'Wed Oct 23 13:00'
        mytime=string.join(current_time()[:-2],' ')[:-3]
        #-- log file output data ---
        outstring=("%s - %s\tLock: %s\ttemp: %s\ttime left: %s\tFID element: %s   %s\n" %
                   (current_exp, mytime, maxlock, VTtemp, remaining_time[:-3], fid_element, overflow_reported))
    
    mylib.open_file("watch_magnet.log",'a').write(outstring)
    mylib.open_file("%s.log" % current_exp,'a').write(outstring)
    print "%s" % (outstring),



    #-- sleep for specified time checking for STOP request ----
    for i in range(6*PAUSE_BETWEEN_CHECKS):
        
        #-- re-read ini file ---
        ini_pars=keyfile.Ini_File(ini_fname)

        #--- check if stop was requested ---
        command=ini_pars.get_string("<command>")
        if command=='stop':
            print "Stop requested. Killing SSH..."
            os.system("kill %s" % prog.pid)
            sys.exit()

        time.sleep(10)

 

